@@ -34,32 +34,15 @@
 #include <png.h> /* just for the version stuff */
 #endif
 
+#ifdef GNOME
+#undef GTK_DISABLE_DEPRECATED
+/* /usr/include/libgnomeui-2.0/libgnomeui/gnome-entry.h:58: error: expected specifier-qualifier-list before 'GtkCombo' */
+#include <gnome.h>
+#endif
+
 #include <gtk/gtk.h>
 #include <gmodule.h>
 
-#if (defined (HAVE_LIBPOPT) && defined (HAVE_POPT_H)) || defined (GNOME)
-#define HAVE_POPT
-#endif
-
-#ifdef GNOME
-#include <gnome.h>
-#else 
-#  ifdef HAVE_POPT_H
-#    include <popt.h>
-#  else
-/* sorry about the mess, but one should not use conditional defined types in 
- * unconditional function call in the first place ... --hb */
-typedef void* poptContext;
-#  endif
-#endif
-
-/* apparently there is no clean way to use glib-2.6 GOption with gnome */
-#if GLIB_CHECK_VERSION(2,5,5) && !defined GNOME
-#  define USE_GOPTION 1
-#else
-#  define USE_GOPTION 0
-#endif
-
 #ifdef HAVE_FREETYPE
 #include <pango/pangoft2.h>
 #endif
@@ -72,7 +55,7 @@ typedef void* poptContext;
 #include "intl.h"
 #include "app_procs.h"
 #include "object.h"
-#include "color.h"
+#include "commands.h"
 #include "tool.h"
 #include "interface.h"
 #include "modify_tool.h"
@@ -83,74 +66,40 @@ typedef void* poptContext;
 #include "load_save.h"
 #include "preferences.h"
 #include "dia_dirs.h"
-#include "render_eps.h"
 #include "sheet.h"
 #include "plug-ins.h"
 #include "recent_files.h"
 #include "authors.h"
 #include "autosave.h"
 #include "dynamic_refresh.h"
-#include "dia_image.h"
 #include "persistence.h"
 #include "sheets.h"
-#include "utils.h"
-
-#if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
-extern DiaExportFilter png_export_filter;
-#endif
+#include "exit_dialog.h"
+#include "newgroup.h"
+#include "dialib.h"
 
 static gboolean
 handle_initial_diagram(const char *input_file_name, 
 		       const char *export_file_name,
 		       const char *export_file_format,
 		       const char *size,
-		       char *show_layers);
+		       char *show_layers,
+		       const char *outdir);
 
 static void create_user_dirs(void);
 static PluginInitResult internal_plugin_init(PluginInfo *info);
-static void process_opts(int argc, char **argv,
-#if USE_GOPTION
+static void process_opts(int *argc, char **argv,
 			 GOptionContext* context, GOptionEntry options[],
-#elif defined HAVE_POPT
-			 poptContext poptCtx, struct poptOption options[],
-#endif
 			 GSList **files, char **export_file_name,
 			 char **export_file_format, char **size,
-			 char **show_layers, gboolean *nosplash);
+			 char **show_layers, gboolean *nosplash,
+			 char **input_directory, char **output_directory);
 static gboolean handle_all_diagrams(GSList *files, char *export_file_name,
-				    char *export_file_format, char *size, char *show_layers);
+				    char *export_file_format, char *size, char *show_layers,
+				    char *input_directory, char *output_directory);
 static void print_credits(gboolean credits);
 
-static gboolean dia_is_interactive = TRUE;
-static void
-stderr_message_internal(const char *title, const char *fmt,
-                        va_list *args,  va_list *args2);
-
-static void
-stderr_message_internal(const char *title, const char *fmt,
-                        va_list *args,  va_list *args2)
-{
-  static gchar *buf = NULL;
-  static gint   alloc = 0;
-  gint len;
-
-  len = format_string_length_upper_bound (fmt, args);
-
-  if (len >= alloc) {
-    if (buf)
-      g_free (buf);
-    
-    alloc = nearest_pow (MAX(len + 1, 1024));
-    
-    buf = g_new (char, alloc);
-  }
-  
-  vsprintf (buf, fmt, *args2);
-  
-  fprintf(stderr,
-          "%s: %s\n", 
-          title,buf);
-}
+static gboolean dia_is_interactive = FALSE;
 
 #ifdef GNOME
 
@@ -190,19 +139,28 @@ save_state (GnomeClient        *client,
 }
 #endif
 
-char *
-build_output_file_name(const char *infname, const char *format)
+static char *
+build_output_file_name(const char *infname, const char *format, const char *outdir)
 {
-  /* FIXME: probably overly confident... */
-  char *p = strrchr(infname,'.');
+  char *pe = strrchr(infname,'.');
+  char *pp = strrchr(infname,G_DIR_SEPARATOR);
   char *tmp;
-  if (!p) {
-    return g_strconcat(infname,".",format,NULL);
-  }
-  tmp = g_malloc0(strlen(infname)+1+strlen(format)+1);
-  memcpy(tmp,infname,p-infname);
+  if (!pp)
+    pp = infname;
+  else
+    pp += 1;
+  if (!pe)
+    return g_strconcat(outdir ? outdir : "", pp,".",format,NULL);
+
+  tmp = g_malloc0(strlen(pp)+1+strlen(format)+1);
+  memcpy(tmp,pp,pe-pp);
   strcat(tmp,".");
   strcat(tmp,format);
+  if (outdir) {
+    char *ret = g_strconcat(outdir, G_DIR_SEPARATOR_S, tmp, NULL);
+    g_free(tmp);
+    return ret;
+  }
   return tmp;
 }
 
@@ -465,6 +423,9 @@ dump_dependencies(void)
 #endif
 #ifdef HAVE_LIBART
   "libart "
+#endif
+#ifdef HAVE_PANGOCAIRO
+  "pangocairo "
 #endif
   "\n");
 
@@ -472,7 +433,7 @@ dump_dependencies(void)
    * Note: this is not meant to be complete but does only include libaries 
    * which may or have cause(d) us trouble in some versions 
    */
-  g_print ("Library versions\n");
+  g_print ("Library versions (at compile time)\n");
 #ifdef HAVE_LIBPNG
   g_print ("libpng  : %s (%s)\n", png_get_header_ver(NULL), PNG_LIBPNG_VER_STRING);
 #endif
@@ -497,7 +458,7 @@ dump_dependencies(void)
   }
 #endif
   {
-    gchar* libxml_rt_version = "?";
+    const gchar* libxml_rt_version = "?";
 #if 0
     /* this is stupid, does not compile on Linux:
      * app_procs.c:504: error: expected identifier before '(' token
@@ -520,7 +481,18 @@ dump_dependencies(void)
   g_print ("glib    : %d.%d.%d (%d.%d.%d)\n", 
            glib_major_version, glib_minor_version, glib_micro_version,
            GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
-  g_print ("pango   : version not available\n"); /* Pango does not provide such */
+#ifdef PANGO_VERSION_ENCODE
+  g_print ("pango   : %s (%d.%d.%d)\n", pango_version_string(), PANGO_VERSION_MAJOR, PANGO_VERSION_MINOR, PANGO_VERSION_MICRO);
+#else
+  g_print ("pango   : version not available (>= 1.14.x)\n"); /* Pango did not provide such */
+#endif
+#if HAVE_CAIRO
+#  ifdef CAIRO_VERSION_STRING
+  g_print ("cairo   : %s (%s)\n", cairo_version_string(), CAIRO_VERSION_STRING);
+#  else
+  g_print ("cairo   : %s (%d.%d.%d)\n", cairo_version_string(), CAIRO_VERSION_MAJOR, CAIRO_VERSION_MINOR, CAIRO_VERSION_MICRO);
+#  endif
+#endif
 #if 0
   {
     gchar  linkedname[1024];
@@ -564,7 +536,8 @@ handle_initial_diagram(const char *in_file_name,
 		       const char *out_file_name,
 		       const char *export_file_format,
 		       const char *size,
-		       char* show_layers) {
+		       char* show_layers,
+		       const char *outdir) {
   DDisplay *ddisp = NULL;
   Diagram *diagram = NULL;
   gboolean made_conversions = FALSE;
@@ -574,8 +547,7 @@ handle_initial_diagram(const char *in_file_name,
     DiaExportFilter *ef = NULL;
 
     /* First try guessing based on extension */
-    export_file_name = build_output_file_name(in_file_name,
-					      export_file_format);
+    export_file_name = build_output_file_name(in_file_name, export_file_format, outdir);
 
     /* to make the --size hack even uglier but work again for the only filter supporting it */
     if (   size && strcmp(export_file_format, "png") == 0)
@@ -589,8 +561,7 @@ handle_initial_diagram(const char *in_file_name,
 	return FALSE;
       }
       g_free (export_file_name);
-      export_file_name = build_output_file_name(in_file_name,
-						ef->extensions[0]);
+      export_file_name = build_output_file_name(in_file_name, ef->extensions[0], outdir);
     }
     made_conversions |= do_convert(in_file_name,
       (out_file_name != NULL?out_file_name:export_file_name),
@@ -626,22 +597,6 @@ handle_initial_diagram(const char *in_file_name,
   return made_conversions;
 }
 
-#ifdef G_OS_WIN32
-static void
-myXmlErrorReporting (void *ctx, const char* msg, ...)
-{
-  va_list args;
-  gchar *string;
-
-  va_start(args, msg);
-  string = g_strdup_vprintf (msg, args);
-  g_print ("%s", string ? string : "xml error (null)?");
-  va_end(args);
-
-  g_free(string);
-}
-#endif
-
 #ifdef HAVE_FREETYPE
 /* Translators:  This is an option, not to be translated */
 #define EPS_PANGO "eps-pango, "
@@ -661,6 +616,7 @@ app_init (int argc, char **argv)
 {
   static gboolean nosplash = FALSE;
   static gboolean nonew = FALSE;
+  static gboolean use_integrated_ui = FALSE;
   static gboolean credits = FALSE;
   static gboolean version = FALSE;
   static gboolean verbose = FALSE;
@@ -670,6 +626,8 @@ app_init (int argc, char **argv)
 #endif
   static char *export_file_name = NULL;
   static char *export_file_format = NULL;
+  static char *input_directory = NULL;
+  static char *output_directory = NULL;
   static char *size = NULL;
   static char *show_layers = NULL;
   gboolean made_conversions = FALSE;
@@ -691,7 +649,6 @@ app_init (int argc, char **argv)
 		    "shape, svg, tex, " WMF
 		    "wpg");
 
-#if USE_GOPTION 
   GOptionContext *context = NULL;
   static GOptionEntry options[] =
   {
@@ -708,8 +665,14 @@ app_init (int argc, char **argv)
      N_("Don't show the splash screen"), NULL },
     {"nonew", 'n', 0, G_OPTION_ARG_NONE, &nonew,
      N_("Don't create empty diagram"), NULL },
+    {"integrated", '\0', 0, G_OPTION_ARG_NONE, &use_integrated_ui,
+     N_("Start integrated user interface (diagrams in tabs)"), NULL },
     {"log-to-stderr", 'l', 0, G_OPTION_ARG_NONE, &log_to_stderr,
      N_("Send error messages to stderr instead of showing dialogs."), NULL },
+    {"input-directory", 'I', 0, G_OPTION_ARG_STRING, NULL /* &input_directory */,
+     N_("Directory containing input files"), N_("DIRECTORY")},
+    {"output-directory", 'O', 0, G_OPTION_ARG_STRING, NULL /* &output_directory */,
+     N_("Directory containing output files"), N_("DIRECTORY")},
     {"credits", 'c', 0, G_OPTION_ARG_NONE, &credits,
      N_("Display credits list and exit"), NULL },
     {"verbose", 0, 0, G_OPTION_ARG_NONE, &verbose,
@@ -718,49 +681,19 @@ app_init (int argc, char **argv)
      N_("Display version and exit"), NULL },
     { NULL }
   };
-#elif defined HAVE_POPT
-  poptContext context = NULL;
-  struct poptOption options[] =
-  {
-    {"export", 'e', POPT_ARG_STRING, NULL /* &export_file_name */, 0,
-     N_("Export loaded file and exit"), N_("OUTPUT")},
-    {"filter",'t', POPT_ARG_STRING, NULL /* &export_file_format */,
-     0, export_format_string, N_("TYPE")
-    },
-    {"size", 's', POPT_ARG_STRING, NULL, 0,
-     N_("Export graphics size"), N_("WxH")},
-    {"show-layers", 'L', POPT_ARG_STRING, NULL, 0,  /* 13.3.2004 sampo@iki.fi */
-     N_("Show only specified layers (e.g. when exporting). Can be either the layer name or a range of layer numbers (X-Y)"),
-     N_("LAYER,LAYER,...")},
-    {"nosplash", 'n', POPT_ARG_NONE, &nosplash, 0,
-     N_("Don't show the splash screen"), NULL },
-    {"nonew", 'n', POPT_ARG_NONE, &nonew, 0,
-     N_("Don't create empty diagram"), NULL },
-    {"log-to-stderr", 'l', POPT_ARG_NONE, &log_to_stderr, 0,
-     N_("Send error messages to stderr instead of showing dialogs."), NULL },
-    {"credits", 'c', POPT_ARG_NONE, &credits, 0,
-     N_("Display credits list and exit"), NULL },
-    {"verbose", 0, POPT_ARG_NONE, &verbose, 0,
-     N_("Generate verbose output"), NULL },
-    {"version", 'v', POPT_ARG_NONE, &version, 0,
-     N_("Display version and exit"), NULL },
-    {"help", 'h', POPT_ARG_NONE, 0, 1, N_("Show this help message") },
-    {(char *) NULL, '\0', 0, NULL, 0}
-  };
-#endif
+  
+  /* for users of app_init() the default is interactive */
+  dia_is_interactive = TRUE;
 
-#if USE_GOPTION
   options[0].arg_data = &export_file_name;
   options[1].arg_data = &export_file_format;
   options[1].description = export_format_string;
   options[2].arg_data = &size;
   options[3].arg_data = &show_layers;
-#elif defined HAVE_POPT
-  options[0].arg = &export_file_name;
-  options[1].arg = &export_file_format;
-  options[2].arg = &size;
-  options[3].arg = &show_layers;
-#endif
+  g_assert(strcmp (options[8].long_name, "input-directory") == 0);
+  options[8].arg_data = &input_directory;
+  g_assert(strcmp (options[9].long_name, "output-directory") == 0);
+  options[9].arg_data = &output_directory;
 
   argv0 = (argc > 0) ? argv[0] : "(none)";
 
@@ -780,12 +713,11 @@ app_init (int argc, char **argv)
 #endif
   textdomain(GETTEXT_PACKAGE);
 
-  process_opts(argc, argv, 
-#if defined HAVE_POPT || USE_GOPTION
+  process_opts(&argc, argv, 
                context, options, 
-#endif
                &files,
-	     &export_file_name, &export_file_format, &size, &show_layers, &nosplash);
+	       &export_file_name, &export_file_format, &size, &show_layers, &nosplash,
+	       &input_directory, &output_directory);
 
 #if defined ENABLE_NLS && defined HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset(GETTEXT_PACKAGE,"UTF-8");  
@@ -798,10 +730,9 @@ app_init (int argc, char **argv)
       gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
 			  argc, argv,
 			  /* haven't found a quick way to pass GOption here */
-			  GNOME_PARAM_POPT_TABLE, options,
+			  GNOME_PARAM_GOPTION_CONTEXT, context,
 			  GNOME_PROGRAM_STANDARD_PROPERTIES,
 			  GNOME_PARAM_NONE);
-    g_object_get(program, "popt-context", &context, NULL);
     client = gnome_master_client();
     if(client == NULL) {
       g_warning(_("Can't connect to session manager!\n"));
@@ -814,7 +745,7 @@ app_init (int argc, char **argv)
     }
 
     /* This smaller icon is 48x48, standard Gnome size */
-    gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/dia_gnome_icon.png");
+    /* gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/dia_gnome_icon.png");*/
 
 #else
 #  ifdef G_THREADS_ENABLED
@@ -855,49 +786,30 @@ app_init (int argc, char **argv)
 
   if (!dia_is_interactive)
     log_to_stderr = TRUE;
-  
-  if (log_to_stderr)
-    set_message_func(stderr_message_internal);
+
+  libdia_init (   (dia_is_interactive ? DIA_INTERACTIVE : 0)
+	       | (log_to_stderr ? DIA_MESSAGE_STDERR : 0)
+	       | (verbose ? DIA_VERBOSE : 0) );
 
   print_credits(credits);
 
-  LIBXML_TEST_VERSION;
-
-#ifdef G_OS_WIN32
-  xmlSetGenericErrorFunc(NULL, myXmlErrorReporting);
-#endif
-
-  stdprops_init();
-
   if (dia_is_interactive) {
-    dia_image_init();
-
-    gdk_rgb_init();
-
-    gtk_rc_parse("diagtkrc"); 
-
-    if (!nosplash) {
-      app_splash_init("");
-    }
-  }
-
-  if (dia_is_interactive)
     create_user_dirs();
 
-  /* Init cursors: */
-  if (dia_is_interactive) {
-    color_init();
+    if (!nosplash)
+      app_splash_init("");
+
+    /* Init cursors: */
     default_cursor = gdk_cursor_new(GDK_LEFT_PTR);
     ddisplay_set_all_cursor(default_cursor);
   }
 
-  object_registry_init();
-
   dia_register_plugins();
   dia_register_builtin_plugin(internal_plugin_init);
 
   load_all_sheets();     /* new mechanism */
 
+  dia_log_message ("object defaults");
   dia_object_defaults_load (NULL, TRUE /* prefs.object_defaults_create_lazy */);
 
   debug_break();
@@ -920,47 +832,76 @@ app_init (int argc, char **argv)
     /* further initialization *before* reading files */  
     active_tool = create_modify_tool();
 
-    create_toolbox();
-
-    persistence_register_window_create("layer_window",
-				       (NullaryFunc*)&create_layer_dialog);
-
-
+    dia_log_message ("ui creation");
+    if (use_integrated_ui) {
+      create_integrated_ui();
+    } else {
+      create_toolbox();
+      /* for the integrated ui case it is integrated */
+      persistence_register_window_create("layer_window",
+				         (NullaryFunc*)&create_layer_dialog);
+    }
+      
     /*fill recent file menu */
     recent_file_history_init();
 
     /* Set up autosave to check every 5 minutes */
-    gtk_timeout_add(5*60*1000, autosave_check_autosave, NULL);
+#if GLIB_CHECK_VERSION(2,14,0)
+    g_timeout_add_seconds(5*60, autosave_check_autosave, NULL);
+#else
+    g_timeout_add(5*60*1000, autosave_check_autosave, NULL);
+#endif
 
+    /* Create Diagram Tree Window */
     create_tree_window();
 
     persistence_register_window_create("sheets_main_dialog",
 				       (NullaryFunc*)&sheets_dialog_create);
 
-
     /* In current setup, we can't find the autosaved files. */
     /*autosave_restore_documents();*/
 
   }
 
+  dia_log_message ("diagrams");
   made_conversions = handle_all_diagrams(files, export_file_name,
-					 export_file_format, size, show_layers);
+					 export_file_format, size, show_layers,
+					 input_directory, output_directory);
   if (dia_is_interactive && files == NULL && !nonew) {
-    gchar *filename = g_filename_from_utf8(_("Diagram1.dia"), -1, NULL, NULL, NULL);
-    Diagram *diagram = new_diagram (filename);
-    g_free(filename);
+    if (use_integrated_ui)
+    {
+      GList * list;
     
-    if (diagram != NULL) {
-      diagram_update_extents(diagram);
-      diagram->is_default = TRUE;
-      layer_dialog_set_diagram(diagram);
-      new_display(diagram);
+      file_new_callback(NULL);  
+      list = dia_open_diagrams();
+      if (list) 
+      {
+        Diagram * diagram = list->data;
+        diagram_update_extents(diagram);
+        diagram->is_default = TRUE;
+      }
+    }
+    else
+    {
+      gchar *filename = g_filename_from_utf8(_("Diagram1.dia"), -1, NULL, NULL, NULL);
+      Diagram *diagram = new_diagram (filename);
+      g_free(filename);
+    
+      if (diagram != NULL) {
+        diagram_update_extents(diagram);
+        diagram->is_default = TRUE;
+        /* I think this is done in diagram_init() with a call to 
+         * layer_dialog_update_diagram_list() */
+        layer_dialog_set_diagram(diagram);
+        new_display(diagram); 
+      }
     }
   }
   g_slist_free(files);
   if (made_conversions) exit(0);
 
   dynobj_refresh_init();
+  dia_log_message ("initialized");
 }
 
 #if 0
@@ -992,9 +933,75 @@ app_exit(void)
   }
 
   if (diagram_modified_exists()) {
+    if (is_integrated_ui ())
+    {
+      GtkWidget                *dialog;
+      int                       result;
+      exit_dialog_item_array_t *items  = NULL;
+      GList *                   list; 
+      Diagram *                 diagram;
+      
+      dialog = exit_dialog_make (GTK_WINDOW (interface_get_toolbox_shell ()), 
+                                _("Exiting Dia"));
+
+      list = dia_open_diagrams();
+      while (list)
+      {
+        diagram = list->data;
+
+        if (diagram_is_modified (diagram))
+        {
+          const gchar * name = diagram_get_name (diagram);
+          const gchar * path = diagram->filename;
+          exit_dialog_add_item (dialog, name, path, diagram);
+        }
+
+        list = g_list_next (list);
+      }
+
+      result = exit_dialog_run (dialog, &items);
+  
+      gtk_widget_destroy (dialog);
+
+      if (result == EXIT_DIALOG_EXIT_CANCEL)
+      {
+        return FALSE;
+      }
+      else if (result == EXIT_DIALOG_EXIT_SAVE_SELECTED)
+      {
+        int i;
+        for (i = 0 ; i < items->array_size ; i++)
+        {
+          gchar *filename;
+
+          diagram  = items->array[i].data;
+          filename = g_filename_from_utf8 (diagram->filename, -1, NULL, NULL, NULL);
+          diagram_update_extents (diagram);
+          if (!diagram_save (diagram, filename)) {
+            exit_dialog_free_items (items);
+            return FALSE;
+	      }
+          g_free (filename);
+        }
+        exit_dialog_free_items (items);
+      } 
+      else if (result == EXIT_DIALOG_EXIT_NO_SAVE) 
+      {
+        list = dia_open_diagrams();
+        while (list) {
+          diagram = list->data;
+
+	  /* slight hack: don't ask again */
+          diagram_set_modified (diagram, FALSE);
+	  undo_clear(diagram->undo);
+          list = g_list_next (list);
+	}
+      }
+    }
+    else
+    {
     GtkWidget *dialog;
     GtkWidget *button;
-
     dialog = gtk_message_dialog_new(
 	       NULL, GTK_DIALOG_MODAL,
                GTK_MESSAGE_QUESTION,
@@ -1022,6 +1029,7 @@ app_exit(void)
       return FALSE;
     }
     gtk_widget_destroy(dialog);
+    }
   }
   prefs_save();
 
@@ -1044,10 +1052,8 @@ app_exit(void)
       slist = g_slist_next(slist);
 
       gtk_widget_destroy(ddisp->shell);
-
     }
     /* The diagram is freed when the last display is destroyed */
-
   }
   
   /* save pluginrc */
@@ -1055,11 +1061,11 @@ app_exit(void)
     dia_pluginrc_write();
 
   gtk_main_quit();
-#ifndef G_OS_WIN32
+
   /* This printf seems to prevent a race condition with unrefs. */
   /* Yuck.  -Lars */
-  g_print(_("Thank you for using Dia.\n"));
-#endif
+  /* Trying to live without it. -Lars 10/8/07*/
+  /* g_print(_("Thank you for using Dia.\n")); */
   app_exit_once = TRUE;
 
   return TRUE;
@@ -1114,6 +1120,9 @@ internal_plugin_init(PluginInfo *info)
 
   /* register the group object type */
   object_register_type(&group_type);
+#ifdef USE_NEWGROUP
+  object_register_type(&newgroup_type);
+#endif
 
   /* register import filters */
   filter_register_import(&dia_import_filter);
@@ -1121,69 +1130,28 @@ internal_plugin_init(PluginInfo *info)
   /* register export filters */
   /* Standard Dia format */
   filter_register_export(&dia_export_filter);
-  /* EPS with PS fonts */
-  filter_register_export(&eps_export_filter);
-#ifdef HAVE_FREETYPE
-  /* EPS with Pango rendering */
-  filter_register_export(&eps_ft2_export_filter);
-#endif
-#if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
-  /* PNG with libart rendering */
-  filter_register_export(&png_export_filter);
-#endif
 
   return DIA_PLUGIN_INIT_OK;
 }
 
 /* Note: running in locale encoding */
 static void
-process_opts(int argc, char **argv,
-#if USE_GOPTION
+process_opts(int *argc, char **argv,
 	     GOptionContext *context, GOptionEntry options[],
-#elif defined HAVE_POPT
-	     poptContext poptCtx, struct poptOption options[],
-#endif
 	     GSList **files, char **export_file_name,
 	     char **export_file_format, char **size,
-	     char **show_layers, gboolean* nosplash)
+	     char **show_layers, gboolean* nosplash,
+	     char **input_directory, char **output_directory)
 {
-#if defined HAVE_POPT && !USE_GOPTION
-  int rc = 0;
-  poptCtx = poptGetContext(PACKAGE, argc, (const char **)argv, options, 0);
-  poptSetOtherOptionHelp(poptCtx, _("[OPTION...] [FILE...]"));
-  while (rc >= 0) {
-    if((rc = poptGetNextOpt(poptCtx)) < -1) {
-      fprintf(stderr,_("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
-	      poptBadOption(poptCtx, 0),
-	      poptStrerror(rc),
-	      argv[0]);
-      exit(1);
-    }
-    if(rc == 1) {
-      poptPrintHelp(poptCtx, stderr, 0);
-      exit(0);
-    }
-  }
-#endif
   if (argv) {
-#if defined HAVE_POPT && !USE_GOPTION
-      while (poptPeekArg(poptCtx)) {
-          char *in_file_name = (char *)poptGetArg(poptCtx);
-	  if (*in_file_name != '\0')
-	    *files = g_slist_append(*files, in_file_name);
-      }
-      poptFreeContext(poptCtx);
-#elif USE_GOPTION
       GError *error = NULL;
       int i;
       
       context = g_option_context_new(_("[FILE...]"));
       g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-#  if GTK_CHECK_VERSION(2,5,7)
-      /* at least Gentoo was providing GLib-2.6 but Gtk+-2.4.14 */
       g_option_context_add_group (context, gtk_get_option_group (FALSE));
-#  endif
-      if (!g_option_context_parse (context, &argc, &argv, &error)) {
+
+      if (!g_option_context_parse (context, argc, &argv, &error)) {
         if (error) { /* IMO !error here is a bug upstream, triggered with --gdk-debug=updates */
 	g_print ("%s", error->message);
 	  g_error_free (error);
@@ -1194,56 +1162,38 @@ process_opts(int argc, char **argv,
         g_option_context_free(context);
 	exit(0);
       }
-      if (argc < 2) {
+      if (*argc < 2) {
         g_option_context_free(context);
 	return;
       }
-      for (i = 1; i < argc; i++) {
-	if (!g_file_test (argv[i], G_FILE_TEST_IS_REGULAR)) {
+      if (*input_directory && !g_file_test(*input_directory, G_FILE_TEST_IS_DIR)) {
+	g_print (_("Input-directory '%s' must exist!\n"), *input_directory);
+        g_option_context_free(context);
+	exit(2);
+      }
+      if (*output_directory && !g_file_test(*output_directory, G_FILE_TEST_IS_DIR)) {
+	g_print (_("Output-directory '%s' must exist!\n"), *output_directory);
+        g_option_context_free(context);
+	exit(2);
+      }
+      for (i = 1; i < *argc; i++) {
+	if (!(*input_directory) && !g_file_test (argv[i], G_FILE_TEST_IS_REGULAR)) {
 	  g_print (_("'%s' not found!\n"), argv[i]);
           g_option_context_free(context);
 	  exit(2);
+	} else if (*input_directory) {
+	  gchar *testpath = g_build_filename(*input_directory, argv[i], NULL);
+	  gboolean found = g_file_test (testpath, G_FILE_TEST_IS_REGULAR);
+	  g_free(testpath);
+	  if (!found) {
+	    g_print (_("'%s' not found in '%s'!\n"), argv[i], *input_directory);
+            g_option_context_free(context);
+	    exit(2);
+	  }
 	}
 	*files = g_slist_append(*files, argv[i]);
       }
       g_option_context_free(context);
-#else
-      int i;
-
-      for (i=1; i<argc; i++) {
-          char *in_file_name = argv[i]; /* unless it's an option... */
-
-          if (0==strcmp(argv[i],"-t")) {
-              if (i < (argc-1)) {
-                  i++;
-                  *export_file_format = argv[i];
-                  continue;
-              }
-          } else if (0 == strcmp(argv[i],"-e")) {
-              if (i < (argc-1)) {
-                  i++;
-                  *export_file_name = argv[i];
-                  continue;
-              }
-          } else if (0 == strcmp(argv[i],"-s")) {
-              if (i < (argc-1)) {
-                  i++;
-                  *size = argv[i];
-                  continue;
-              }
-          } else if (0 == strcmp(argv[i],"-L")) {
-              if (i < (argc-1)) {
-                  i++;
-                  *show_layers = argv[i];
-                  continue;
-              }
-          } else if (0 == strcmp(argv[i],"-n")) {
-	      *nosplash = 1;
-	      continue;
-          }
-	  *files = g_slist_append(*files, in_file_name);
-      }
-#endif
   }
   if (*export_file_name || *export_file_format || *size)
     dia_is_interactive = FALSE;
@@ -1251,15 +1201,19 @@ process_opts(int argc, char **argv,
 
 static gboolean
 handle_all_diagrams(GSList *files, char *export_file_name,
-		    char *export_file_format, char *size, char *show_layers)
+		    char *export_file_format, char *size, char *show_layers,
+		    char *input_directory, char *output_directory)
 {
   GSList *node = NULL;
   gboolean made_conversions = FALSE;
 
   for (node = files; node; node = node->next) {
+    gchar *inpath = input_directory ? g_build_filename(input_directory, node->data, NULL) : node->data;
     made_conversions |=
-      handle_initial_diagram(node->data, export_file_name,
-			     export_file_format, size, show_layers);
+      handle_initial_diagram(inpath, export_file_name,
+			     export_file_format, size, show_layers, output_directory);
+    if (inpath != node->data)
+      g_free(inpath);
   }
   return made_conversions;
 }
@@ -1305,23 +1259,6 @@ print_credits(gboolean credits)
   }
 }
 
-/* parses a string of the form "[0-9]*x[0-9]*" and transforms it into
-   two long values width and height. */
-void
-parse_size(gchar *size, long *width, long *height)
-{
-  if (size) {
-    gchar** array = g_strsplit(size, "x", 3);
-    *width  = (array[0])? strtol(array[0], NULL, 10): 0;
-    *height = (array[1])? strtol(array[1], NULL, 10): 0;
-    g_strfreev(array);
-  }
-  else {
-    *width  = 0;
-    *height = 0;
-  }
-}
-
 int app_is_embedded(void)
 {
   return 0;
