@@ -27,13 +27,23 @@
 #include <math.h>
 #include <glib.h>
 
+#ifdef GNOME
+#undef GTK_DISABLE_DEPRECATED
+#  include <gnome.h> 	 
+#endif 	 
+
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 
- #ifdef GNOME 	 
- #  include <gnome.h> 	 
- #endif 	 
+#include <textedit.h>
+#include <focus.h>
+#include "confirm.h"
 
+/** Functions called on menu selects.
+ *  Note that GTK (at least up to 2.12) doesn't disable the keyboard shortcuts
+ *  when the menu is made insensitive, so we have to check the constrains again
+ *  in the functions.
+ */
 #ifdef G_OS_WIN32
 /*
  * Instead of polluting the Dia namespace with windoze headers, declare the
@@ -49,10 +59,6 @@ ShellExecuteA (long        hwnd,
                int         nShowCmd);
 #endif
 
-#include "paginate_psprint.h"
-#ifdef G_OS_WIN32
-#  include "paginate_gdiprint.h"
-#endif
 #include "intl.h"
 #include "commands.h"
 #include "app_procs.h"
@@ -60,11 +66,12 @@ ShellExecuteA (long        hwnd,
 #include "display.h"
 #include "object_ops.h"
 #include "cut_n_paste.h"
+#include "interface.h"
 #include "load_save.h"
 #include "utils.h"
 #include "message.h"
 #include "grid.h"
-#include "properties.h"
+#include "properties-dialog.h"
 #include "propinternals.h"
 #include "preferences.h"
 #include "layer_dialog.h"
@@ -99,19 +106,29 @@ file_pagesetup_callback (GtkAction *action)
 }
 
 void
-file_print_callback (GtkAction *action)
+file_print_callback (GtkAction *_action)
 {
   Diagram *dia;
+  DDisplay *ddisp;
+  GtkAction *action;
 
   dia = ddisplay_active_diagram();
   if (!dia) return;
-#ifdef G_OS_WIN32
-  /* This option could be used with Gnome too. Does it make sense there ? */
-  if (!prefs.prefer_psprint)
-    diagram_print_gdi(dia);
-  else
-#endif
-    diagram_print_ps(dia);
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+  
+  action = menus_get_action ("FilePrintGTK");
+  if (!action)
+    action = menus_get_action ("FilePrintGDI");
+  if (!action)
+    action = menus_get_action ("FilePrintPS");
+
+  if (action) {
+    if (confirm_export_size (dia, GTK_WINDOW(ddisp->shell), CONFIRM_PRINT|CONFIRM_PAGES))
+      gtk_action_activate (action);
+  } else {
+    message_error (_("No print plug-in found!"));
+  }
 }
 
 void
@@ -146,139 +163,6 @@ file_preferences_callback (GtkAction *action)
 }
 
 
-void
-edit_copy_callback (GtkAction *action)
-{
-  GList *copy_list;
-  DDisplay *ddisp;
-
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
-  copy_list = parent_list_affected(diagram_get_sorted_selected(ddisp->diagram));
-
-  cnp_store_objects(object_copy_list(copy_list), 1);
-  g_list_free(copy_list);
-
-  ddisplay_do_update_menu_sensitivity(ddisp);
-}
-
-void
-edit_cut_callback (GtkAction *action)
-{
-  GList *cut_list;
-  DDisplay *ddisp;
-  Change *change;
-
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
-
-  diagram_selected_break_external(ddisp->diagram);
-
-  cut_list = parent_list_affected(diagram_get_sorted_selected(ddisp->diagram));
-
-  cnp_store_objects(object_copy_list(cut_list), 0);
-
-  change = undo_delete_objects_children(ddisp->diagram, cut_list);
-  (change->apply)(change, ddisp->diagram);
-  
-  ddisplay_do_update_menu_sensitivity(ddisp);
-  diagram_flush(ddisp->diagram);
-
-  diagram_modified(ddisp->diagram);
-  diagram_update_extents(ddisp->diagram);
-  undo_set_transactionpoint(ddisp->diagram->undo);
-}
-
-void
-edit_paste_callback (GtkAction *action)
-{
-  GList *paste_list;
-  DDisplay *ddisp;
-  Point paste_corner;
-  Point delta;
-  Change *change;
-  int generation = 0;
-  
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
-
-  if (!cnp_exist_stored_objects()) {
-    message_warning(_("No existing object to paste.\n"));
-    return;
-  }
-  
-  paste_list = cnp_get_stored_objects(&generation); /* Gets a copy */
-
-  paste_corner = object_list_corner(paste_list);
-  
-  delta.x = ddisp->visible.left - paste_corner.x;
-  delta.y = ddisp->visible.top - paste_corner.y;
-
-  /* Move down some 10% of the visible area. */
-  delta.x += (ddisp->visible.right - ddisp->visible.left) * 0.1 * generation;
-  delta.y += (ddisp->visible.bottom - ddisp->visible.top) * 0.1 * generation;
-
-  if (generation)
-    object_list_move_delta(paste_list, &delta);
-
-  change = undo_insert_objects(ddisp->diagram, paste_list, 0);
-  (change->apply)(change, ddisp->diagram);
-
-  diagram_modified(ddisp->diagram);
-  undo_set_transactionpoint(ddisp->diagram->undo);
-  
-  diagram_remove_all_selected(ddisp->diagram, TRUE);
-  diagram_select_list(ddisp->diagram, paste_list);
-
-  diagram_update_extents(ddisp->diagram);
-  diagram_flush(ddisp->diagram);
-}
-
-/*
- * ALAN: Paste should probably paste to different position, feels
- * wrong somehow.  ALAN: The offset should increase a little each time
- * if you paste/duplicate several times in a row, because it is
- * clearer what is happening than if you were to piling them all in
- * one place.
- *
- * completely untested, basically it is copy+paste munged together
- */
-void
-edit_duplicate_callback (GtkAction *action)
-{ 
-  GList *duplicate_list;
-  DDisplay *ddisp;
-  Point duplicate_corner;
-  Point delta;
-  Change *change;
-
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
-  duplicate_list = object_copy_list(diagram_get_sorted_selected(ddisp->diagram));
-  duplicate_corner = object_list_corner(duplicate_list);
-  
-  /* Move down some 10% of the visible area. */
-  delta.x = (ddisp->visible.right - ddisp->visible.left)*0.05;
-  delta.y = (ddisp->visible.bottom - ddisp->visible.top)*0.05;
-
-  object_list_move_delta(duplicate_list, &delta);
-
-  change = undo_insert_objects(ddisp->diagram, duplicate_list, 0);
-  (change->apply)(change, ddisp->diagram);
-
-  diagram_modified(ddisp->diagram);
-  undo_set_transactionpoint(ddisp->diagram->undo);
-  
-  diagram_remove_all_selected(ddisp->diagram, TRUE);
-  diagram_select_list(ddisp->diagram, duplicate_list);
-
-  diagram_flush(ddisp->diagram);
-  
-  ddisplay_do_update_menu_sensitivity(ddisp);
-}
-
-
-
 
 /* Signal handler for getting the clipboard contents */
 /* Note that the clipboard is for M$-style cut/copy/paste copying, while
@@ -331,8 +215,8 @@ static void
 received_clipboard_handler(GtkClipboard *clipboard, 
 			   const gchar *text,
 			   gpointer data) {
-  Focus *focus = active_focus();
   DDisplay *ddisp = (DDisplay *)data;
+  Focus *focus = get_active_focus((DiagramData *) ddisp->diagram);
   
   if (text == NULL) return;
 
@@ -362,19 +246,235 @@ make_text_prop_singleton(GPtrArray **props, TextProperty **prop)
 }
 
 
+void
+edit_copy_callback (GtkAction *action)
+{
+  GList *copy_list;
+  DDisplay *ddisp;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+  if (textedit_mode(ddisp)) {
+    Focus *focus = get_active_focus((DiagramData *) ddisp->diagram);
+    DiaObject *obj = focus_get_object(focus);
+    GPtrArray *textprops;
+    TextProperty *prop;
+    
+    if (obj->ops->get_props == NULL) 
+      return;
+    
+    make_text_prop_singleton(&textprops,&prop);
+    /* Get the first text property */
+    obj->ops->get_props(obj, textprops);
+    
+    /* GTK docs claim the selection clipboard is ignored on Win32.
+     * The "clipboard" clipboard is mostly ignored in Unix
+     */
+#ifdef G_OS_WIN32
+    gtk_clipboard_set_text(gtk_clipboard_get(GDK_NONE),
+			   prop->text_data, -1);
+#else
+    gtk_clipboard_set_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY),
+			   prop->text_data, -1);
+#endif
+    prop_list_free(textprops);
+  } else {
+    copy_list = parent_list_affected(diagram_get_sorted_selected(ddisp->diagram));
+    
+    cnp_store_objects(object_copy_list(copy_list), 1);
+    g_list_free(copy_list);
+    
+    ddisplay_do_update_menu_sensitivity(ddisp);
+  }
+}
+
+void
+edit_cut_callback (GtkAction *action)
+{
+  GList *cut_list;
+  DDisplay *ddisp;
+  Change *change;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+
+  if (textedit_mode(ddisp)) {
+  } else {
+    diagram_selected_break_external(ddisp->diagram);
+
+    cut_list = parent_list_affected(diagram_get_sorted_selected(ddisp->diagram));
+
+    cnp_store_objects(object_copy_list(cut_list), 0);
+
+    change = undo_delete_objects_children(ddisp->diagram, cut_list);
+    (change->apply)(change, ddisp->diagram);
+  
+    ddisplay_do_update_menu_sensitivity(ddisp);
+    diagram_flush(ddisp->diagram);
+
+    diagram_modified(ddisp->diagram);
+    diagram_update_extents(ddisp->diagram);
+    undo_set_transactionpoint(ddisp->diagram->undo);
+  }
+}
+
+void
+edit_paste_callback (GtkAction *action)
+{
+  GList *paste_list;
+  DDisplay *ddisp;
+  Point paste_corner;
+  Point delta;
+  Change *change;
+  int generation = 0;
+  
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+  if (textedit_mode(ddisp)) {
+#ifdef G_OS_WIN32
+    gtk_clipboard_request_text(gtk_clipboard_get(GDK_NONE), 
+			       received_clipboard_handler, ddisp);
+#else
+    gtk_clipboard_request_text(gtk_clipboard_get(GDK_SELECTION_PRIMARY), 
+			       received_clipboard_handler, ddisp);
+#endif
+  } else {
+    if (!cnp_exist_stored_objects()) {
+      message_warning(_("No existing object to paste.\n"));
+      return;
+    }
+  
+    paste_list = cnp_get_stored_objects(&generation); /* Gets a copy */
+
+    paste_corner = object_list_corner(paste_list);
+  
+    delta.x = ddisp->visible.left - paste_corner.x;
+    delta.y = ddisp->visible.top - paste_corner.y;
+
+    /* Move down some 10% of the visible area. */
+    delta.x += (ddisp->visible.right - ddisp->visible.left) * 0.1 * generation;
+    delta.y += (ddisp->visible.bottom - ddisp->visible.top) * 0.1 * generation;
+
+    if (generation)
+      object_list_move_delta(paste_list, &delta);
+
+    change = undo_insert_objects(ddisp->diagram, paste_list, 0);
+    (change->apply)(change, ddisp->diagram);
+
+    diagram_modified(ddisp->diagram);
+    undo_set_transactionpoint(ddisp->diagram->undo);
+  
+    diagram_remove_all_selected(ddisp->diagram, TRUE);
+    diagram_select_list(ddisp->diagram, paste_list);
+
+    diagram_update_extents(ddisp->diagram);
+    diagram_flush(ddisp->diagram);
+  }
+}
+
+/*
+ * ALAN: Paste should probably paste to different position, feels
+ * wrong somehow.  ALAN: The offset should increase a little each time
+ * if you paste/duplicate several times in a row, because it is
+ * clearer what is happening than if you were to piling them all in
+ * one place.
+ *
+ * completely untested, basically it is copy+paste munged together
+ */
+void
+edit_duplicate_callback (GtkAction *action)
+{ 
+  GList *duplicate_list;
+  DDisplay *ddisp;
+  Point duplicate_corner;
+  Point delta;
+  Change *change;
+
+  ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+  duplicate_list = object_copy_list(diagram_get_sorted_selected(ddisp->diagram));
+  duplicate_corner = object_list_corner(duplicate_list);
+  
+  /* Move down some 10% of the visible area. */
+  delta.x = (ddisp->visible.right - ddisp->visible.left)*0.05;
+  delta.y = (ddisp->visible.bottom - ddisp->visible.top)*0.05;
+
+  object_list_move_delta(duplicate_list, &delta);
+
+  change = undo_insert_objects(ddisp->diagram, duplicate_list, 0);
+  (change->apply)(change, ddisp->diagram);
+
+  diagram_modified(ddisp->diagram);
+  undo_set_transactionpoint(ddisp->diagram->undo);
+  
+  diagram_remove_all_selected(ddisp->diagram, TRUE);
+  diagram_select_list(ddisp->diagram, duplicate_list);
+
+  diagram_flush(ddisp->diagram);
+  
+  ddisplay_do_update_menu_sensitivity(ddisp);
+}
+
+void
+objects_move_up_layer(GtkAction *action)
+{
+  DDisplay *ddisp = ddisplay_active();
+  GList *selected_list;
+  Change *change;
+
+  if (!ddisp || textedit_mode(ddisp)) return;
+  selected_list = diagram_get_sorted_selected(ddisp->diagram);
+
+  change = undo_move_object_other_layer(ddisp->diagram, selected_list, TRUE);
+  
+  (change->apply)(change, ddisp->diagram);
+
+  diagram_modified(ddisp->diagram);
+  undo_set_transactionpoint(ddisp->diagram->undo);
+  
+  diagram_flush(ddisp->diagram);
+  
+  ddisplay_do_update_menu_sensitivity(ddisp);
+}
+
+ void
+objects_move_down_layer(GtkAction *action)
+{
+  DDisplay *ddisp = ddisplay_active();
+  GList *selected_list;
+  Change *change;
+
+  if (!ddisp || textedit_mode(ddisp)) return;
+  selected_list = diagram_get_sorted_selected(ddisp->diagram);
+
+  /* Must check if move is legal here */
+
+  change = undo_move_object_other_layer(ddisp->diagram, selected_list, FALSE);
+  
+  (change->apply)(change, ddisp->diagram);
+
+  diagram_modified(ddisp->diagram);
+  undo_set_transactionpoint(ddisp->diagram->undo);
+  
+  diagram_flush(ddisp->diagram);
+  
+  ddisplay_do_update_menu_sensitivity(ddisp);
+}
+
 void
 edit_copy_text_callback (GtkAction *action)
 {
-  Focus *focus = active_focus();
-  DDisplay *ddisp;
+  Focus *focus;
+  DDisplay *ddisp = ddisplay_active();
   DiaObject *obj;
   GPtrArray *textprops;
   TextProperty *prop;
 
-  if ((focus == NULL) || (!focus->has_focus)) return;
+  if (ddisp == NULL) return;
 
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
+  focus = get_active_focus((DiagramData *) ddisp->diagram);
+
+  if ((focus == NULL) || (!focus->has_focus)) return;
 
   obj = focus_get_object(focus);
 
@@ -401,7 +501,7 @@ edit_copy_text_callback (GtkAction *action)
 void
 edit_cut_text_callback (GtkAction *action)
 {
-  Focus *focus = active_focus();
+  Focus *focus;
   DDisplay *ddisp;
   DiaObject *obj;
   Text *text;
@@ -409,11 +509,12 @@ edit_cut_text_callback (GtkAction *action)
   TextProperty *prop;
   ObjectChange *change;
 
-  if ((focus == NULL) || (!focus->has_focus)) return;
-
   ddisp = ddisplay_active();
   if (!ddisp) return;
 
+  focus = get_active_focus((DiagramData *) ddisp->diagram);
+  if ((focus == NULL) || (!focus->has_focus)) return;
+
   obj = focus_get_object(focus);
   text = (Text*)focus->user_data;
 
@@ -469,24 +570,30 @@ edit_delete_callback (GtkAction *action)
   GList *delete_list;
   DDisplay *ddisp;
 
-  Change *change;
-
   ddisp = ddisplay_active();
   if (!ddisp) return;
-
-  diagram_selected_break_external(ddisp->diagram);
-
-  delete_list = diagram_get_sorted_selected(ddisp->diagram);
-  change = undo_delete_objects_children(ddisp->diagram, delete_list);
-  (change->apply)(change, ddisp->diagram);
-  g_list_free(delete_list);
-  
+  if (textedit_mode(ddisp)) {
+    ObjectChange *change = NULL;
+    Focus *focus = get_active_focus((DiagramData *) ddisp->diagram);
+    if (!text_delete_key_handler(focus, &change)) {
+      return;
+    }
+    object_add_updates(focus->obj, ddisp->diagram);    
+  } else {
+    Change *change = NULL;
+    diagram_selected_break_external(ddisp->diagram);
+    
+    delete_list = diagram_get_sorted_selected(ddisp->diagram);
+    change = undo_delete_objects_children(ddisp->diagram, delete_list);
+    g_list_free(delete_list);
+    (change->apply)(change, ddisp->diagram);
+  }
   diagram_modified(ddisp->diagram);
   diagram_update_extents(ddisp->diagram);
-
+  
   ddisplay_do_update_menu_sensitivity(ddisp);
   diagram_flush(ddisp->diagram);
-
+  
   undo_set_transactionpoint(ddisp->diagram->undo);
 } 
 
@@ -498,6 +605,7 @@ edit_undo_callback (GtkAction *action)
   dia = ddisplay_active_diagram();
   if (!dia) return;
 
+/* Handle text undo edit here! */
   undo_revert_to_last_tp(dia->undo);
   diagram_modified(dia);
   diagram_update_extents(dia);
@@ -510,6 +618,7 @@ edit_redo_callback (GtkAction *action)
 {
   Diagram *dia;
   
+/* Handle text undo edit here! */
   dia = ddisplay_active_diagram();
   if (!dia) return;
 
@@ -625,7 +734,7 @@ help_about_callback (GtkAction *action)
 	"Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.\n");
 
   gchar *dirname = dia_get_data_directory("");
-  gchar *filename = g_build_filename (dirname, "dia_logo.png", NULL);
+  gchar *filename = g_build_filename (dirname, "dia-splash.png", NULL);
   GdkPixbuf *logo = gdk_pixbuf_new_from_file(filename, NULL);
 
   gtk_about_dialog_set_url_hook (activate_url, NULL, NULL);
@@ -634,8 +743,8 @@ help_about_callback (GtkAction *action)
         "name", "Dia",
 	"version", VERSION,
 	"comments", _("A program for drawing structured diagrams."),
-	"copyright", "(C) 1998-2006 The Free Software Foundation and the authors",
-	"website", "http://www.gnome.org/projects/dia/",
+	"copyright", "(C) 1998-2009 The Free Software Foundation and the authors",
+	"website", "http://live.gnome.org/Dia",
 	"authors", authors,
 	"documenters", documentors,
 	"translator-credits", strcmp (translators, "translator_credits-PLEASE_ADD_YOURSELF_HERE")
@@ -680,28 +789,13 @@ view_zoom_out_callback (GtkAction *action)
   ddisplay_zoom(ddisp, &middle, M_SQRT1_2);
 }
 
-void
+void 
 view_zoom_set_callback (GtkAction *action)
 {
-  DDisplay *ddisp;
-  real scale;
-  Point middle;
-  Rectangle *visible;
   int factor;
-
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
-
   /* HACK the actual factor is a suffix to the action name */
   factor = atoi (gtk_action_get_name (action) + strlen ("ViewZoom"));
-
-  visible = &ddisp->visible;
-  middle.x = visible->left*0.5 + visible->right*0.5;
-  middle.y = visible->top*0.5 + visible->bottom*0.5;
-
-  scale = ((real) factor)/1000.0 * DDISPLAY_NORMAL_ZOOM;
-
-  ddisplay_zoom(ddisp, &middle, scale / ddisp->zoom_factor);  
+  view_zoom_set (factor);
 }
 
 void
@@ -822,28 +916,12 @@ view_toggle_rulers_callback (GtkToggleAction *action)
   if (!ddisp) return;
 
   if (!gtk_toggle_action_get_active (action)) {
-    if (GTK_WIDGET_VISIBLE (ddisp->origin)) {
-      gtk_widget_unmap (ddisp->origin);
-      gtk_widget_unmap (ddisp->hrule);
-      gtk_widget_unmap (ddisp->vrule);
-      
-      GTK_WIDGET_UNSET_FLAGS (ddisp->origin, GTK_VISIBLE);
-      GTK_WIDGET_UNSET_FLAGS (ddisp->hrule, GTK_VISIBLE);
-      GTK_WIDGET_UNSET_FLAGS (ddisp->vrule, GTK_VISIBLE);
-      
-      gtk_widget_queue_resize (GTK_WIDGET (ddisp->origin->parent));
+    if (display_get_rulers_showing(ddisp)) {
+      display_rulers_hide (ddisp);
     }
   } else {
-    if (!GTK_WIDGET_VISIBLE (ddisp->origin)) {
-      GTK_WIDGET_SET_FLAGS (ddisp->origin, GTK_VISIBLE);
-      GTK_WIDGET_SET_FLAGS (ddisp->hrule, GTK_VISIBLE);
-      GTK_WIDGET_SET_FLAGS (ddisp->vrule, GTK_VISIBLE);
-      
-      gtk_widget_map (ddisp->origin);
-      gtk_widget_map (ddisp->hrule);
-      gtk_widget_map (ddisp->vrule);
-      
-      gtk_widget_queue_resize (GTK_WIDGET (ddisp->origin->parent));
+    if (!display_get_rulers_showing(ddisp)) {
+      display_rulers_show (ddisp);
     }
   }
 }
@@ -874,50 +952,11 @@ void
 view_show_all_callback (GtkAction *action)
 {
   DDisplay *ddisp;
-  Diagram *dia;
-  real magnify_x, magnify_y;
-  int width, height;
-  Point middle;
 
   ddisp = ddisplay_active();
   if (!ddisp) return;
-  dia = ddisp->diagram;
-
-  width = dia_renderer_get_width_pixels (ddisp->renderer);
-  height = dia_renderer_get_height_pixels (ddisp->renderer);
-
-  /* if there is something selected show that instead of all exisiting objects */
-  if (dia->data->selected) {
-    GList *list = dia->data->selected;
-    Rectangle extents = ((DiaObject*)list->data)->bounding_box;
-    list = g_list_next(list);
-    while (list) {
-      DiaObject *obj = (DiaObject *)list->data;
-      rectangle_union(&extents, &(obj->bounding_box));
-      list = g_list_next(list);
-    }
-    magnify_x = (real)width / (extents.right - extents.left) / ddisp->zoom_factor;
-    magnify_y = (real)height / (extents.bottom - extents.top) / ddisp->zoom_factor;
-    middle.x = extents.left + (extents.right - extents.left) / 2.0;
-    middle.y = extents.top + (extents.bottom - extents.top) / 2.0;
-  } else {
-    magnify_x = (real)width /
-      (dia->data->extents.right - dia->data->extents.left) / ddisp->zoom_factor;
-    magnify_y = (real)height /
-      (dia->data->extents.bottom - dia->data->extents.top) / ddisp->zoom_factor;
-
-    middle.x = dia->data->extents.left +
-      (dia->data->extents.right - dia->data->extents.left) / 2.0;
-    middle.y = dia->data->extents.top +
-      (dia->data->extents.bottom - dia->data->extents.top) / 2.0;
-  }
-
-  ddisplay_zoom (ddisp, &middle, 
-		 ((magnify_x<magnify_y)?magnify_x:magnify_y)/1.05);
-
-  ddisplay_update_scrollbars(ddisp);
-  ddisplay_add_update_all(ddisp);
-  ddisplay_flush(ddisp);
+  
+  ddisplay_show_all (ddisp);
 }
 
 void
@@ -940,6 +979,66 @@ view_diagram_properties_callback (GtkAction *action)
   diagram_properties_show(ddisp->diagram);
 }
 
+void
+view_main_toolbar_callback (GtkAction *action)
+{
+  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action)) == TRUE)
+  {
+    integrated_ui_main_toolbar_show ();
+  }
+  else
+  {
+    integrated_ui_main_toolbar_hide ();
+  }
+}
+
+void
+view_main_statusbar_callback (GtkAction *action)
+{
+  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action)) == TRUE)
+  {
+    integrated_ui_main_statusbar_show ();
+  }
+  else
+  {
+    integrated_ui_main_statusbar_hide ();
+  }
+}
+
+void
+view_layers_callback (GtkAction *action)
+{
+  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION(action)) == TRUE)
+  {
+    integrated_ui_layer_view_show ();
+  }
+  else
+  {
+    integrated_ui_layer_view_hide ();
+  }
+}
+
+void 
+layers_add_layer_callback (GtkAction *action)
+{
+  Diagram *dia;
+
+  dia = ddisplay_active_diagram();
+  if (!dia) return;
+
+  diagram_edit_layer (dia, NULL);
+}
+
+void 
+layers_rename_layer_callback (GtkAction *action)
+{
+  Diagram *dia;
+
+  dia = ddisplay_active_diagram();
+  if (!dia) return;
+
+  diagram_edit_layer (dia, dia->data->active_layer);
+}
 
 void
 objects_place_over_callback (GtkAction *action)
@@ -962,34 +1061,45 @@ objects_place_up_callback (GtkAction *action)
 void
 objects_place_down_callback (GtkAction *action)
 {
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+
   diagram_place_down_selected(ddisplay_active_diagram());
 }
 
 void
 objects_parent_callback (GtkAction *action)
 {
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+
   diagram_parent_selected(ddisplay_active_diagram());
 }
 
 void
 objects_unparent_callback (GtkAction *action)
 {
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+
   diagram_unparent_selected(ddisplay_active_diagram());
 }
 
 void
 objects_unparent_children_callback (GtkAction *action)
 {
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+
   diagram_unparent_children_selected(ddisplay_active_diagram());
 }
 
 void
 objects_group_callback (GtkAction *action)
 {
-  DDisplay *ddisp;
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
 
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
   diagram_group_selected(ddisplay_active_diagram());
   ddisplay_do_update_menu_sensitivity(ddisp);
 } 
@@ -997,10 +1107,9 @@ objects_group_callback (GtkAction *action)
 void
 objects_ungroup_callback (GtkAction *action)
 {
-  DDisplay *ddisp;
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
 
-  ddisp = ddisplay_active();
-  if (!ddisp) return;
   diagram_ungroup_selected(ddisplay_active_diagram());
   ddisplay_do_update_menu_sensitivity(ddisp);
 } 
@@ -1012,11 +1121,10 @@ dialogs_properties_callback (GtkAction *action)
   DiaObject *selected;
 
   dia = ddisplay_active_diagram();
-  if (!dia) return;
+  if (!dia || textedit_mode(ddisplay_active())) return;
 
   if (dia->data->selected != NULL) {
-    selected = dia->data->selected->data;
-    properties_show(dia, selected);
+    object_list_properties_show(dia, dia->data->selected);
   } else {
     diagram_properties_show(dia);
   } 
@@ -1038,6 +1146,9 @@ objects_align_h_callback (GtkAction *action)
   Diagram *dia;
   GList *objects;
 
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+
   /* HACK align is suffix to action name */
   a = gtk_action_get_name (action) + strlen ("ObjectsAlign");
   if (0 == strcmp ("Left", a)) {
@@ -1082,6 +1193,9 @@ objects_align_v_callback (GtkAction *action)
   Diagram *dia;
   GList *objects;
 
+  DDisplay *ddisp = ddisplay_active();
+  if (!ddisp || textedit_mode(ddisp)) return;
+
   /* HACK align is suffix to action name */
   a = gtk_action_get_name (action) + strlen ("ObjectsAlign");
   if (0 == strcmp ("Top", a)) {
@@ -1118,3 +1232,4 @@ objects_align_v_callback (GtkAction *action)
   undo_set_transactionpoint(dia->undo);
 }
 
+
