@@ -27,6 +27,7 @@
 #include "highlight.h"
 #include "textedit.h"
 #include "parent.h"
+#include "message.h"
 
 static void create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
 				     DDisplay *ddisp);
@@ -46,9 +47,7 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
   Handle *handle1;
   Handle *handle2;
   DiaObject *obj;
-  DiaObject *parent_obj;
   real click_distance;
-  GList *avoid = NULL;
 
   ddisplay_untransform_coords(ddisp,
 			      (int)event->x, (int)event->y,
@@ -64,10 +63,17 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
                                    tool->user_data,
                                    &handle1, &handle2);
 
+  tool->obj = obj; /* ensure that tool->obj is initialised in case we
+		      return early. */
+  if (!obj) {
+    tool->moving = FALSE;
+    tool->handle = NULL;
+    message_error(_("'%s' creation failed"), tool->objtype ? tool->objtype->name : "NULL");
+    return;
+  }
+
   diagram_add_object(ddisp->diagram, obj);
 
-  /* Make sure to not parent to itself (!) */
-  avoid = g_list_prepend(avoid, obj);
   /* Try a connect */
   if (handle1 != NULL &&
       handle1->connect_type != HANDLE_NONCONNECTABLE) {
@@ -76,38 +82,15 @@ create_object_button_press(CreateObjectTool *tool, GdkEventButton *event,
       object_find_connectpoint_display(ddisp, &origpoint, obj, TRUE);
     if (connectionpoint != NULL) {
       (obj->ops->move)(obj, &origpoint);
-      /* Make sure to not parent to the object connected to */
-      avoid = g_list_prepend(avoid, connectionpoint->object);
     }
   }
   
-
-  parent_obj = diagram_find_clicked_object_except(ddisp->diagram, 
-						  &clickedpoint,
-						  click_distance,
-						  avoid);
-
-  g_list_free(avoid);
-
-  /* starting point is within another object */
-  if (parent_obj && object_flags_set(parent_obj, DIA_OBJECT_CAN_PARENT))
-  {
-    Change *change = undo_parenting(ddisp->diagram, parent_obj, obj, TRUE);
-    (change->apply)(change, ddisp->diagram);
-    /*
-    obj->parent = parent_obj;
-    parent_obj->children = g_list_append(parent_obj->children, obj);
-    */
-  }
-
   if (!(event->state & GDK_SHIFT_MASK)) {
     /* Not Multi-select => remove current selection */
     diagram_remove_all_selected(ddisp->diagram, TRUE);
   }
   diagram_select(ddisp->diagram, obj);
 
-  tool->obj = obj;
-
   /* Connect first handle if possible: */
   if ((handle1!= NULL) &&
       (handle1->connect_type != HANDLE_NONCONNECTABLE)) {
@@ -146,6 +129,14 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
 {
   GList *list = NULL;
   DiaObject *obj = tool->obj;
+  gboolean reset;
+
+  GList *parent_candidates;
+
+  g_return_if_fail (obj != NULL);
+  if (!obj) /* not sure if this isn't enough */
+    return; /* could be a legal invariant */
+
   if (tool->moving) {
     gdk_pointer_ungrab (event->time);
 
@@ -156,6 +147,26 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
 
   }
 
+  parent_candidates = 
+    layer_find_objects_containing_rectangle(obj->parent_layer, 
+					    &obj->bounding_box);
+
+  /* whole object must be within another object to parent it */
+  for (; parent_candidates != NULL; parent_candidates = g_list_next(parent_candidates)) {
+    DiaObject *parent_obj = (DiaObject *) parent_candidates->data;
+    if (obj != parent_obj 
+	&& object_within_parent(obj, parent_obj)) {
+      Change *change = undo_parenting(ddisp->diagram, parent_obj, obj, TRUE);
+      (change->apply)(change, ddisp->diagram);
+      break;
+    /*
+    obj->parent = parent_obj;
+    parent_obj->children = g_list_append(parent_obj->children, obj);
+    */
+    }
+  }
+  g_list_free(parent_candidates);
+
   list = g_list_prepend(list, tool->obj);
 
   undo_insert_objects(ddisp->diagram, list, 1); 
@@ -168,20 +179,34 @@ create_object_button_release(CreateObjectTool *tool, GdkEventButton *event,
     }
     tool->moving = FALSE;
     tool->handle = NULL;
-    tool->obj = NULL;
+    tool->obj = NULL;    
+  }
+  
+  {
+    /* remove position from status bar */
+    GtkStatusbar *statusbar = GTK_STATUSBAR (ddisp->modified_status);
+    guint context_id = gtk_statusbar_get_context_id (statusbar, "ObjectPos");
+    gtk_statusbar_pop (statusbar, context_id);
   }
   
   highlight_reset_all(ddisp->diagram);
-  textedit_activate_object(ddisp, obj, NULL);
+  reset = prefs.reset_tools_after_create != tool->invert_persistence;
+  /* kind of backward: first starting editing to see if it is possible at all, than GUI reflection */
+  if (textedit_activate_object(ddisp, obj, NULL) && reset) {
+    gtk_action_activate (menus_get_action ("ToolsTextedit"));
+    reset = FALSE; /* don't switch off textedit below */
+  }
   diagram_update_extents(ddisp->diagram);
   diagram_modified(ddisp->diagram);
 
   undo_set_transactionpoint(ddisp->diagram->undo);
   
-  if (prefs.reset_tools_after_create != tool->invert_persistence)
+  if (reset)
       tool_reset();
   ddisplay_set_all_cursor(default_cursor);
+  ddisplay_do_update_menu_sensitivity(ddisp);
 }
+
 static void
 create_object_motion(CreateObjectTool *tool, GdkEventMotion *event,
 		   DDisplay *ddisp)
