@@ -23,14 +23,14 @@
 
 #include "intl.h"
 #include "diagram.h"
+#include "object.h"
 #include "group.h"
 #include "object_ops.h"
-#include "render_eps.h"
 #include "focus.h"
 #include "message.h"
 #include "menus.h"
 #include "preferences.h"
-#include "properties.h"
+#include "properties-dialog.h"
 #include "cut_n_paste.h"
 #include "layer_dialog.h"
 #include "app_procs.h"
@@ -40,12 +40,34 @@
 #include "diagram_tree_window.h"
 #include "autosave.h"
 #include "dynamic_refresh.h"
+#include "textedit.h"
+#include "lib/diamarshal.h"
+#include "parent.h"
 
 static GList *open_diagrams = NULL;
 
-static void diagram_class_init (DiagramClass *klass);
-static void diagram_init(Diagram *obj, const char *filename);
+struct _ObjectExtent
+{
+  DiaObject *object;
+  Rectangle extent;
+};
 
+typedef struct _ObjectExtent ObjectExtent;
+
+static gint diagram_parent_sort_cb(gconstpointer a, gconstpointer b);
+
+
+static void diagram_class_init (DiagramClass *klass);
+static gboolean diagram_init(Diagram *obj, const char *filename);
+static void diagram_update_for_filename(Diagram *dia);
+
+enum {
+  SELECTION_CHANGED,
+  REMOVED,
+  LAST_SIGNAL
+};
+
+static guint diagram_signals[LAST_SIGNAL] = { 0, };
 static gpointer parent_class = NULL;
 
 GType
@@ -68,7 +90,7 @@ diagram_get_type (void)
 	NULL            /* init */
       };
 
-      object_type = g_type_register_static (G_TYPE_OBJECT,
+      object_type = g_type_register_static (DIA_TYPE_DIAGRAM_DATA,
                                             "Diagram",
                                             &object_info, 0);
     }
@@ -80,22 +102,35 @@ static void
 diagram_finalize(GObject *object) 
 {
   Diagram *dia = DIA_DIAGRAM(object);
-  Diagram *other_diagram;
 
   assert(dia->displays==NULL);
-  g_object_unref(dia->data);
-  dia->data = NULL;
   
-  g_free(dia->filename);
-
   open_diagrams = g_list_remove(open_diagrams, dia);
   layer_dialog_update_diagram_list();
 
-  undo_destroy(dia->undo);
+  if (dia->undo)
+    undo_destroy(dia->undo);
+  dia->undo = NULL;
   
   diagram_tree_remove(diagram_tree(), dia);
 
   diagram_cleanup_autosave(dia);
+
+  if (dia->filename)
+    g_free(dia->filename);
+  dia->filename = NULL;
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+static void
+_diagram_removed (Diagram* dia)
+{
+}
+
+static void
+_diagram_selection_changed (Diagram* dia, int n)
+{
 }
 
 static void
@@ -105,15 +140,29 @@ diagram_class_init (DiagramClass *klass)
 
   parent_class = g_type_class_peek_parent (klass);
 
-  object_class->finalize = diagram_finalize;
-}
+  diagram_signals[REMOVED] =
+    g_signal_new ("removed",
+	          G_TYPE_FROM_CLASS (klass),
+	          G_SIGNAL_RUN_FIRST,
+	          G_STRUCT_OFFSET (DiagramClass, removed),
+	          NULL, NULL,
+	          dia_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 
-/** Creates the raw, uninitialize diagram */
-Diagram *
-diagram_new() {
-  Diagram *dia = g_object_new(DIA_TYPE_DIAGRAM, NULL);
-  g_object_ref(dia);
-  return dia;
+  diagram_signals[SELECTION_CHANGED] =
+    g_signal_new ("selection_changed",
+	          G_TYPE_FROM_CLASS (klass),
+	          G_SIGNAL_RUN_FIRST,
+	          G_STRUCT_OFFSET (DiagramClass, selection_changed),
+	          NULL, NULL,
+	          dia_marshal_VOID__INT,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_INT);
+
+  klass->removed = _diagram_removed;
+  klass->selection_changed = _diagram_selection_changed;
+
+  object_class->finalize = diagram_finalize;
 }
 
 GList *
@@ -123,31 +172,63 @@ dia_open_diagrams(void)
 }
 
 /** Initializes a diagram with standard info and sets it to be called
- * 'filename'.
+ * 'filename'.  
+ * Returns TRUE if everything went ok, FALSE otherwise.
+ * Will return FALSE if filename is not a legal string in the current
+ * encoding.
  */
-static void
+static gboolean
 diagram_init(Diagram *dia, const char *filename)
 {
-  if (dia->data)
-    diagram_data_destroy(dia->data);
+  gchar *newfilename = NULL;
+  GError *error = NULL;
+ 
+  dia->data = &dia->parent_instance; /* compatibility */
+
+  dia->pagebreak_color = prefs.new_diagram.pagebreak_color;
   
-  dia->data = new_diagram_data(&prefs.new_diagram);
-  dia->data->grid.width_x = prefs.grid.x;
-  dia->data->grid.width_y = prefs.grid.y;
-  dia->data->grid.width_w = prefs.grid.w;
-  dia->data->grid.hex = prefs.grid.hex;
-  dia->data->grid.dynamic = prefs.grid.dynamic;
-  dia->data->grid.major_lines = prefs.grid.major_lines;
+  get_paper_info (&dia->data->paper, -1, &prefs.new_diagram);
+
+  dia->grid.width_x = prefs.grid.x;
+  dia->grid.width_y = prefs.grid.y;
+  dia->grid.width_w = prefs.grid.w;
+  dia->grid.hex_size = 1.0;
+  dia->grid.colour = prefs.new_diagram.grid_color;
+  dia->grid.hex = prefs.grid.hex;
+  dia->grid.visible_x = 1;
+  dia->grid.visible_y = 1;
+  dia->grid.dynamic = prefs.grid.dynamic;
+  dia->grid.major_lines = prefs.grid.major_lines;
+
+  dia->guides.nhguides = 0;
+  dia->guides.hguides = NULL;
+  dia->guides.nvguides = 0;
+  dia->guides.vguides = NULL;
 
   if (dia->filename != NULL)
     g_free(dia->filename);
+  /* Make sure the filename is absolute */
+  if (!g_path_is_absolute(filename)) {
+    gchar *pwd = g_get_current_dir();
+
+    newfilename = g_build_filename(pwd, filename, NULL);
+    g_free(pwd);
+    filename = newfilename;
+  }
   /* All Diagram functions assumes filename in filesystem encoding */
-  dia->filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
   
+  dia->filename = g_filename_to_utf8(filename, -1, NULL, NULL, &error);
+  if (error != NULL) {
+    message_error(_("Couldn't convert filename '%s' to UTF-8: %s\n"),
+		  dia_message_filename(filename), error->message);
+    g_error_free(error);
+    dia->filename = g_strdup(_("Error"));
+    return FALSE;
+  }
+
   dia->unsaved = TRUE;
   dia->mollified = FALSE;
   dia->autosavefilename = NULL;
-  dia->related_dialogs = NULL;
 
   if (dia->undo)
     undo_destroy(dia->undo);
@@ -158,6 +239,9 @@ diagram_init(Diagram *dia, const char *filename)
 
   if (app_is_interactive())
     layer_dialog_update_diagram_list();
+
+  g_free(newfilename);
+  return TRUE;
 }
 
 int
@@ -165,19 +249,41 @@ diagram_load_into(Diagram         *diagram,
 		  const char      *filename,
 		  DiaImportFilter *ifilter)
 {
+  gboolean was_default = diagram->is_default;
   if (!ifilter)
     ifilter = filter_guess_import_filter(filename);
   if (!ifilter)  /* default to native format */
     ifilter = &dia_import_filter;
 
-  diagram_init(diagram, filename);
-
   if (ifilter->import_func(filename, diagram->data, ifilter->user_data)) {
-    diagram->unsaved = FALSE;
-    diagram_set_modified(diagram, FALSE);
-    if (app_is_interactive())
-      recent_file_history_add(filename);
-    diagram_tree_add(diagram_tree(), diagram);
+    if (ifilter != &dia_import_filter) {
+      /* When loading non-Dia files, change filename to reflect that saving
+       * will produce a Dia file. See bug #440093 */
+      if (strcmp (diagram->filename, filename) == 0) {
+	/* not a real load into but initial load */
+	gchar *old_filename = g_strdup (diagram->filename);
+        gchar *suffix_offset = g_utf8_strrchr(old_filename, -1, (gunichar)'.');
+        gchar *new_filename;
+        if (suffix_offset != NULL) {
+	  new_filename = g_strndup(old_filename, suffix_offset - old_filename);
+	  g_free(old_filename);
+	} else {
+	  new_filename = old_filename;
+	}
+        old_filename = g_strconcat(new_filename, ".dia", NULL);
+        g_free(new_filename);
+        diagram_set_filename(diagram, old_filename);
+        g_free(old_filename);
+        diagram->unsaved = TRUE;
+	diagram_update_for_filename (diagram);
+	diagram_modified(diagram);
+      }
+    } else {
+      /* the initial diagram should have confirmed filename  */
+      diagram->unsaved = 
+	  strcmp(filename, diagram->filename) == 0 ? FALSE : was_default;
+    }
+    diagram_set_modified(diagram, TRUE);
     return TRUE;
   } else
     return FALSE;
@@ -186,32 +292,70 @@ diagram_load_into(Diagram         *diagram,
 Diagram *
 diagram_load(const char *filename, DiaImportFilter *ifilter)
 {
-  Diagram *diagram;
+  Diagram *diagram = NULL;
+  GList *diagrams;
+  gboolean was_default = FALSE;
 
-  diagram = new_diagram(filename);
-
-  if (!diagram_load_into (diagram, filename, ifilter)) {
-    diagram_destroy(diagram);
-    diagram = NULL;
+  for (diagrams = open_diagrams; diagrams != NULL; diagrams = g_list_next(diagrams)) {
+    Diagram *old_diagram = (Diagram*)diagrams->data;
+    if (old_diagram->is_default) {
+      diagram = old_diagram;
+      was_default = TRUE;
+      break;
+    }	
   }
 
+  /* TODO: Make diagram not be initialized twice */
+  if (diagram == NULL) {
+    diagram = new_diagram(filename);
+  }
+  if (diagram == NULL) return NULL;
+
+  if (   !diagram_init(diagram, filename)
+      || !diagram_load_into (diagram, filename, ifilter)) {
+    if (!was_default) /* don't kill the default diagram on import failure */
+      diagram_destroy(diagram);
+    diagram = NULL;
+  } else {
+    /* not modifying 'diagram->unsaved' the state depends on the import filter used */
+    diagram_set_modified(diagram, FALSE);
+    if (app_is_interactive()) {
+      recent_file_history_add(filename);
+      diagram_tree_add(diagram_tree(), diagram);
+    }
+  }
+  
+  if (diagram != NULL && was_default && app_is_interactive()) {
+    diagram_update_for_filename(diagram);
+    diagram->is_default = FALSE;
+    if ( g_slist_length(diagram->displays) == 1 )
+      display_set_active (diagram->displays->data);
+  }
+  
   return diagram;
 }
 
+/** Create a new diagram with the given filename.
+ * If the diagram could not be created, e.g. because the filename is not
+ * a legal string in the current encoding, return NULL.
+ */
 Diagram *
 new_diagram(const char *filename)  /* Note: filename is copied */
 {
-  Diagram *dia = diagram_new();
+  Diagram *dia = g_object_new(DIA_TYPE_DIAGRAM, NULL);
 
-  diagram_init(dia, filename);
-
-  return dia;
+  if (diagram_init(dia, filename)) {
+    return dia;
+  } else {
+    g_object_unref(dia);
+    return NULL;
+  }
 }
 
 void
 diagram_destroy(Diagram *dia)
 {
-  diagram_close_related_dialogs(dia);
+  g_signal_emit (dia, diagram_signals[REMOVED], 0);
   g_object_unref(dia);
 }
 
@@ -230,14 +374,26 @@ void
 diagram_modified(Diagram *dia)
 {
   GSList *displays;
+  gchar *dia_name = diagram_get_name(dia);
+  gchar *extra = g_path_get_dirname (dia->filename); 
+  gchar *title = g_strdup_printf ("%s%s (%s)", diagram_is_modified(dia) ? "*" : "", dia_name, extra ? extra : " ");
+
+  g_free (dia_name);
+  g_free (extra);
   displays = dia->displays;
-  while (displays != NULL) {
-    DDisplay *display = (DDisplay*) displays->data;
-    ddisplay_update_statusbar(display);
+  while (displays!=NULL) {
+    DDisplay *ddisp = (DDisplay *) displays->data;
+
+    ddisplay_set_title(ddisp, title);
+    
     displays = g_slist_next(displays);
   }
-  if (diagram_is_modified(dia)) dia->autosaved = FALSE;
+  if (diagram_is_modified(dia)) {
+    dia->autosaved = FALSE;
+    dia->is_default = FALSE;
+  }
   /*  diagram_set_modified(dia, TRUE);*/
+  g_free (title);
 }
 
 /** Set this diagram explicitly modified.  This should not be called
@@ -279,7 +435,8 @@ diagram_selected_any_parents(Diagram *dia) {
   for (selected = dia->data->selected;
        selected != NULL; selected = selected->next) {
     DiaObject *obj = (DiaObject*)selected->data;
-    if (obj->can_parent && obj->children != NULL) return TRUE;
+    if (object_flags_set(obj, DIA_OBJECT_CAN_PARENT) && obj->children != NULL) 
+      return TRUE;
   }
   return FALSE;
 }
@@ -307,7 +464,7 @@ diagram_selected_can_parent(Diagram *dia) {
   for (selected = dia->data->selected;
        selected != NULL; selected = selected->next) {
     DiaObject *obj = (DiaObject*)selected->data;
-    if (obj->can_parent) {
+    if (object_flags_set(obj, DIA_OBJECT_CAN_PARENT)) {
       parents = g_list_prepend(parents, obj);
     }
   }
@@ -316,23 +473,8 @@ diagram_selected_can_parent(Diagram *dia) {
     DiaObject *obj = (DiaObject*)selected->data;
     if (obj->parent == NULL) {
       GList *parent_tmp;
-      Rectangle obj_bb = obj->bounding_box;
       for (parent_tmp = parents; parent_tmp != NULL; parent_tmp = parent_tmp->next) {
-	DiaObject *p = (DiaObject*)parent_tmp->data;
-	if (p == obj) continue;
-	if (obj_bb.left > p->bounding_box.left &&
-	    obj_bb.right < p->bounding_box.right &&
-	    obj_bb.top > p->bounding_box.top &&
-	    obj_bb.bottom < p->bounding_box.bottom) {
-	  printf("Obj %f, %f x %f, %f inside %f, %f x %f, %f\n",
-		 obj_bb.left,
-		 obj_bb.top,
-		 obj_bb.right,
-		 obj_bb.bottom,
-		 p->bounding_box.left,
-		 p->bounding_box.top,
-		 p->bounding_box.right,
-		 p->bounding_box.bottom);
+	if (object_within_parent(obj, (DiaObject*)parent_tmp->data)) {
 	  g_list_free(parents);
 	  return TRUE;
 	}
@@ -343,115 +485,167 @@ diagram_selected_can_parent(Diagram *dia) {
   return FALSE;
 }
 
+/** Returns TRUE if an object is fully enclosed by a another object, which 
+ * can be a parent */
+gboolean
+object_within_parent(DiaObject *obj, DiaObject *p)
+{
+  Rectangle obj_bb = obj->bounding_box;
+  if (!object_flags_set(p, DIA_OBJECT_CAN_PARENT))
+    return FALSE;
+  if (p == obj) 
+    return FALSE;
+  if (obj_bb.left > p->bounding_box.left &&
+      obj_bb.right < p->bounding_box.right &&
+      obj_bb.top > p->bounding_box.top &&
+      obj_bb.bottom < p->bounding_box.bottom)
+    return TRUE;
+  return FALSE;
+}
+
 /*
   This is the real implementation of the sensitivity update.
   TODO: move it to the DDisplay as it belongs to it IMHO
  */
 void 
-diagram_update_menu_sensitivity (Diagram *dia, UpdatableMenuItems *items)
+diagram_update_menu_sensitivity (Diagram *dia)
 {
+  gint selected_count = g_list_length (dia->data->selected);
+  DDisplay *ddisp = ddisplay_active();
+  gboolean focus_active = (get_active_focus(dia->data) != NULL);
+  gboolean textedit_active = ddisp ? textedit_mode(ddisp) : FALSE;
+  GtkAction *action;
+
   /* Edit menu */
-  gtk_widget_set_sensitive(GTK_WIDGET(items->copy),
-			   dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->cut),
-			   dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->paste),
-			   cnp_exist_stored_objects());
-  #ifndef GNOME
-  gtk_widget_set_sensitive(GTK_WIDGET(items->edit_delete),
-			   dia->data->selected_count > 0);
-  #endif
-  gtk_widget_set_sensitive(GTK_WIDGET(items->copy_text),
-			   active_focus() != NULL);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->cut_text),
-			   active_focus() != NULL);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->paste_text),
-			   active_focus() != NULL);
-  
+  if ((action = menus_get_action ("EditUndo")) != NULL)
+    gtk_action_set_sensitive (action, dia ? undo_available(dia->undo, TRUE) : FALSE);
+  if ((action = menus_get_action ("EditRedo")) != NULL)
+    gtk_action_set_sensitive (action, dia ? undo_available(dia->undo, FALSE) : FALSE);
+  if ((action = menus_get_action ("EditCopy")) != NULL)
+    gtk_action_set_sensitive (action, textedit_active || selected_count > 0);
+  if ((action = menus_get_action ("EditCut")) != NULL)
+    gtk_action_set_sensitive (action, textedit_mode(ddisp) || selected_count > 0);
+  if ((action = menus_get_action ("EditPaste")) != NULL)
+    gtk_action_set_sensitive (action, textedit_active || cnp_exist_stored_objects());
+  if ((action = menus_get_action ("EditDelete")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+  if ((action = menus_get_action ("EditDuplicate")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+
+  if ((action = menus_get_action ("EditCopytext")) != NULL)
+    gtk_action_set_sensitive (action, focus_active);
+  if ((action = menus_get_action ("EditCuttext")) != NULL)
+    gtk_action_set_sensitive (action, focus_active);
+  if ((action = menus_get_action ("EditPastetext")) != NULL)
+    gtk_action_set_sensitive (action, focus_active);
+
   /* Objects menu */
-  gtk_widget_set_sensitive(GTK_WIDGET(items->send_to_back),
-			   dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->bring_to_front),
-			   dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->send_backwards),
-			   dia->data->selected_count > 0);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->bring_forwards),
-			   dia->data->selected_count > 0);
-    
-  gtk_widget_set_sensitive(GTK_WIDGET(items->parent),
-			   diagram_selected_can_parent(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->unparent),
-			   diagram_selected_any_children(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->unparent_children),
-			   diagram_selected_any_parents(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->group),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->ungroup),
-			   diagram_selected_any_groups(dia));
-  gtk_widget_set_sensitive(GTK_WIDGET(items->properties),
-			   dia->data->selected_count > 0);
-  
+  if ((action = menus_get_action ("ObjectsSendtoback")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+  if ((action = menus_get_action ("ObjectsBringtofront")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+  if ((action = menus_get_action ("ObjectsSendbackwards")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+  if ((action = menus_get_action ("ObjectsBringforwards")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+
+  if ((action = menus_get_action ("ObjectsLayerAbove")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+  if ((action = menus_get_action ("ObjectsLayerBelow")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 0);
+
+  if ((action = menus_get_action ("ObjectsGroup")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsUngroup")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && diagram_selected_any_groups (dia));
+  if ((action = menus_get_action ("ObjectsParent")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && diagram_selected_can_parent (dia));
+  if ((action = menus_get_action ("ObjectsUnparent")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && diagram_selected_any_children (dia));
+  if ((action = menus_get_action ("ObjectsUnparentchildren")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && diagram_selected_any_parents (dia));
+
+  if ((action = menus_get_action ("ObjectsProperties")) != NULL)
+    gtk_action_set_sensitive (action, selected_count > 0);
+
   /* Objects->Align menu */
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_l),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_c),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_r),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_e),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_h_a),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_t),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_c),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_b),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_e),
-			   dia->data->selected_count > 1);
-  gtk_widget_set_sensitive(GTK_WIDGET(items->align_v_a),
-			   dia->data->selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignLeft")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignCenter")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignRight")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignSpreadouthorizontally")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignAdjacent")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignTop")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignMiddle")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignBottom")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignSpreadoutvertically")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+  if ((action = menus_get_action ("ObjectsAlignStacked")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active && selected_count > 1);
+
+  /* Select menu */
+  if ((action = menus_get_action ("SelectAll")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectNone")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectInvert")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectTransitive")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectConnected")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectSametype")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+
+  if ((action = menus_get_action ("SelectReplace")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectUnion")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectIntersection")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectRemove")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+  if ((action = menus_get_action ("SelectInverse")) != NULL)
+    gtk_action_set_sensitive (action, !textedit_active);
+
+  /* Tools menu */
+  {
+    /* Keep in sync with menus.c(tool_entries) */
+    static gchar * action_names[] = {
+      "ToolsModify", "ToolsMagnify",  "ToolsTextedit",  "ToolsScroll", 
+      "ToolsText", "ToolsBox", "ToolsEllipse", "ToolsPolygon", "ToolsBeziergon",
+      "ToolsLine", "ToolsArc", "ToolsZigzagline", "ToolsPolyline","ToolsBezierline", 
+      "ToolsImage", "ToolsOutline", NULL
+    };
+    int i;
+    for (i = 0; action_names[i] != NULL; ++i) {
+      if ((action = menus_get_action (action_names[i])) != NULL)
+        gtk_action_set_sensitive (action, !textedit_active);
+    }
+  }
+  /* View menu - should not need disabling yet */
 }
     
   
-void diagram_update_menubar_sensitivity(Diagram *dia, UpdatableMenuItems *items)
-{
-    diagram_update_menu_sensitivity (dia, items);
-}
-
-
-void diagram_update_popupmenu_sensitivity(Diagram *dia)
-{
-  static int initialized = 0;
-  static UpdatableMenuItems items;
-  
-
-  char *display = "<Display>";
-  
-  if (initialized==0) {
-      menus_initialize_updatable_items (&items, NULL, display);
-      
-      initialized = 1;
-  }
-  
-  diagram_update_menu_sensitivity (dia, &items);
-}
-
 void
 diagram_add_ddisplay(Diagram *dia, DDisplay *ddisp)
 {
   dia->displays = g_slist_prepend(dia->displays, ddisp);
-  dia->display_count++;
 }
 
 void
 diagram_remove_ddisplay(Diagram *dia, DDisplay *ddisp)
 {
   dia->displays = g_slist_remove(dia->displays, ddisp);
-  dia->display_count--;
 
-  if (dia->display_count == 0) {
+  if (g_slist_length(dia->displays) == 0) {
     if (!app_is_embedded()) {
       /* Don't delete embedded diagram when last view is closed */
       diagram_destroy(dia);
@@ -548,22 +742,19 @@ void
 diagram_remove_all_selected(Diagram *diagram, int delete_empty)
 {
   object_add_updates_list(diagram->data->selected, diagram);
-	
+  textedit_remove_focus_all(diagram);
   data_remove_all_selected(diagram->data);
-  
-  remove_focus();
+  g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0, g_list_length (diagram->data->selected));
 }
 
 void
 diagram_unselect_object(Diagram *diagram, DiaObject *obj)
 {
   object_add_updates(obj, diagram);
-  
-  data_unselect(diagram->data, obj);
-  
-  if ((active_focus()!=NULL) && (active_focus()->obj == obj)) {
-    remove_focus();
-  }
+  textedit_remove_focus(obj, diagram);
+  data_unselect(DIA_DIAGRAM_DATA(diagram), obj);
+  g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0,
+		 g_list_length (DIA_DIAGRAM_DATA(diagram)->selected));
 }
 
 void
@@ -572,6 +763,8 @@ diagram_unselect_objects(Diagram *dia, GList *obj_list)
   GList *list;
   DiaObject *obj;
 
+  /* otherwise we would signal objects step by step */
+  g_signal_handlers_block_by_func (dia, _diagram_selection_changed, NULL);
   list = obj_list;
   while (list != NULL) {
     obj = (DiaObject *) list->data;
@@ -582,19 +775,34 @@ diagram_unselect_objects(Diagram *dia, GList *obj_list)
 
     list = g_list_next(list);
   }
+  g_signal_handlers_unblock_by_func (dia, _diagram_selection_changed, NULL);
+  g_signal_emit (dia, diagram_signals[SELECTION_CHANGED], 0, g_list_length (dia->data->selected));
 }
 
+/** Make a single object selected.
+ * Note that an object inside a closed group cannot be made selected, nor
+ * can an object in a non-active layer.
+ * @param diagram The diagram that the object belongs to (sorta redundant now)
+ * @param obj The object that should be made selected.
+ */
 void
 diagram_select(Diagram *diagram, DiaObject *obj)
 {
-  data_select(diagram->data, obj);
-  object_add_updates(obj, diagram);
+  if (dia_object_is_selectable(obj)) {
+    data_select(diagram->data, obj);
+    obj->ops->selectf(obj, NULL, NULL);
+    object_add_updates(obj, diagram);
+    g_signal_emit (diagram, diagram_signals[SELECTION_CHANGED], 0,
+		   g_list_length (diagram->data->selected));
+  }
 }
 
 void
 diagram_select_list(Diagram *dia, GList *list)
 {
-
+  g_return_if_fail (dia && list);
+  /* otherwise we would signal objects step by step */
+  g_signal_handlers_block_by_func (dia, _diagram_selection_changed, NULL);
   while (list != NULL) {
     DiaObject *obj = (DiaObject *)list->data;
 
@@ -602,6 +810,11 @@ diagram_select_list(Diagram *dia, GList *list)
 
     list = g_list_next(list);
   }
+  if (get_active_focus((DiagramData*) dia) == NULL) {
+    textedit_activate_first(ddisplay_active());
+  }
+  g_signal_handlers_unblock_by_func (dia, _diagram_selection_changed, NULL);
+  g_signal_emit (dia, diagram_signals[SELECTION_CHANGED], 0, g_list_length (dia->data->selected));
 }
 
 int
@@ -778,7 +991,7 @@ diagram_find_closest_connectionpoint(Diagram *dia,
 				     DiaObject *notthis)
 {
   real dist = 100000000.0;
-  int i;
+  guint i;
   for (i=0;i<dia->data->layers->len;i++) {
     Layer *layer = (Layer*)g_ptr_array_index(dia->data->layers, i);
     ConnectionPoint *this_cp;
@@ -840,16 +1053,21 @@ strip_connections(DiaObject *obj, GList *not_strip_list, Diagram *dia)
 }
 
 
-gint diagram_parent_sort_cb(object_extent ** a, object_extent **b)
+/* GCompareFunc */
+static gint 
+diagram_parent_sort_cb(gconstpointer _a, gconstpointer _b)
 {
-  if ((*a)->extent->left < (*b)->extent->left)
+  ObjectExtent **a = (ObjectExtent **)_a;
+  ObjectExtent **b = (ObjectExtent **)_b;
+
+  if ((*a)->extent.left < (*b)->extent.left)
     return 1;
-  else if ((*a)->extent->left > (*b)->extent->left)
+  else if ((*a)->extent.left > (*b)->extent.left)
     return -1;
   else
-    if ((*a)->extent->top < (*b)->extent->top)
+    if ((*a)->extent.top < (*b)->extent.top)
       return 1;
-    else if ((*a)->extent->top > (*b)->extent->top)
+    else if ((*a)->extent.top > (*b)->extent.top)
       return -1;
     else
       return 0;
@@ -863,48 +1081,48 @@ void diagram_parent_selected(Diagram *dia)
   GList *list = dia->data->selected;
   int length = g_list_length(list);
   int idx, idx2;
-  object_extent *oe;
+  ObjectExtent *oe;
   gboolean any_parented = FALSE;
-  GPtrArray *rects = g_ptr_array_sized_new(length);
+  GPtrArray *extents = g_ptr_array_sized_new(length);
   while (list)
   {
-    oe = g_new(object_extent, 1);
+    oe = g_new(ObjectExtent, 1);
     oe->object = list->data;
-    oe->extent = parent_handle_extents(list->data);
-    g_ptr_array_add(rects, oe);
+    parent_handle_extents(list->data, &oe->extent);
+    g_ptr_array_add(extents, oe);
     list = g_list_next(list);
   }
-  /* sort all the objects by its left position */
-  g_ptr_array_sort(rects, diagram_parent_sort_cb);
+  /* sort all the objects by their left position */
+  g_ptr_array_sort(extents, diagram_parent_sort_cb);
 
   for (idx = 0; idx < length; idx++)
   {
-    object_extent *rect = g_ptr_array_index(rects, idx);
-    if (rect->object->parent)
+    ObjectExtent *oe1 = g_ptr_array_index(extents, idx);
+    if (oe1->object->parent)
       continue;
 
     for (idx2 = idx + 1; idx2 < length; idx2++)
     {
-      object_extent *rect2 = g_ptr_array_index(rects, idx2);
-      if (!rect2->object->can_parent)
+      ObjectExtent *oe2 = g_ptr_array_index(extents, idx2);
+      if (!object_flags_set(oe2->object, DIA_OBJECT_CAN_PARENT))
         continue;
 
-      if (rect->extent->right <= rect2->extent->right
-        && rect->extent->bottom <= rect2->extent->bottom)
+      if (oe1->extent.right <= oe2->extent.right
+        && oe1->extent.bottom <= oe2->extent.bottom)
       {
 	Change *change;
-	change = undo_parenting(dia, rect2->object, rect->object, TRUE);
+	change = undo_parenting(dia, oe2->object, oe1->object, TRUE);
 	(change->apply)(change, dia);
 	any_parented = TRUE;
 	/*
-        rect->object->parent = rect2->object;
-	rect2->object->children = g_list_append(rect2->object->children, rect->object);
+        oe1->object->parent = oe2->object;
+	oe2->object->children = g_list_append(oe2->object->children, oe1->object);
 	*/
 	break;
       }
     }
   }
-  g_ptr_array_free(rects, TRUE);
+  g_ptr_array_free(extents, TRUE);
   if (any_parented) {
     diagram_modified(dia);
     diagram_flush(dia);
@@ -947,13 +1165,12 @@ void diagram_unparent_selected(Diagram *dia)
 void diagram_unparent_children_selected(Diagram *dia)
 {
   GList *list;
-  GList *child_ptr;
   DiaObject *obj, *child;
   gboolean any_unparented = FALSE;
   for (list = dia->data->selected; list != NULL; list = g_list_next(list))
   {
     obj = (DiaObject *) list->data;
-    if (!obj->can_parent || !obj->children)
+    if (!object_flags_set(obj, DIA_OBJECT_CAN_PARENT) || !obj->children)
       continue;
 
     any_unparented = TRUE;
@@ -968,23 +1185,9 @@ void diagram_unparent_children_selected(Diagram *dia)
       /* This will remove one item from the list, so the while terminates. */
       (change->apply)(change, dia);
     }
-    /*
-    for (child_ptr = obj->children; child_ptr != NULL; 
-	 child_ptr = g_list_next(child_ptr))
-    {
-      Change *change;
-      child = (DiaObject *) child_ptr->data;
-      change = undo_parenting(dia, obj, child, FALSE);
-      (change->apply)(change, dia);
-    }
-    */
     if (obj->children != NULL)
       printf("Obj still has %d children\n",
 	     g_list_length(obj->children));
-    /*
-    g_list_free(obj->children);
-    obj->children = NULL;
-    */
   }
   if (any_unparented) {
     diagram_modified(dia);
@@ -1002,8 +1205,37 @@ void diagram_group_selected(Diagram *dia)
   GList *orig_list;
   Change *change;
 
-  dia->data->selected = parent_list_affected(dia->data->selected);
+  if (g_list_length(dia->data->selected) < 1) {
+    message_error(_("Trying to group with no selected objects."));
+    return;
+  }
+  
+#ifdef USE_NEWGROUP
+  list = dia->data->selected;
+  current_parent = ((DiaObject *) list->data)->parent;
+  while (list != NULL) {
+    obj = (DiaObject *)list->data;
+    if (obj->parent != current_parent) {
+      message_warning(_("You cannot group objects that belong to different groups or have different parents"));
+      return;
+    }
+  }
 
+  group = ...
+
+  list = dia->data->selected;
+  while (list != NULL) {
+    obj = (DiaObject *)list->data;
+    
+  }
+#else
+#if 0
+  /* the following is wrong as it screws up the selected list, see bug #153525
+     * I just don't get what was originally intented so please speak up if you know  --hb
+     */    
+  dia->data->selected = parent_list_affected(dia->data->selected);
+#endif
+    
   orig_list = g_list_copy(dia->data->active_layer->objects);
   
   /* We have to rebuild the selection list so that it is the same
@@ -1019,38 +1251,24 @@ void diagram_group_selected(Diagram *dia)
     /* The connections aren't reattached by ungroup. */
     strip_connections(obj, dia->data->selected, dia);
     
-    /* Have to hide any open properties dialog
-     * if it contains some object in cut_list */
-    /* Now handled by undo_apply
-    properties_hide_if_shown(dia, obj);
-    */
-
-    /* Remove focus if active */
-    /* Now handled by undo apply
-    if ((active_focus()!=NULL) && (active_focus()->obj == obj)) {
-      remove_focus();
-    }
-    */
-
-    /* Now handled by the undo apply
-    object_add_updates(obj, dia);
-    */
     list = g_list_next(list);
   }
 
   /* Remove list of selected objects */
+  textedit_remove_focus_all(dia);
   data_remove_all_selected(dia->data);
   
   group = group_create(group_list);
   change = undo_group_objects(dia, group_list, group, orig_list);
   (change->apply)(change, dia);
+#endif
 
   /* Select the created group */
   diagram_select(dia, group);
   
   diagram_modified(dia);
   diagram_flush(dia);
-
+  
   undo_set_transactionpoint(dia->undo);
 }
 
@@ -1058,17 +1276,17 @@ void diagram_ungroup_selected(Diagram *dia)
 {
   DiaObject *group;
   GList *group_list;
-/*   GList *list; */
-  GList *selected;
+  GList *selected, *selection_copy;
   int group_index;
   int any_groups = 0;
   
-  if (dia->data->selected_count < 1) {
+  if (g_list_length(dia->data->selected) < 1) {
     message_error("Trying to ungroup with no selected objects.");
     return;
   }
   
-  selected = dia->data->selected;
+  selection_copy = g_list_copy(dia->data->selected);
+  selected = selection_copy;
   while (selected != NULL) {
     group = (DiaObject *)selected->data;
 
@@ -1080,34 +1298,17 @@ void diagram_ungroup_selected(Diagram *dia)
 
       group_list = group_objects(group);
       diagram_select_list(dia, group_list);
-      /* Now handled by undo apply */      
-      /*list = group_list;
-      while (list != NULL) {
-	DiaObject *obj = (DiaObject *)list->data;
-	object_add_updates(obj, dia);
-	list = g_list_next(list);
-      }
-      */
-
-      /* Now handled by undo apply
-      layer_replace_object_with_list(dia->data->active_layer,
-				     group, g_list_copy(group_list));
-      */
 
       group_index = layer_object_index(dia->data->active_layer, group);
     
       change = undo_ungroup_objects(dia, group_list, group, group_index);
       (change->apply)(change, dia);
-      /* Now handled by undo apply 
-      diagram_tree_add_objects(diagram_tree(), dia, group_list);
-      diagram_tree_remove_object(diagram_tree(), group);
-      properties_hide_if_shown(dia, group);
-      */
 
       any_groups = 1;
     }
     selected = g_list_next(selected);
   }
+  g_list_free(selection_copy);
   
   if (any_groups) {
     diagram_modified(dia);
@@ -1139,7 +1340,7 @@ diagram_place_under_selected(Diagram *dia)
   GList *sorted_list;
   GList *orig_list;
 
-  if (dia->data->selected_count == 0)
+  if (g_list_length (dia->data->selected) == 0)
     return;
 
   orig_list = g_list_copy(dia->data->active_layer->objects);
@@ -1161,7 +1362,7 @@ diagram_place_over_selected(Diagram *dia)
   GList *sorted_list;
   GList *orig_list;
 
-  if (dia->data->selected_count == 0)
+  if (g_list_length (dia->data->selected) == 0)
     return;
 
   orig_list = g_list_copy(dia->data->active_layer->objects);
@@ -1185,7 +1386,7 @@ diagram_place_up_selected(Diagram *dia)
   GList *tmp, *stmp;
   GList *new_list = NULL;
 
-  if (dia->data->selected_count == 0)
+  if (g_list_length (dia->data->selected) == 0)
     return;
 
   orig_list = g_list_copy(dia->data->active_layer->objects);
@@ -1228,7 +1429,7 @@ diagram_place_down_selected(Diagram *dia)
   GList *tmp, *stmp;
   GList *new_list = NULL;
 
-  if (dia->data->selected_count == 0)
+  if (g_list_length (dia->data->selected) == 0)
     return;
 
   orig_list = g_list_copy(dia->data->active_layer->objects);
@@ -1237,7 +1438,7 @@ diagram_place_down_selected(Diagram *dia)
   object_add_updates_list(orig_list, dia);
 
   /* Sanity check */
-  g_assert(dia->data->selected_count == g_list_length(sorted_list));
+  g_assert(g_list_length (dia->data->selected) == g_list_length(sorted_list));
 
   new_list = g_list_copy(orig_list);
   tmp = new_list;
@@ -1268,14 +1469,25 @@ diagram_place_down_selected(Diagram *dia)
 }
 
 void
-diagram_set_filename(Diagram *dia, char *filename)
+diagram_set_filename(Diagram *dia, const char *filename)
+{
+  g_free(dia->filename);
+  dia->filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
+  
+  diagram_update_for_filename(dia);
+}
+
+/** Update the various areas that require updating when changing filename
+ * This will ensure that all places that use the filename are updated:
+ * Window titles, layer dialog, recent files, diagram tree.
+ * @param dia The diagram whose filename has changed.
+ */
+static void
+diagram_update_for_filename(Diagram *dia)
 {
   GSList *l;
   DDisplay *ddisp;
   char *title;
-  
-  g_free(dia->filename);
-  dia->filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
 
   title = diagram_get_name(dia);
 
@@ -1291,7 +1503,6 @@ diagram_set_filename(Diagram *dia, char *filename)
   g_free(title);
 
   layer_dialog_update_diagram_list();
-  recent_file_history_add((const char *)filename);
 
   diagram_tree_update_name(diagram_tree(), dia);
 }
@@ -1303,14 +1514,7 @@ diagram_set_filename(Diagram *dia, char *filename)
 gchar *
 diagram_get_name(Diagram *dia)
 {
-  gchar *title = strrchr(dia->filename, G_DIR_SEPARATOR);
-  if (title==NULL) {
-    title = dia->filename;
-  } else {
-    title++;
-  }
-  
-  return g_strdup(title);
+  return g_path_get_basename(dia->filename);
 }
 
 int diagram_modified_exists(void)
@@ -1335,27 +1539,3 @@ void diagram_object_modified(Diagram *dia, DiaObject *object)
 {
   diagram_tree_update_object(diagram_tree(), dia, object);
 }
-
-static 
-void close_dialog(gpointer data, gpointer user_data)
-{
-  g_signal_emit_by_name(G_OBJECT(data), "delete_event");
-}
-
-void diagram_close_related_dialogs(Diagram *dia)
-{
-  GSList *tmp;
-  tmp = g_slist_copy(dia->related_dialogs);
-  g_slist_foreach(tmp, close_dialog, NULL);
-  g_slist_free(tmp);
-}
-
-void diagram_add_related_dialog(Diagram *dia, gpointer data)
-{
-  dia->related_dialogs = g_slist_prepend(dia->related_dialogs, data);
-}
-
-void diagram_remove_related_dialog(Diagram *dia, gpointer data)
-{
-  dia->related_dialogs = g_slist_remove(dia->related_dialogs, data);
-}
