@@ -23,12 +23,13 @@
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
+#ifdef GNOME
+#undef GTK_DISABLE_DEPRECATED
+#include <gnome.h>
+#endif
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#ifdef GNOME
-#include <gnome.h>
-#endif
 #include "display.h"
 #include "diagram.h"
 #include "tool.h"
@@ -43,19 +44,35 @@
 #include "diamenu.h"
 #include "preferences.h"
 #include "scroll_tool.h"
+#include "commands.h"
+#include "highlight.h"
+#include "textedit.h"
+#include "lib/parent.h"
 
 /* This contains the point that was clicked to get this menu */
 static Point object_menu_clicked_point;
 
+typedef struct {
+	GdkEvent *event; /* Button down event which may be holding */
+	DDisplay *ddisp; /* DDisplay where event occurred */
+	guint     tag;   /* Tag for timeout */
+} HoldTimeoutData;
+
+static HoldTimeoutData hold_data = {NULL, NULL, 0};
+
+	
+
 static void
 object_menu_proxy(GtkWidget *widget, gpointer data)
 {
   DiaMenuItem *dia_menu_item;
   ObjectChange *obj_change;
-
+  DiaObject *obj;
   DDisplay *ddisp = ddisplay_active();
-  Object *obj = (Object *)ddisp->diagram->data->selected->data;
 
+  if (!ddisp) return;
+
+  obj = (DiaObject *)ddisp->diagram->data->selected->data;
   dia_menu_item = (DiaMenuItem *) data;
 
 
@@ -80,18 +97,42 @@ object_menu_proxy(GtkWidget *widget, gpointer data)
     undo_clear(ddisp->diagram->undo);
   }
 
-
   diagram_flush(ddisp->diagram);
 }
 
 static void
-dia_menu_free(DiaMenu *dia_menu) {
+dia_menu_free(DiaMenu *dia_menu) 
+{
   if (dia_menu->app_data)
     gtk_object_destroy((GtkObject *)dia_menu->app_data);
   dia_menu->app_data = NULL;
   dia_menu->app_data_free = NULL;
 }
 
+/*
+  This add a Properties... menu item to the GtkMenu passed, at the
+  end and set the callback to raise de properties dialog
+
+  pass TRUE in separator if you want to insert a separator before the poperty
+  menu item.
+*/
+static void 
+add_properties_menu_item (GtkMenu *menu, gboolean separator) 
+{
+  GtkWidget *menu_item = NULL;
+
+  if (separator) {
+    menu_item = gtk_menu_item_new();
+    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+    gtk_widget_show(menu_item);
+  }
+
+  menu_item = gtk_menu_item_new_with_label(_("Properties..."));
+  g_signal_connect(GTK_OBJECT(menu_item), "activate", G_CALLBACK(dialogs_properties_callback), NULL);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+  gtk_widget_show(menu_item);
+}
+
 static void
 create_object_menu(DiaMenu *dia_menu)
 {
@@ -100,7 +141,6 @@ create_object_menu(DiaMenu *dia_menu)
   GtkWidget *menu_item;
 
   menu = gtk_menu_new();
-      //FIXME?: gtk_menu_ensure_uline_accel_group (GTK_MENU (menu)) ;
 
   if ( dia_menu->title ) {
     menu_item = gtk_menu_item_new_with_label(gettext(dia_menu->title));
@@ -109,6 +149,7 @@ create_object_menu(DiaMenu *dia_menu)
     gtk_widget_show(menu_item);
   }
 
+  /* separator below the menu title */
   menu_item = gtk_menu_item_new();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
   gtk_widget_show(menu_item);
@@ -123,8 +164,6 @@ create_object_menu(DiaMenu *dia_menu)
         menu_item = gtk_check_menu_item_new();
       gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item),
                                      item->active & DIAMENU_TOGGLE_ON);
-      gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menu_item),
-                                          TRUE);
     } else {
       if (item->text)
         menu_item = gtk_menu_item_new_with_label(gettext(item->text));
@@ -136,8 +175,8 @@ create_object_menu(DiaMenu *dia_menu)
     item->app_data = menu_item;
     if ( dia_menu->items[i].callback ) {
           /* only connect signal handler if there is actually a callback */
-      gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
-                         (GtkSignalFunc)object_menu_proxy, &dia_menu->items[i]);
+      g_signal_connect (G_OBJECT (menu_item), "activate",
+                        G_CALLBACK (object_menu_proxy), &dia_menu->items[i]);
     } else { 
       if ( item->callback_data ) { 
             /* This menu item is a submenu if it has no callback, but does
@@ -153,24 +192,34 @@ create_object_menu(DiaMenu *dia_menu)
     }
   }
 
+  /* Finally add a Properties... menu item for objects*/
+  add_properties_menu_item(GTK_MENU (menu), i > 0);
+
   dia_menu->app_data = menu;
   dia_menu->app_data_free = dia_menu_free;
 }
 
+static DiaMenuItem empty_menu_items[] = { {0, } };
+static DiaMenu empty_menu = {
+  NULL,
+  0, /* NOT: sizeof(empty_menu_items)/sizeof(DiaMenuItem), */
+  empty_menu_items,
+  NULL
+};
+
 static void
 popup_object_menu(DDisplay *ddisp, GdkEventButton *bevent)
 {
   Diagram *diagram;
-  Object *obj;
+  DiaObject *obj;
   GtkMenu *menu = NULL;
   DiaMenu *dia_menu = NULL;
-  GtkWidget *menu_item;
   GList *selected_list;
-  static GtkWidget *no_menu = NULL;
   int i;
+  int num_items;
   
   diagram = ddisp->diagram;
-  if (diagram->data->selected_count != 1)
+  if (g_list_length (diagram->data->selected) != 1)
     return;
   
   selected_list = diagram->data->selected;
@@ -178,54 +227,49 @@ popup_object_menu(DDisplay *ddisp, GdkEventButton *bevent)
   /* Have to have exactly one selected object */
   if (selected_list == NULL || g_list_next(selected_list) != NULL) {
     message_error("Selected list is %s while selected_count is %d\n",
-		  (selected_list?"long":"empty"), diagram->data->selected_count);
+		  (selected_list?"long":"empty"), g_list_length (diagram->data->selected));
     return;
   }
   
-  obj = (Object *)g_list_first(selected_list)->data;
+  obj = (DiaObject *)g_list_first(selected_list)->data;
   
   /* Possibly react differently at a handle? */
 
-  /* Get its menu */
-  if (obj->ops->get_object_menu == NULL) {
-    dia_menu = NULL;
+  /* Get its menu, and remember the # of object-generated items */
+  if (obj->ops->get_object_menu == NULL || (obj->ops->get_object_menu)(obj, &object_menu_clicked_point) == NULL) {
+    dia_menu = &empty_menu;
+    if (dia_menu->title &&
+	(0 != strcmp(dia_menu->title,obj->type->name))) {
+      dia_menu->app_data_free(dia_menu);
+    }
+    dia_menu->title = obj->type->name;
+    num_items = 0;
   } else {
     dia_menu = (obj->ops->get_object_menu)(obj, &object_menu_clicked_point);
+    num_items = dia_menu->num_items;
   }
 
-  if (dia_menu != NULL) {
-    if (dia_menu->app_data == NULL)
-      create_object_menu(dia_menu);
-    
-    /* Update active/nonactive menuitems */
-    for (i=0;i<dia_menu->num_items;i++) {
-      DiaMenuItem *item = &dia_menu->items[i];
-      gtk_widget_set_sensitive(GTK_WIDGET(item->app_data),
-			       item->active & DIAMENU_ACTIVE);
-      if (item->active & DIAMENU_TOGGLE) {
-	g_signal_handlers_block_by_func(GTK_CHECK_MENU_ITEM(item->app_data),
-					(GtkSignalFunc)object_menu_proxy, item);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item->app_data),
-				       item->active & DIAMENU_TOGGLE_ON);
-	g_signal_handlers_unblock_by_func(GTK_CHECK_MENU_ITEM(item->app_data),
-					  (GtkSignalFunc)object_menu_proxy, item);
-      }
-    }
-
-    menu = GTK_MENU(dia_menu->app_data);
-  } else {
-    if (no_menu == NULL) {
-      no_menu = gtk_menu_new();
-
-      menu_item = gtk_menu_item_new_with_label(_("No object menu"));
-      gtk_menu_shell_append (GTK_MENU_SHELL (no_menu), menu_item);
-      gtk_widget_show(menu_item);
-      gtk_widget_set_sensitive(menu_item, FALSE);
-    }
-    menu = GTK_MENU(no_menu);
+  if (dia_menu->app_data == NULL) {
+    create_object_menu(dia_menu);
   }
+  /* Update active/nonactive menuitems */
+  for (i=0;i<num_items;i++) {
+    DiaMenuItem *item = &dia_menu->items[i];
+    gtk_widget_set_sensitive(GTK_WIDGET(item->app_data),
+			     item->active & DIAMENU_ACTIVE);
+    if (item->active & DIAMENU_TOGGLE) {
+      g_signal_handlers_block_by_func (G_OBJECT (GTK_CHECK_MENU_ITEM (item->app_data)),
+				      G_CALLBACK (object_menu_proxy), item);
+      gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(item->app_data),
+				      item->active & DIAMENU_TOGGLE_ON);
+      g_signal_handlers_unblock_by_func (G_OBJECT (GTK_CHECK_MENU_ITEM(item->app_data)),
+					G_CALLBACK (object_menu_proxy), item);
+    }
+  }
+
+  menu = GTK_MENU(dia_menu->app_data);
+  /* add the properties menu item to raise the properties from the contextual menu */
   
-  popup_shell = ddisp->shell;
   gtk_menu_popup(menu, NULL, NULL, NULL, NULL, bevent->button, bevent->time);
 }
 
@@ -241,7 +285,7 @@ ddisplay_focus_in_event(GtkWidget *widget, GdkEventFocus *event, gpointer data)
   ddisp = (DDisplay *)data;
 
   GTK_WIDGET_SET_FLAGS(widget, GTK_HAS_FOCUS);
-      /* FIXME?: gtk_widget_draw_focus(widget); */
+
   gtk_im_context_focus_in(GTK_IM_CONTEXT(ddisp->im_context));
   
   return FALSE;
@@ -308,6 +352,10 @@ ddisplay_size_allocate (GtkWidget *widget,
   g_return_if_fail (allocation != NULL);
   g_return_if_fail (data != NULL);
 
+#if 0
+  g_print ("ddisp::size_allocate: %d,%d -> %d,%d\n", allocation->width, allocation->height,
+	   widget->allocation.width, widget->allocation.height);
+#endif
   widget->allocation = *allocation;
   ddisp = (DDisplay *)data;
 }
@@ -317,16 +365,16 @@ ddisplay_popup_menu(DDisplay *ddisp, GdkEventButton *event)
 {
   GtkWidget *menu;
 
-  popup_shell = ddisp->shell;
-  menus_get_image_menu(&menu, NULL);
+  menu = menus_get_display_popup();
 
   gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
 		 event->button, event->time);
 }
-
-static void handle_key_event(DDisplay *ddisp, Focus *focus, guint keysym,
-                             const gchar *str, int strlen) {
-  Object *obj = focus->obj;
+static void 
+handle_key_event(DDisplay *ddisp, Focus *focus, guint keysym,
+                 const gchar *str, int strlen) 
+{
+  DiaObject *obj = focus_get_object(focus);
   Point p = obj->position;
   ObjectChange *obj_change = NULL;
   gboolean modified;
@@ -344,19 +392,21 @@ static void handle_key_event(DDisplay *ddisp, Focus *focus, guint keysym,
   object_add_updates(obj, ddisp->diagram);
 
   if (modified) {
-    diagram_modified(ddisp->diagram);
     if (obj_change != NULL) {
       undo_object_change(ddisp->diagram, obj, obj_change);
       undo_set_transactionpoint(ddisp->diagram->undo);
     }
+    diagram_modified(ddisp->diagram);
   }
 
   diagram_flush(ddisp->diagram);
 }
   
 
-void ddisplay_im_context_commit(GtkIMContext *context, const gchar  *str,
-                                DDisplay     *ddisp) {
+void 
+ddisplay_im_context_commit(GtkIMContext *context, const gchar  *str,
+                           DDisplay     *ddisp) 
+{
       /* When using IM, we'll not get many key events past the IM filter,
          mostly IM Commits.
 
@@ -364,26 +414,113 @@ void ddisplay_im_context_commit(GtkIMContext *context, const gchar  *str,
          (the default IM on X should perform the local->UTF8 conversion)
       */
       
-  handle_key_event(ddisp, active_focus(), 0, str, g_utf8_strlen(str,-1));
+  Focus *focus = get_active_focus((DiagramData *) ddisp->diagram);
+
+  ddisplay_im_context_preedit_reset(ddisp, focus);
+  
+  if (focus != NULL)
+    handle_key_event(ddisp, focus, 0, str, g_utf8_strlen(str,-1));
 }
 
-void ddisplay_im_context_preedit_changed(GtkIMContext *context,
-                                         DDisplay *ddisp) {
-/*  char *str;
-  PangoAttrList *attrs;
+void 
+ddisplay_im_context_preedit_changed(GtkIMContext *context,
+                                    DDisplay *ddisp) 
+{
   gint cursor_pos;
-    
-  gtk_im_context_get_preedit_string(ddisp->im_context,
-                                    &str,&attrs,&cursor_pos);
+  Focus *focus = get_active_focus((DiagramData *) ddisp->diagram);
 
-  g_message("received a 'preedit changed'; str=%s cursor_pos=%i",
-            str,cursor_pos);
-
-  g_free(str);
-  pango_attr_list_unref(attrs);
-*/
+  ddisplay_im_context_preedit_reset(ddisp, focus);
+  
+  gtk_im_context_get_preedit_string(context, &ddisp->preedit_string,
+                                    &ddisp->preedit_attrs, &cursor_pos);
+  if (ddisp->preedit_string != NULL) {
+    if (focus != NULL) {
+      handle_key_event(ddisp, focus, 0, ddisp->preedit_string,
+                       g_utf8_strlen(ddisp->preedit_string,-1));
+    } else {
+      ddisplay_im_context_preedit_reset(ddisp, focus);
+    }
+  }
 }
 
+static void
+_scroll_page (DDisplay *ddisp, Direction dir)
+{
+  Point delta = {0, 0};
+
+  switch (dir) {
+  case DIR_LEFT :
+    delta.x = ddisp->diagram->data->paper.width * ddisp->diagram->data->paper.scaling;
+    break;
+  case DIR_RIGHT :
+    delta.x = -ddisp->diagram->data->paper.width * ddisp->diagram->data->paper.scaling;
+    break;
+  case DIR_UP :
+    delta.y = -ddisp->diagram->data->paper.height * ddisp->diagram->data->paper.scaling;
+    break;
+  case DIR_DOWN :
+    delta.y = ddisp->diagram->data->paper.height * ddisp->diagram->data->paper.scaling;
+    break;
+  }
+  ddisplay_scroll(ddisp, &delta);
+  ddisplay_flush(ddisp);
+}
+
+static void
+_scroll_step (DDisplay *ddisp, guint keyval)
+{
+  switch (keyval) {
+  case GDK_Up :
+    ddisplay_scroll_up(ddisp);
+    ddisplay_flush(ddisp);
+    break;
+  case GDK_Down:
+    ddisplay_scroll_down(ddisp);
+    ddisplay_flush(ddisp);
+    break;
+  case GDK_Left:
+    ddisplay_scroll_left(ddisp);
+    ddisplay_flush(ddisp);
+    break;
+  case GDK_Right:
+    ddisplay_scroll_right(ddisp);
+    ddisplay_flush(ddisp);
+    break;
+  default :
+    g_assert_not_reached  ();
+  }
+}
+
+/** Cleanup/Remove Timeout Handler for Button Press and Hold
+ */
+static void 
+hold_remove_handler(void) 
+{
+  if (hold_data.tag != 0) {
+    g_source_remove(hold_data.tag);
+    hold_data.tag = 0;
+    gdk_event_free(hold_data.event);
+  }
+}
+
+/** Timeout Handler for Button Press and Hold
+ * If this function is called, then the button must still be down,
+ * indicating that the user has pressed and held the button, but not moved
+ * the pointer (mouse).
+ * Dynamic data is cleaned up in ddisplay_canvas_events
+ */
+static gboolean 
+hold_timeout_handler(gpointer data) 
+{
+  if (active_tool->button_hold_func)
+    (*active_tool->button_hold_func) (active_tool, (GdkEventButton *)(hold_data.event), hold_data.ddisp);
+  hold_remove_handler();
+  return FALSE;
+}
+
+
+/** Main input handler for a diagram canvas.
+ */
 gint
 ddisplay_canvas_events (GtkWidget *canvas,
 			GdkEvent  *event,
@@ -398,14 +535,16 @@ ddisplay_canvas_events (GtkWidget *canvas,
   GdkModifierType tmask;
   guint state = 0;
   Focus *focus;
-  Object *obj;
+  DiaObject *obj;
   Rectangle *visible;
   Point middle;
   int return_val;
   int key_handled;
   int width, height;
   int new_size;
-
+  int im_context_used;
+  static gboolean moving = FALSE;
+  
   return_val = FALSE;
  
   if (!canvas->window) 
@@ -431,7 +570,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
               if (sevent->state & GDK_SHIFT_MASK)
                   ddisplay_scroll_left(ddisp);
               else if (sevent->state & GDK_CONTROL_MASK) {
-                  ddisplay_untransform_coords(ddisp, sevent->x, sevent->y, &middle.x, &middle.y);
+                  ddisplay_untransform_coords(ddisp, (int)sevent->x, (int)sevent->y, &middle.x, &middle.y);
                   ddisplay_zoom(ddisp, &middle, 2);
               }
               else 
@@ -441,7 +580,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
               if (sevent->state & GDK_SHIFT_MASK)
                   ddisplay_scroll_right(ddisp);
               else if (sevent->state & GDK_CONTROL_MASK) { 
-                    ddisplay_untransform_coords(ddisp, sevent->x, sevent->y, &middle.x, &middle.y);
+                    ddisplay_untransform_coords(ddisp, (int)sevent->x, (int)sevent->y, &middle.x, &middle.y);
                     ddisplay_zoom(ddisp, &middle, 0.5);
               }
               else
@@ -474,11 +613,16 @@ ddisplay_canvas_events (GtkWidget *canvas,
                                  ddisp->canvas->allocation.height);
           ddisplay_update_scrollbars(ddisp);
         }
-        display_set_active(ddisp);
+	/* If the UI is not integrated, resizing should set the resized
+	 * window as active.  With integrated UI, there is only one window.
+	 */
+        if (is_integrated_ui () == 0)
+          display_set_active(ddisp);
         break;
 
       case GDK_FOCUS_CHANGE: {
 	GdkEventFocus *focus = (GdkEventFocus*)event;
+	hold_remove_handler();
 	if (focus->in) {
 	  display_set_active(ddisp);
 	  ddisplay_do_update_menu_sensitivity(ddisp);
@@ -487,6 +631,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
       }
       case GDK_2BUTTON_PRESS:
         display_set_active(ddisp);
+	hold_remove_handler();
         bevent = (GdkEventButton *) event;
         state = bevent->state;
 
@@ -495,7 +640,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
             case 1:
               if (transient_tool)
                 break;
-              if (*active_tool->double_click_func)
+              if (active_tool->double_click_func)
                 (*active_tool->double_click_func) (active_tool, bevent, ddisp);
               break;
 
@@ -526,13 +671,21 @@ ddisplay_canvas_events (GtkWidget *canvas,
               if (transient_tool)
                 break;
                   /* get the focus again, may be lost by zoom combo */
+	      moving = TRUE;
               gtk_widget_grab_focus(canvas);
-              if (*active_tool->button_press_func)
+              if (active_tool->button_press_func)
                 (*active_tool->button_press_func) (active_tool, bevent, ddisp);
-              break;
 
+			  /* Detect user holding down the button.
+			   * Set timeout for 1sec. If timeout is called, user must still
+			   * be holding. If user releases button, remove timeout. */
+			  hold_data.event = gdk_event_copy(event); /* need to free later */
+			  hold_data.ddisp = ddisp;
+			  hold_data.tag = g_timeout_add(1000, hold_timeout_handler, NULL);
+
+              break;
             case 2:
-              if (ddisp->menu_bar == NULL) {
+              if (ddisp->menu_bar == NULL && !is_integrated_ui()) {
                 popup_object_menu(ddisp, bevent);
               }
 	      else if (!transient_tool) {
@@ -546,7 +699,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
               if (transient_tool)
                 break;
               if (ddisp->menu_bar == NULL) {
-                if (bevent->state & GDK_CONTROL_MASK) {
+                if (bevent->state & GDK_CONTROL_MASK || is_integrated_ui ()) {
                       /* for two button mouse users ... */
                   popup_object_menu(ddisp, bevent);
                   break;
@@ -571,9 +724,13 @@ ddisplay_canvas_events (GtkWidget *canvas,
         switch (bevent->button)
         {
             case 1:
-              if (*active_tool->button_release_func)
+	      if (moving)
+  		moving = FALSE;		      
+              if (active_tool->button_release_func)
                 (*active_tool->button_release_func) (active_tool,
                                                      bevent, ddisp);
+	      /* Button Press and Hold - remove handler then deallocate memory */
+	      hold_remove_handler();
               break;
 
             case 2:
@@ -597,6 +754,7 @@ ddisplay_canvas_events (GtkWidget *canvas,
       case GDK_MOTION_NOTIFY:
 	/*  get the pointer position  */
 	gdk_window_get_pointer (canvas->window, &tx, &ty, &tmask);
+	hold_remove_handler();
 
         mevent = (GdkEventMotion *) event;
         state = mevent->state;
@@ -609,58 +767,108 @@ ddisplay_canvas_events (GtkWidget *canvas,
         }
         if (transient_tool && (*transient_tool->motion_func)) 
           (*transient_tool->motion_func) (transient_tool, mevent, ddisp);
-        else if (*active_tool->motion_func)
+        else if (active_tool->motion_func)
           (*active_tool->motion_func) (active_tool, mevent, ddisp);
         break;
 
       case GDK_KEY_PRESS:
+	if (moving)	/*Disable Keyboard accels whilst draggin an object*/ 
+		break;
         display_set_active(ddisp);
         kevent = (GdkEventKey *)event;
         state = kevent->state;
         key_handled = FALSE;
+        im_context_used = FALSE;
+#if 0
+	printf("Key input %d in state %d\n", kevent->keyval, textedit_mode(ddisp));
+#endif
+        focus = get_active_focus((DiagramData *) ddisp->diagram);
+        if (focus != NULL) {
+	  /* Keys goes to the active focus. */
+	  obj = focus_get_object(focus);
+	  if (diagram_is_selected(ddisp->diagram, obj)) {
 
-        focus = active_focus();
-        if ((focus != NULL) &&
-            !(state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)) ) {
-              /* Keys goes to the active focus. */
-          obj = focus->obj;
-          if (diagram_is_selected(ddisp->diagram, obj)) {
-
-            if (!gtk_im_context_filter_keypress(
-                  GTK_IM_CONTEXT(ddisp->im_context), kevent)) {
-              
-                  /*! key event not swallowed by the input method ? */
-              handle_key_event(ddisp, focus, kevent->keyval,
-                               kevent->string, kevent->length);
-
-              diagram_flush(ddisp->diagram);
-            }
-          }
-          return_val = key_handled = TRUE;
-        }
+	    if (!gtk_im_context_filter_keypress(
+		  GTK_IM_CONTEXT(ddisp->im_context), kevent)) {
+	      
+	      switch (kevent->keyval) {
+		  case GDK_Tab:
+		    focus = textedit_move_focus(ddisp, focus,
+						(state & GDK_SHIFT_MASK) == 0);
+		    obj = focus_get_object(focus);
+		    break;
+		  case GDK_Escape:
+		    textedit_deactivate_focus();
+		    tool_reset ();
+		    break;
+		  default:
+		    /*! key event not swallowed by the input method ? */
+		    handle_key_event(ddisp, focus, kevent->keyval,
+				     kevent->string, kevent->length);
+		    
+		    diagram_flush(ddisp->diagram);
+	      }
+	    }
+	    return_val = key_handled = im_context_used = TRUE;
+	  }
+	}
 
+#if 0 /* modifier requirment added 2004-07-17, IMO reenabling unmodified keys here
+               * shouldn't break im_context handling. How to test?    --hb
+               */
+        if (!key_handled && (state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))) {
+#else            
         if (!key_handled) {
-              /* No focus to receive keys, take care of it ourselves. */
+#endif
+              /* IM doesn't need receive keys, take care of it ourselves. */
           return_val = TRUE;
-          gtk_im_context_reset(GTK_IM_CONTEXT(ddisp->im_context));
           
           switch(kevent->keyval) {
+	      case GDK_Home :
+	        /* match upper left corner of the diagram with it's view */
+		ddisplay_set_origo(ddisp, ddisp->diagram->data->extents.left, ddisp->diagram->data->extents.top);
+		ddisplay_update_scrollbars(ddisp);
+		ddisplay_add_update_all(ddisp);
+	        break;
+	      case GDK_End :
+	        /* match lower right corner of the diagram with it's view */
+		visible = &ddisp->visible;
+		ddisplay_set_origo(ddisp, 
+		                   ddisp->diagram->data->extents.right - (visible->right - visible->left), 
+				   ddisp->diagram->data->extents.bottom - (visible->bottom - visible->top));
+		ddisplay_update_scrollbars(ddisp);
+		ddisplay_add_update_all(ddisp);
+	        break;
+	      case GDK_Page_Up :
+	        _scroll_page (ddisp, !(state & GDK_CONTROL_MASK) ? DIR_UP : DIR_LEFT);
+	        break;
+	      case GDK_Page_Down :
+	        _scroll_page (ddisp, !(state & GDK_CONTROL_MASK) ? DIR_DOWN : DIR_RIGHT);
+	        break;
               case GDK_Up:
-                ddisplay_scroll_up(ddisp);
-                ddisplay_flush(ddisp);
-                break;
               case GDK_Down:
-                ddisplay_scroll_down(ddisp);
-                ddisplay_flush(ddisp);
-                break;
               case GDK_Left:
-                ddisplay_scroll_left(ddisp);
-                ddisplay_flush(ddisp);
-                break;
               case GDK_Right:
-                ddisplay_scroll_right(ddisp);
-                ddisplay_flush(ddisp);
-                break;
+	        if (g_list_length (ddisp->diagram->data->selected) > 0) {
+		  Diagram *dia = ddisp->diagram;
+		  GList *objects = dia->data->selected;
+		  Direction dir = GDK_Up == kevent->keyval ? DIR_UP :
+				  GDK_Down == kevent->keyval ? DIR_DOWN :
+				  GDK_Right == kevent->keyval ? DIR_RIGHT : DIR_LEFT;
+		  object_add_updates_list(objects, dia);
+		  object_list_nudge(objects, dia, dir, 
+				    /* step one pixel or more with <ctrl> */
+				    ddisplay_untransform_length (ddisp, (state & GDK_SHIFT_MASK) ? 10 : 1));
+		  diagram_update_connections_selection(dia);
+		  object_add_updates_list(objects, dia);
+		  diagram_modified(dia);
+		  diagram_flush(dia);     
+
+		  undo_set_transactionpoint(dia->undo);
+		} else {
+		  _scroll_step (ddisp, kevent->keyval);
+		}
+		break;
               case GDK_KP_Add:
               case GDK_plus:
                 visible = &ddisp->visible;
@@ -682,14 +890,30 @@ ddisplay_canvas_events (GtkWidget *canvas,
                 if (active_tool->type == MAGNIFY_TOOL)
                   set_zoom_out(active_tool);
                 break;
+              case GDK_Escape:
+                view_unfullscreen();
+                break;
+	      case GDK_F2:
+	      case GDK_Return:
+		gtk_action_activate (menus_get_action ("ToolsTextedit"));
+		break;
               default:
-                if (kevent->string && 0 == strcmp(" ",kevent->string)) {
+                if (kevent->string && kevent->keyval == ' ') {
                   tool_select_former();
-                } else { 
+                } else if ((kevent->state & (GDK_MOD1_MASK|GDK_CONTROL_MASK)) == 0 && 
+			   kevent->length != 0) {
+		  /* Find first editable */
+#ifdef NEW_TEXT_EDIT
+		  modify_edit_first_text(ddisp);
                   return_val = FALSE;
+#endif
                 }
           }
         }
+
+        if (!im_context_used)
+          gtk_im_context_reset(GTK_IM_CONTEXT(ddisp->im_context));
+        
         break;
 
       case GDK_KEY_RELEASE:
@@ -756,32 +980,18 @@ ddisplay_destroy (GtkWidget *widget, gpointer data)
   
   ddisp = (DDisplay *) data;
 
-  if (popup_shell == ddisp->shell) {
-    popup_shell = NULL;
-  }
-
   ddisplay_really_destroy(ddisp);
 }
 
-inline int 
-round_up (double x)
-{
-  if (x - (int) x > 0.001)
-    return (int) x + 1;
-  else
-    return (int) x ;
-}
-
-
 /* returns NULL if object cannot be created */
-Object *
-ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
+DiaObject *
+ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, DiaObjectType *otype,
 		     gpointer user_data)
 {
   Point droppoint;
   Point droppoint_orig;
   Handle *handle1, *handle2;
-  Object *obj, *p_obj;
+  DiaObject *obj, *p_obj;
   GList *list;
   real click_distance;
 
@@ -796,6 +1006,8 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
                                    user_data,
                                    &handle1, &handle2);
 
+  if (!obj)
+    return NULL;
 
   click_distance = ddisplay_untransform_length(ddisp, 3.0);
 
@@ -806,9 +1018,10 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
   p_obj = diagram_find_clicked_object(ddisp->diagram, &droppoint_orig,
 				    click_distance);
 
-  if (p_obj && p_obj->can_parent) /* the tool was dropped inside an object that takes children*/
+  if (p_obj && object_flags_set(p_obj, DIA_OBJECT_CAN_PARENT))
+    /* the tool was dropped inside an object that takes children*/
   {
-    Rectangle *p_ext, *c_ext;
+    Rectangle p_ext, c_ext;
     real parent_height, child_height, parent_width, child_width;
     real vadjust = 0.0, hadjust = 0.0;
     Point new_pos;
@@ -821,36 +1034,33 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
      * since we don't have the 'rendered bbox'.
      * -Lars
      */
-    p_ext = parent_handle_extents(p_obj);
-    c_ext = parent_handle_extents(obj);
+    parent_handle_extents(p_obj, &p_ext);
+    parent_handle_extents(obj, &c_ext);
 
-    parent_height = p_ext->bottom - p_ext->top;
-    child_height = c_ext->bottom - c_ext->top;
+    parent_height = p_ext.bottom - p_ext.top;
+    child_height = c_ext.bottom - c_ext.top;
 
-    parent_width = p_ext->right - p_ext->left;
-    child_width = c_ext->right - c_ext->left;
+    parent_width = p_ext.right - p_ext.left;
+    child_width = c_ext.right - c_ext.left;
 
     /* we need the pre-snap position, but must remember that handles can
      * be to the left of the droppoint */
-    c_ext->left = droppoint_orig.x - (obj->position.x - c_ext->left);
-    c_ext->top  = droppoint_orig.y - (obj->position.y - c_ext->top);
-    c_ext->right = c_ext->left + child_width;
-    c_ext->bottom = c_ext->top + child_height;
+    c_ext.left = droppoint_orig.x - (obj->position.x - c_ext.left);
+    c_ext.top  = droppoint_orig.y - (obj->position.y - c_ext.top);
+    c_ext.right = c_ext.left + child_width;
+    c_ext.bottom = c_ext.top + child_height;
 
-    if (c_ext->left < p_ext->left) {
-      hadjust = p_ext->left - c_ext->left;
-    } else if (c_ext->right > p_ext->right) {
-      hadjust = p_ext->right - c_ext->right;
+    if (c_ext.left < p_ext.left) {
+      hadjust = p_ext.left - c_ext.left;
+    } else if (c_ext.right > p_ext.right) {
+      hadjust = p_ext.right - c_ext.right;
     }
-    if (c_ext->top < p_ext->top) {
-      vadjust = p_ext->top - c_ext->top;
-    } else if (c_ext->bottom > p_ext->bottom) {
-      vadjust = p_ext->bottom - c_ext->bottom;
+    if (c_ext.top < p_ext.top) {
+      vadjust = p_ext.top - c_ext.top;
+    } else if (c_ext.bottom > p_ext.bottom) {
+      vadjust = p_ext.bottom - c_ext.bottom;
     }
 
-    g_free(p_ext);
-    g_free(c_ext);
-
     if (child_width > parent_width ||
 	child_height > parent_height) {
       message_error(_("The object you dropped cannot fit into its parent. \nEither expand the parent object, or drop the object elsewhere."));
@@ -866,16 +1076,16 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
     }
   }
 
-
   diagram_add_object(ddisp->diagram, obj);
   diagram_remove_all_selected(ddisp->diagram, TRUE); /* unselect all */
   diagram_select(ddisp->diagram, obj);
   obj->ops->selectf(obj, &droppoint, ddisp->renderer);
+  textedit_activate_object(ddisp, obj, NULL);
 
   /* Connect first handle if possible: */
   if ((handle1 != NULL) &&
       (handle1->connect_type != HANDLE_NONCONNECTABLE)) {
-    object_connect_display(ddisp, obj, handle1);
+    object_connect_display(ddisp, obj, handle1, FALSE);
   }
   object_add_updates(obj, ddisp->diagram);
   ddisplay_do_update_menu_sensitivity(ddisp);
@@ -884,9 +1094,9 @@ ddisplay_drop_object(DDisplay *ddisp, gint x, gint y, ObjectType *otype,
   list = g_list_prepend(NULL, obj);
   undo_insert_objects(ddisp->diagram, list, 1);
   diagram_update_extents(ddisp->diagram);
-  diagram_modified(ddisp->diagram);
 
   undo_set_transactionpoint(ddisp->diagram->undo);
+  diagram_modified(ddisp->diagram);
   if (prefs.reset_tools_after_create)
     tool_reset();
   return obj;
