@@ -23,12 +23,14 @@
 #include <string.h>
 #include <stdio.h>
 #include <math.h>
-#include <gdk/gdkkeysyms.h>
 
 #ifdef GNOME
+#undef GTK_DISABLE_DEPRECATED
 #include <gnome.h>
 #endif
 
+#include <gdk/gdkkeysyms.h>
+
 #include "intl.h"
 
 #include "display.h"
@@ -48,7 +50,7 @@
 #include "load_save.h"
 #include "dia-props.h"
 #include "render_gdk.h"
-#include "render_libart.h"
+#include "diatransform.h"
 
 static GHashTable *display_ht = NULL;
 static GdkCursor *current_cursor = NULL;
@@ -68,38 +70,26 @@ static guint display_hash(DDisplay *ddisp);
 static void
 update_zoom_status(DDisplay *ddisp)
 {
-  GtkWidget *zoomcombo;
   gchar* zoom_text;
 
-  zoomcombo = ddisp->zoom_status;
-  zoom_text = g_strdup_printf("%.1f%%",
-	   ddisp->zoom_factor * 100.0 / DDISPLAY_NORMAL_ZOOM);
-  gtk_entry_set_text(GTK_ENTRY(gtk_object_get_user_data(GTK_OBJECT(zoomcombo))),
-		     zoom_text);
-  g_free(zoom_text); /* Copied by gtk_entry_set_text */
-}
-
-static void
-update_modified_status(DDisplay *ddisp)
-{
-  GtkStatusbar *statusbar;
-  guint context_id;
-
-  if (diagram_is_modified(ddisp->diagram))
+  if (is_integrated_ui ())
   {
-    statusbar = GTK_STATUSBAR (ddisp->modified_status);
-    context_id = gtk_statusbar_get_context_id (statusbar, "Changed");
-  
-    gtk_statusbar_pop (statusbar, context_id); 
-    gtk_statusbar_push (statusbar, context_id, _("Diagram modified!"));
+    zoom_text = g_strdup_printf("%.0f%%",
+	   ddisp->zoom_factor * 100.0 / DDISPLAY_NORMAL_ZOOM);
+
+    integrated_ui_toolbar_set_zoom_text (ddisp->common_toolbar, zoom_text);  
   }
   else
   {
-    statusbar = GTK_STATUSBAR (ddisp->modified_status);
-    context_id = gtk_statusbar_get_context_id (statusbar, "Changed");
-  
-    gtk_statusbar_pop (statusbar, context_id); 
+    GtkWidget *zoomcombo;
+    zoom_text = g_strdup_printf("%.1f%%",
+	     ddisp->zoom_factor * 100.0 / DDISPLAY_NORMAL_ZOOM);
+    zoomcombo = ddisp->zoom_status;
+    gtk_entry_set_text(GTK_ENTRY (g_object_get_data (G_OBJECT(zoomcombo), "user_data")),
+		       zoom_text);
   }
+
+  g_free(zoom_text); /* Copied by gtk_entry_set_text */
 }
 
 static void
@@ -108,6 +98,10 @@ selection_changed (Diagram* dia, int n, DDisplay* ddisp)
   GtkStatusbar *statusbar;
   guint context_id;
 
+  /* nothing to do if there is no display with the diagram anymore */
+  if (g_slist_length(dia->displays) < 1)
+    return;
+
   statusbar = GTK_STATUSBAR (ddisp->modified_status);
   context_id = gtk_statusbar_get_context_id (statusbar, "Selection");
 
@@ -115,7 +109,10 @@ selection_changed (Diagram* dia, int n, DDisplay* ddisp)
   {
     gchar *msg;
 
-    msg = g_strdup_printf (_("Selection of %d objects"), n);
+    /* http://www.gnu.org/software/gettext/manual/html_chapter/gettext_10.html#SEC150 
+     * Althoug the single objects wont get triggered here some languages have variations on the other numbers 
+     */
+    msg = g_strdup_printf (ngettext ("Selection of %d object", "Selection of %d objects", n), n);
     gtk_statusbar_pop (statusbar, context_id);
     gtk_statusbar_push (statusbar, context_id, msg);
     g_free (msg);
@@ -137,21 +134,34 @@ selection_changed (Diagram* dia, int n, DDisplay* ddisp)
   {
     gtk_statusbar_pop (statusbar, context_id); 
   }
+  /* selection-changed signal can also be emitted from outside of the dia core */
+  ddisplay_do_update_menu_sensitivity (ddisp);
 }
 
 static void
-append_im_menu (DDisplay* ddisp, GtkMenuItem* im_menu_item)
+append_im_menu (DDisplay* ddisp, GtkAction* action)
 {
-  GtkWidget* im_menu;
-  GtkWidget* im_menu_tearoff;
+  GSList    *proxies;
+  GtkWidget *im_menu;
+  /* GtkWidget *im_menu_tearoff; */
   
-  im_menu = gtk_menu_new ();
-  im_menu_tearoff = gtk_tearoff_menu_item_new ();
-  gtk_menu_shell_append (GTK_MENU_SHELL(im_menu), im_menu_tearoff);
-  gtk_im_multicontext_append_menuitems (
-      GTK_IM_MULTICONTEXT(ddisp->im_context),
-      GTK_MENU_SHELL(im_menu));
-  gtk_menu_item_set_submenu (GTK_MENU_ITEM(im_menu_item), im_menu);
+  proxies = gtk_action_get_proxies (action);
+  while (proxies) {
+    if (GTK_IS_MENU_ITEM (proxies->data)) {
+      im_menu = gtk_menu_new ();
+      /* tearoff should be added depending on gtk settings
+      im_menu_tearoff = gtk_tearoff_menu_item_new ();
+      gtk_menu_shell_append (GTK_MENU_SHELL(im_menu), im_menu_tearoff);
+      */
+      gtk_im_multicontext_append_menuitems (
+        GTK_IM_MULTICONTEXT(ddisp->im_context),
+        GTK_MENU_SHELL(im_menu));
+      gtk_menu_item_set_submenu (GTK_MENU_ITEM(proxies->data), im_menu);
+      gtk_widget_show (GTK_WIDGET (proxies->data));
+      gtk_widget_show (GTK_WIDGET (im_menu));
+    }
+    proxies = proxies->next;
+  }
 }
 
 /** Initialize the various GTK-level thinks in a display after the internal
@@ -161,9 +171,7 @@ append_im_menu (DDisplay* ddisp, GtkMenuItem* im_menu_item)
 static void
 initialize_display_widgets(DDisplay *ddisp)
 {
-  GtkMenuItem* im_menu_item;
-  GtkWidget* im_menu;
-  GtkWidget* im_menu_tearoff;
+  GtkAction* im_menu_item;
   static gboolean input_methods_done = FALSE;
   Diagram *dia = ddisp->diagram;
   gchar *filename;
@@ -187,7 +195,7 @@ initialize_display_widgets(DDisplay *ddisp)
   }
   create_display_shell(ddisp, prefs.new_view.width, prefs.new_view.height,
 		       filename, prefs.new_view.use_menu_bar, !app_is_embedded());
-  
+
   ddisplay_update_statusbar (ddisp);
 
   ddisplay_set_origo(ddisp, ddisp->visible.left, ddisp->visible.top);
@@ -204,17 +212,16 @@ initialize_display_widgets(DDisplay *ddisp)
   g_hash_table_insert (display_ht, ddisp->canvas, ddisp);
 
   if (!input_methods_done) {
-      im_menu_item = menus_get_item_from_path("<Display>/Input Methods", NULL);
-      if (im_menu_item) {
-        append_im_menu (ddisp, im_menu_item);
-	input_methods_done = TRUE;
-      }
+      im_menu_item = menus_get_action ("InputMethods");
+      g_assert (im_menu_item);
+      append_im_menu (ddisp, im_menu_item);
+      input_methods_done = TRUE;
   }
   /* the diagram menubar gets recreated for every diagram */ 	 
   if (ddisp->menu_bar) {
-    im_menu_item = menus_get_item_from_path("<DisplayMBar>/Input Methods", ddisp->mbar_item_factory);
-    if (im_menu_item)
-      append_im_menu (ddisp, im_menu_item);
+    im_menu_item = gtk_action_group_get_action (ddisp->actions, "InputMethods");
+    g_assert (im_menu_item);
+    append_im_menu (ddisp, im_menu_item);
   }	 
 }
 
@@ -235,9 +242,6 @@ copy_display(DDisplay *orig_ddisp)
   
   ddisp = g_new0(DDisplay,1);
 
-  /* initialize the whole struct to 0 so that we are sure to catch errors.*/
-  memset (&ddisp->updatable_menu_items, 0, sizeof (UpdatableMenuItems));
-  
   ddisp->diagram = orig_ddisp->diagram;
   /* Every display has its own reference */
   g_object_ref(dia);
@@ -246,6 +250,7 @@ copy_display(DDisplay *orig_ddisp)
 
   ddisp->show_cx_pts = orig_ddisp->show_cx_pts;
 
+  
   ddisp->autoscroll = orig_ddisp->autoscroll;
   ddisp->mainpoint_magnetism = orig_ddisp->mainpoint_magnetism;
 
@@ -273,28 +278,11 @@ copy_display(DDisplay *orig_ddisp)
 DDisplay *
 new_display(Diagram *dia)
 {
-  GtkMenuItem* im_menu_item;
   DDisplay *ddisp;
   Rectangle visible;
-  static gboolean input_methods_done = FALSE;
   
   ddisp = g_new0(DDisplay,1);
 
-  ddisp->menu_bar = NULL;
-  ddisp->mbar_item_factory = NULL;
-
-  ddisp->rulers = NULL;
-  /*
-  ddisp->visible_grid = NULL;
-  ddisp->snap_to_grid = NULL;
-  ddisp->show_cx_pts_mitem = NULL;
-#ifdef HAVE_LIBART
-  ddisp->antialiased = NULL;
-#endif
-  */
-  /* initialize the whole struct to 0 so that we are sure to catch errors.*/
-  memset (&ddisp->updatable_menu_items, 0, sizeof (UpdatableMenuItems));
-  
   ddisp->diagram = dia;
   /* Every display has it's own reference */
   g_object_ref(dia);
@@ -305,9 +293,9 @@ new_display(Diagram *dia)
   ddisp->show_cx_pts = prefs.show_cx_pts;
 
   ddisp->autoscroll = TRUE;
-  ddisp->mainpoint_magnetism = TRUE;
+  ddisp->mainpoint_magnetism = prefs.snap_object;
 
-  ddisp->aa_renderer = 0;
+  ddisp->aa_renderer = prefs.view_antialised;
   
   ddisp->update_areas = NULL;
   ddisp->display_areas = NULL;
@@ -484,8 +472,12 @@ ddisplay_add_update(DDisplay *ddisp, Rectangle *rect)
   Rectangle *r;
   int top,bottom,left,right;
   Rectangle *visible;
-  int width = dia_renderer_get_width_pixels (ddisp->renderer);
-  int height = dia_renderer_get_height_pixels (ddisp->renderer);
+  int width, height;
+
+  if (!ddisp->renderer)
+    return; /* can happen at creation time of the diagram */
+  width = dia_renderer_get_width_pixels (ddisp->renderer);
+  height = dia_renderer_get_height_pixels (ddisp->renderer);
 
   if (!rectangle_intersects(rect, &ddisp->visible))
     return;
@@ -524,6 +516,8 @@ ddisplay_add_display_area(DDisplay *ddisp,
 {
   IRectangle *r;
 
+  if (!ddisp->renderer)
+    return; /* if we don't have a renderer yet prefer ignoring over crashing */
   if (left < 0)
     left = 0;
   if (top < 0)
@@ -618,8 +612,7 @@ ddisplay_flush(DDisplay *ddisp)
 {
   /* if no update is queued, queue update */
   if (!ddisp->update_id)
-    ddisp->update_id = gtk_idle_add((GtkFunction) ddisplay_update_handler,
-				    ddisp);
+    ddisp->update_id = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, (GSourceFunc)ddisplay_update_handler, ddisp, NULL);
 }
 
 static void
@@ -793,19 +786,19 @@ ddisplay_zoom(DDisplay *ddisp, Point *point, real magnify)
 void
 ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap)
 {
-  GtkCheckMenuItem *snap_to_grid;
+  GtkToggleAction *snap_to_grid;
   ddisp->grid.snap = snap;
 
-  if (ddisp->menu_bar == NULL) {
-    snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Snap To Grid", NULL));
-  } else {
-    snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Snap To Grid", ddisp->mbar_item_factory));
-  }
-
+    if (ddisp->menu_bar == NULL) {
+      snap_to_grid = GTK_TOGGLE_ACTION (menus_get_action ("ViewSnaptogrid"));
+      if (is_integrated_ui ())
+         integrated_ui_toolbar_grid_snap_synchronize_to_display (ddisp);
+    } else {
+      snap_to_grid = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewSnaptogrid"));
+    }
   /* Currently, this can cause double emit, but that's a small problem.
    */
-  gtk_check_menu_item_set_active(snap_to_grid,
-				 ddisp->grid.snap);
+  gtk_toggle_action_set_active (snap_to_grid, ddisp->grid.snap);
   ddisplay_update_statusbar(ddisp);
 }
 
@@ -813,8 +806,11 @@ ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap)
 static void
 update_snap_grid_status(DDisplay *ddisp)
 {
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ddisp->grid_status),
-			       ddisp->grid.snap);
+  if (ddisp->grid_status)
+  {
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ddisp->grid_status),
+                                  ddisp->grid.snap);
+  }
 }
 
 /** Set the display's mainpoint magnetism setting, updating menu and button
@@ -822,19 +818,20 @@ update_snap_grid_status(DDisplay *ddisp)
 void
 ddisplay_set_snap_to_objects(DDisplay *ddisp, gboolean magnetic)
 {
-  GtkCheckMenuItem *mainpoint_magnetism;
+  GtkToggleAction *mainpoint_magnetism;
   ddisp->mainpoint_magnetism = magnetic;
 
   if (ddisp->menu_bar == NULL) {
-    mainpoint_magnetism = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Snap To Objects", NULL));
+    mainpoint_magnetism = GTK_TOGGLE_ACTION (menus_get_action ("ViewSnaptoobjects"));
+    if (is_integrated_ui ())
+      integrated_ui_toolbar_object_snap_synchronize_to_display (ddisp);
   } else {
-    mainpoint_magnetism = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Snap To Objects", ddisp->mbar_item_factory));
+    mainpoint_magnetism = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewSnaptoobjects"));
   }
 
   /* Currently, this can cause double emit, but that's a small problem.
    */
-  gtk_check_menu_item_set_active(mainpoint_magnetism,
-				 ddisp->mainpoint_magnetism);
+  gtk_toggle_action_set_active (mainpoint_magnetism, ddisp->mainpoint_magnetism);
   ddisplay_update_statusbar(ddisp);
 }
 
@@ -842,8 +839,11 @@ ddisplay_set_snap_to_objects(DDisplay *ddisp, gboolean magnetic)
 static void
 update_mainpoint_status(DDisplay *ddisp)
 {
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ddisp->mainpoint_status),
-			       ddisp->mainpoint_magnetism);
+  if (ddisp->mainpoint_status)
+  {
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ddisp->mainpoint_status),
+                                 ddisp->mainpoint_magnetism);
+  }
 }
 
 /** Scroll display to where point x,y (window coords) is visible */
@@ -1001,14 +1001,89 @@ ddisplay_scroll_to_object(DDisplay *ddisp, DiaObject *obj)
   p.x = (r.left+r.right)/2;
   p.y = (r.top+r.bottom)/2;
 
+  display_set_active(ddisp);
   return ddisplay_scroll_center_point(ddisp, &p);
 }
 
+/** Ensure the object is visible but minimize scrolling
+ */
+gboolean
+ddisplay_present_object(DDisplay *ddisp, DiaObject *obj)
+{
+  const Rectangle *r = dia_object_get_enclosing_box(obj);
+  const Rectangle *v = &ddisp->visible;
+
+  display_set_active(ddisp);
+  if  (!rectangle_in_rectangle(v, r)) {
+    Point delta = { 0, 0 };
+
+    if ((r->right - r->left) > (v->right - v->left)) /* object bigger than visible area */
+      delta.x = (r->left - v->left + r->right - v->right) / 2;
+    else if (r->left < v->left)
+      delta.x = r->left - v->left;
+    else if  (r->right > v->right)
+      delta.x = r->right - v->right;
+
+    if ((r->bottom - r->top) > (v->bottom - v->top)) /* object bigger than visible area */
+      delta.y = (r->top - v->top + r->bottom - v->bottom) / 2;
+    else if (r->top < v->top)
+      delta.y = r->top - v->top;
+    else if  (r->bottom > v->bottom)
+      delta.y = r->bottom - v->bottom;
+
+    ddisplay_scroll(ddisp, &delta);
+    return TRUE;
+  }
+  return FALSE;
+}
+
+/**
+ * Kind of dirty way to init an antialiased renderer, there should be some plug-in interface to do this.
+ * Now with the Libart renderer being a plug-in and the cairo renderer having issues with highlighting
+ * (  http://bugzilla.gnome.org/show_bug.cgi?id=576548 ) it seems reasonable to have default at
+ * Libart, also becuase you loose less when it is switched off ;-)
+ */
+static DiaRenderer *
+new_aa_renderer (DDisplay *ddisp)
+{
+  GType renderer_type;
+
+  renderer_type = g_type_from_name ("DiaLibartRenderer");
+  if (renderer_type) {
+    DiaRenderer *renderer = g_object_new(renderer_type, NULL);
+    g_object_set (renderer,
+                  "transform", dia_transform_new (&ddisp->visible, &ddisp->zoom_factor),
+		  NULL);
+    return renderer;
+  } 
+  
+  renderer_type = g_type_from_name ("DiaCairoInteractiveRenderer");
+  if (renderer_type) {
+    DiaRenderer *renderer = g_object_new(renderer_type, NULL);
+    g_object_set (renderer,
+                  "zoom", &ddisp->zoom_factor,
+		  "rect", &ddisp->visible,
+		  NULL);
+    return renderer;
+  }
+  
+  /* we really should not come here but instead disable the menu command earlier */
+  message_warning (_("No anti-aliased renderer found"));
+  /* fallback: built-in libart renderer */
+  return new_gdk_renderer (ddisp); 
+}
+
 void
 ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer)
 {
   int width, height;
 
+  /* dont mix new renderer with old updates */
+  if (ddisp->update_id) {
+    g_source_remove (ddisp->update_id);
+    ddisp->update_id = 0;
+  }
+
   if (ddisp->renderer)
     g_object_unref (ddisp->renderer);
 
@@ -1018,9 +1093,7 @@ ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer)
   height = ddisp->canvas->allocation.height;
 
   if (ddisp->aa_renderer){
-    ddisp->renderer = new_libart_renderer(
-                         dia_transform_new (&ddisp->visible, 
-                                            &ddisp->zoom_factor), 1);
+    ddisp->renderer = new_aa_renderer (ddisp);
   } else {
     ddisp->renderer = new_gdk_renderer(ddisp);
   }
@@ -1034,9 +1107,7 @@ ddisplay_resize_canvas(DDisplay *ddisp,
 {
   if (ddisp->renderer==NULL) {
     if (ddisp->aa_renderer)
-      ddisp->renderer = new_libart_renderer(
-                           dia_transform_new (&ddisp->visible, 
-                                              &ddisp->zoom_factor), 1);
+      ddisp->renderer = new_aa_renderer (ddisp);
     else
       ddisp->renderer = new_gdk_renderer(ddisp);
   }
@@ -1067,20 +1138,20 @@ ddisplay_active_diagram(void)
 static void 
 ddisp_destroy(DDisplay *ddisp)
 {
-  if (ddisp->update_id) {
-    gtk_idle_remove(ddisp->update_id);
-    ddisp->update_id = 0;
-  }
-
   g_signal_handlers_disconnect_by_func (ddisp->diagram, selection_changed, ddisp);
 
   g_object_unref (G_OBJECT (ddisp->im_context));
   ddisp->im_context = NULL;
 
-  ddisplay_im_context_preedit_reset(ddisp, active_focus());
+  ddisplay_im_context_preedit_reset(ddisp, get_active_focus((DiagramData *) ddisp->diagram));
 
   /* This calls ddisplay_really_destroy */
-  gtk_widget_destroy (ddisp->shell);
+  if (ddisp->is_standalone_window)
+    gtk_widget_destroy (ddisp->shell);
+  else {
+    gtk_widget_destroy (ddisp->container);
+    ddisplay_really_destroy (ddisp);
+  }
 }
 
 static void
@@ -1089,19 +1160,22 @@ are_you_sure_close_dialog_respond(GtkWidget *widget, /* the dialog */
                                   gpointer   user_data) /* the display */
 {
   DDisplay *ddisp = (DDisplay *)user_data;
+  gboolean close_ddisp = TRUE;
 
   switch (response_id) {
   case GTK_RESPONSE_YES :  
     /* save changes */
-    diagram_save(ddisp->diagram, ddisp->diagram->filename);
+    if (!diagram_save(ddisp->diagram, ddisp->diagram->filename))
+      close_ddisp = FALSE;
   
-    if (ddisp->update_id) {
-      gtk_idle_remove(ddisp->update_id);
+    if (ddisp->update_id && close_ddisp) {
+      g_source_remove (ddisp->update_id);
       ddisp->update_id = 0;
     }
     /* fall through */
   case GTK_RESPONSE_NO :
-    ddisp_destroy (ddisp);
+    if (close_ddisp)
+      ddisp_destroy (ddisp);
     /* fall through */
   case GTK_RESPONSE_CANCEL :
   case GTK_RESPONSE_NONE :
@@ -1124,7 +1198,7 @@ ddisplay_close(DDisplay *ddisp)
 
   dia = ddisp->diagram;
   
-  if ( (dia->display_count > 1) ||
+  if ( (g_slist_length(dia->displays) > 1) ||
        (!diagram_is_modified(dia)) ) {
     ddisp_destroy(ddisp);
     return;
@@ -1138,8 +1212,7 @@ ddisplay_close(DDisplay *ddisp)
                                   GTK_DIALOG_MODAL,
                                   GTK_MESSAGE_QUESTION,
                                   GTK_BUTTONS_NONE, /* no standard buttons */
-				  _("Closing diagram without saving"),
-				  NULL);
+				  _("Closing diagram without saving"));
   gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
     _("The diagram '%s'\n"
       "has not been saved. Save changes now?"), fname);
@@ -1167,49 +1240,42 @@ ddisplay_close(DDisplay *ddisp)
 void
 display_update_menu_state(DDisplay *ddisp)
 {
-  GtkCheckMenuItem *rulers;
-  GtkCheckMenuItem *visible_grid;
-  GtkCheckMenuItem *snap_to_grid;
-  GtkCheckMenuItem *show_cx_pts;
-#ifdef HAVE_LIBART
-  GtkCheckMenuItem *antialiased;
-#endif
+  GtkToggleAction *rulers;
+  GtkToggleAction *visible_grid;
+  GtkToggleAction *snap_to_grid;
+  GtkToggleAction *show_cx_pts;
+  GtkToggleAction *antialiased;
 
   if (ddisp->menu_bar == NULL) {
-    rulers       = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Show Rulers", NULL));
-    visible_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Show Grid", NULL));
-    snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Snap To Grid", NULL));
-    show_cx_pts  = 
-      GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/Show Connection Points", NULL));
-#ifdef HAVE_LIBART
-    antialiased = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<Display>/View/AntiAliased", NULL));
-#endif
+    rulers       = GTK_TOGGLE_ACTION (menus_get_action ("ViewShowrulers"));
+    visible_grid = GTK_TOGGLE_ACTION (menus_get_action ("ViewShowgrid"));
+    snap_to_grid = GTK_TOGGLE_ACTION (menus_get_action ("ViewSnaptogrid"));
+    show_cx_pts  = GTK_TOGGLE_ACTION (menus_get_action ("ViewShowconnectionpoints"));
+    antialiased  = GTK_TOGGLE_ACTION (menus_get_action ("ViewAntialiased"));
   } else {
-    rulers       = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Show Rulers", ddisp->mbar_item_factory));
-    visible_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Show Grid", ddisp->mbar_item_factory));
-    snap_to_grid = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Snap To Grid", ddisp->mbar_item_factory));
-    show_cx_pts  = 
-      GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/Show Connection Points", ddisp->mbar_item_factory));
-#ifdef HAVE_LIBART
-    antialiased = GTK_CHECK_MENU_ITEM(menus_get_item_from_path("<DisplayMBar>/View/AntiAliased", ddisp->mbar_item_factory));
-#endif
+    rulers       = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewShowrulers"));
+    visible_grid = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewShowgrid"));
+    snap_to_grid = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewSnaptogrid"));
+    show_cx_pts  = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewShowconnectionpoints"));
+
+    antialiased  = GTK_TOGGLE_ACTION (gtk_action_group_get_action (ddisp->actions, "ViewAntialiased"));
   }
+  gtk_action_set_sensitive (menus_get_action ("ViewAntialiased"), 
+		            g_type_from_name ("DiaCairoInteractiveRenderer") != 0 || g_type_from_name ("DiaLibartRenderer") != 0);
 
 
   ddisplay_do_update_menu_sensitivity (ddisp);
-  
-  gtk_check_menu_item_set_active(rulers,
-				 GTK_WIDGET_VISIBLE (ddisp->hrule) ? 1 : 0); 
-  gtk_check_menu_item_set_active(visible_grid,
+
+  gtk_toggle_action_set_active (rulers, display_get_rulers_showing(ddisp));
+  gtk_toggle_action_set_active (visible_grid,
 				 ddisp->grid.visible);
-  gtk_check_menu_item_set_active(snap_to_grid,
+  gtk_toggle_action_set_active (snap_to_grid,
 				 ddisp->grid.snap);
-  gtk_check_menu_item_set_active(show_cx_pts,
+  gtk_toggle_action_set_active (show_cx_pts,
 				 ddisp->show_cx_pts); 
-#ifdef HAVE_LIBART
-  gtk_check_menu_item_set_active(antialiased,
-				 ddisp->aa_renderer);
-#endif 
+
+  gtk_toggle_action_set_active (antialiased,
+				ddisp->aa_renderer);
 }
 
 void 
@@ -1217,13 +1283,11 @@ ddisplay_do_update_menu_sensitivity (DDisplay *ddisp)
 {
     Diagram *dia;
     
-    dia = ddisp->diagram; 
-    if (ddisp->menu_bar) {
-	diagram_update_menubar_sensitivity(dia, &ddisp->updatable_menu_items);
-    }
-    else {
-	diagram_update_popupmenu_sensitivity(dia);
+    if (ddisp == NULL) {
+      return;
     }
+    dia = ddisp->diagram;
+    diagram_update_menu_sensitivity (dia);
 }
 
 
@@ -1235,6 +1299,11 @@ ddisplay_really_destroy(DDisplay *ddisp)
   if (active_display == ddisp)
     display_set_active(NULL);
 
+  /* last chance to avoid crashing in the idle update */
+  if (ddisp->update_id) {
+    g_source_remove (ddisp->update_id);
+    ddisp->update_id = 0;
+  }
   
   if (ddisp->diagram) {
     diagram_remove_ddisplay(ddisp->diagram, ddisp);
@@ -1253,7 +1322,7 @@ ddisplay_really_destroy(DDisplay *ddisp)
   ddisplay_free_update_areas(ddisp);
   /* Free display_areas list */
   ddisplay_free_display_areas(ddisp);
-  
+
   g_free(ddisp);
 }
 
@@ -1261,7 +1330,37 @@ ddisplay_really_destroy(DDisplay *ddisp)
 void
 ddisplay_set_title(DDisplay  *ddisp, char *title)
 {
-  gtk_window_set_title (GTK_WINDOW (ddisp->shell), title);
+  if (ddisp->is_standalone_window)
+    gtk_window_set_title (GTK_WINDOW (ddisp->shell), title);
+  else
+  {
+    GtkNotebook *notebook = g_object_get_data (G_OBJECT (ddisp->shell), 
+                                               DIA_MAIN_NOTEBOOK);
+    /* Find the page with ddisp then set the label on the tab */
+    gint num_pages = gtk_notebook_get_n_pages (notebook);
+    gint num;
+    GtkWidget *page;
+    for (num = 0 ; num < num_pages ; num++)
+    {
+      page = gtk_notebook_get_nth_page (notebook,num);
+      if (g_object_get_data (G_OBJECT (page), "DDisplay") == ddisp)
+      {
+        GtkLabel *label = g_object_get_data (G_OBJECT (page), "tab-label");
+        /* not using the passed in title here, because it may be too long */
+        gchar *name = diagram_get_name(ddisp->diagram);
+        gtk_label_set_text(label,name);
+        g_free(name);
+        break;
+      }
+    }
+    /* now modify the application window title */
+    {
+      const gchar *pname = g_get_prgname();
+      gchar *fulltitle = g_strdup_printf ("%s - %s", title, pname ? pname : "Dia");
+      gtk_window_set_title (GTK_WINDOW (ddisp->shell), fulltitle);
+      g_free(fulltitle);
+    }
+  }
 }
 
 void
@@ -1294,7 +1393,63 @@ ddisplay_set_all_cursor(GdkCursor *cursor)
 void
 ddisplay_set_cursor(DDisplay *ddisp, GdkCursor *cursor)
 {
-  gdk_window_set_cursor(ddisp->canvas->window, cursor);
+  if (ddisp->canvas->window)
+    gdk_window_set_cursor(ddisp->canvas->window, cursor);
+}
+
+/** Returns whether the rulers are currently showing on the display.
+ */
+gboolean display_get_rulers_showing(DDisplay *ddisp) {
+  return ddisp->rulers_are_showing;
+}
+
+
+/**
+ * Shows the rulers and sets flag ddisp->rulers_are_showing.  This
+ * is needed to detect whether a show() has been issued.  There is a 
+ * delay between the time that gtk_widget_show() is called and the time
+ * when GTK_WIDGET_IS_VISIBLE(w) will indicate true.
+ * @param ddisp The display to show the rulers on.
+ */
+void display_rulers_show (DDisplay *ddisp)
+{
+  if (ddisp)
+  {
+    GtkWidget *parent = GTK_WIDGET (ddisp->origin->parent);
+
+    gtk_widget_show (ddisp->origin);
+    gtk_widget_show (ddisp->hrule);
+    gtk_widget_show (ddisp->vrule);
+
+    if (GTK_WIDGET_VISIBLE (parent))
+      gtk_widget_queue_resize (parent);
+
+    ddisp->rulers_are_showing = TRUE;
+  }
+}
+
+/**
+ * Hides the rulers and resets the flag ddisp->rulers_are_showing.  This
+ * is needed to detect whether a hide() has been issued.  There is a 
+ * delay between the time that gtk_widget_hide() is called and the time
+ * when GTK_WIDGET_IS_VISIBLE(w) will indicate false.
+ * @param ddisp The display to hide the rulers on.
+ */
+void display_rulers_hide (DDisplay *ddisp)
+{
+  if (ddisp)
+  {
+    GtkWidget *parent = GTK_WIDGET (ddisp->origin->parent);
+
+    gtk_widget_hide (ddisp->origin);
+    gtk_widget_hide (ddisp->hrule);
+    gtk_widget_hide (ddisp->vrule);
+    
+    if (GTK_WIDGET_VISIBLE (parent))
+      gtk_widget_queue_resize (parent);
+
+    ddisp->rulers_are_showing = FALSE;
+  }
 }
 
 void 
@@ -1303,7 +1458,6 @@ ddisplay_update_statusbar(DDisplay *ddisp)
   update_zoom_status (ddisp);
   update_snap_grid_status (ddisp);
   update_mainpoint_status (ddisp);
-  update_modified_status (ddisp);
 }
 
 void
@@ -1317,16 +1471,52 @@ display_set_active(DDisplay *ddisp)
     diagram_properties_set_diagram(ddisp ? ddisp->diagram : NULL);
 
     if (ddisp) {
-      display_update_menu_state(ddisp);
+      if (ddisp->is_standalone_window)
+      {
+        display_update_menu_state(ddisp);
 
-      if (prefs.toolbox_on_top) {
-        gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
-                                     GTK_WINDOW(ddisp->shell));
+        if (prefs.toolbox_on_top) {
+          gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
+                                       GTK_WINDOW(ddisp->shell));
+        } else {
+          gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
+                                       NULL);
+        }
       } else {
-        gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
-                                     NULL);
+        GtkNotebook *notebook = g_object_get_data (G_OBJECT (ddisp->shell), 
+                                                   DIA_MAIN_NOTEBOOK);
+        /* Find the page with ddisp then set the label on the tab */
+        gint num_pages = gtk_notebook_get_n_pages (notebook);
+        gint num;
+        GtkWidget *page;
+        for (num = 0 ; num < num_pages ; num++)
+        {
+          page = gtk_notebook_get_nth_page (notebook,num);
+          if (g_object_get_data (G_OBJECT (page), "DDisplay") == ddisp)
+          {
+            gtk_notebook_set_current_page (notebook,num);
+            break;
+          }
+        }
+        /* synchronize_ui_to_active_display (ddisp); */
+        /* updates display title, etc */
+        diagram_modified(ddisp->diagram);
+
+        /* ZOOM */
+        update_zoom_status (ddisp);
+
+        /* Snap to grid */
+        ddisplay_set_snap_to_grid (ddisp, ddisp->grid.snap); /* menus */
+
+        /* Object snapping */
+        ddisplay_set_snap_to_objects (ddisp, ddisp->mainpoint_magnetism);
+
+        display_update_menu_state (ddisp);
+
+        gtk_window_present (GTK_WINDOW(ddisp->shell));
       }
     } else {
+      /* TODO: Prevent gtk_window_set_transient_for() in Integrated UI case */
       gtk_window_set_transient_for(GTK_WINDOW(interface_get_toolbox_shell()),
                                    NULL);
     }
@@ -1354,3 +1544,80 @@ ddisplay_im_context_preedit_reset(DDisplay *ddisp, Focus *focus)
     ddisp->preedit_attrs = NULL;
   }
 }
+
+/** Get the active focus for the given display, or NULL.
+ *
+ * @param ddisp Display to get active focus for.  This display need not
+ *              be the currently active display.
+ * @returns The focus that is active for the given display, or NULL if no
+ *          focus is active (i.e. no text is being edited).
+ */
+Focus *
+ddisplay_active_focus(DDisplay *ddisp)
+{
+  return ddisp ? ddisp->active_focus : NULL;
+}
+
+/** Set the currently active focus for this display.  This field should be
+ *  set to non-null when a text is being edited and to null when no text
+ *  is being edited.  Only textedit.c really needs to call this function.
+ *
+ * @param ddisp The display to set active focus for.
+ * @param focus The focus that should be active for this display.  May be
+ *              NULL, indicating that no text is currently being edited on
+ *              this display.
+ */
+void
+ddisplay_set_active_focus(DDisplay *ddisp, Focus *focus)
+{
+  ddisp->active_focus = focus;
+}
+
+void
+ddisplay_show_all (DDisplay *ddisp)
+{
+  Diagram *dia;
+  real magnify_x, magnify_y;
+  int width, height;
+  Point middle;
+
+  g_return_if_fail (ddisp != NULL);
+  
+  dia = ddisp->diagram;
+
+  width = dia_renderer_get_width_pixels (ddisp->renderer);
+  height = dia_renderer_get_height_pixels (ddisp->renderer);
+
+  /* if there is something selected show that instead of all exisiting objects */
+  if (dia->data->selected) {
+    GList *list = dia->data->selected;
+    Rectangle extents = *dia_object_get_enclosing_box ((DiaObject*)list->data);
+    list = g_list_next(list);
+    while (list) {
+      DiaObject *obj = (DiaObject *)list->data;
+      rectangle_union(&extents, dia_object_get_enclosing_box (obj));
+      list = g_list_next(list);
+    }
+    magnify_x = (real)width / (extents.right - extents.left) / ddisp->zoom_factor;
+    magnify_y = (real)height / (extents.bottom - extents.top) / ddisp->zoom_factor;
+    middle.x = extents.left + (extents.right - extents.left) / 2.0;
+    middle.y = extents.top + (extents.bottom - extents.top) / 2.0;
+  } else {
+    magnify_x = (real)width /
+      (dia->data->extents.right - dia->data->extents.left) / ddisp->zoom_factor;
+    magnify_y = (real)height /
+      (dia->data->extents.bottom - dia->data->extents.top) / ddisp->zoom_factor;
+
+    middle.x = dia->data->extents.left +
+      (dia->data->extents.right - dia->data->extents.left) / 2.0;
+    middle.y = dia->data->extents.top +
+      (dia->data->extents.bottom - dia->data->extents.top) / 2.0;
+  }
+
+  ddisplay_zoom (ddisp, &middle, 
+		 ((magnify_x<magnify_y)?magnify_x:magnify_y)/1.05);
+
+  ddisplay_update_scrollbars(ddisp);
+  ddisplay_add_update_all(ddisp);
+  ddisplay_flush(ddisp);
+}
