@@ -33,9 +33,9 @@ typedef struct _DDisplay DDisplay;
 /** This is close to, but not quite the same as, the physical display size
  * in most cases */
 
-#define DDISPLAY_MAX_ZOOM 500.0
+#define DDISPLAY_MAX_ZOOM 2000.0
 #define DDISPLAY_NORMAL_ZOOM 20.0
-#define DDISPLAY_MIN_ZOOM 1.0
+#define DDISPLAY_MIN_ZOOM 0.2
 
 /* The zoom amount should be uniform.  Pixels per cm should be defined by the
  * renderer alone.  But that'd take a lot of fiddling in renderers. */
@@ -51,14 +51,13 @@ struct _DDisplay {
   GtkWidget      *canvas;              /* canvas widget for this ddisplay   */
   GtkWidget      *hsb, *vsb;           /* widgets for scroll bars           */
   GtkWidget      *hrule, *vrule;       /* widgets for rulers                */
-  GtkWidget      *origin;              /* widgets for rulers                */
+  GtkWidget      *origin;              /* either decoration or menu button  */
   GtkWidget      *menu_bar;            /* widget for the menu bar           */
   GtkUIManager   *ui_manager;     /* ui manager used to create the menu bar */
   GtkActionGroup *actions;        
 
   /* menu bar widgets */
   GtkMenuItem *rulers;
-  UpdatableMenuItems updatable_menu_items;
 
   GtkWidget *zoom_status;         
   GtkWidget *grid_status;
@@ -92,6 +91,22 @@ struct _DDisplay {
   /* Preedit String */
   gchar *preedit_string;
   PangoAttrList *preedit_attrs;
+ 
+  /* Is there another case?  Like I see embedded-dia modules, do these do something
+   * in addition??? */  
+  gboolean   is_standalone_window;
+
+  /* Points to Integrated UI Toolbar */
+  GtkToolbar *common_toolbar;
+
+  /* Points to widget containing the diagram if not standalone window */
+  GtkWidget *container; 
+
+  /* Private field, indicates if rulers are shown for this diagram. */
+  gboolean rulers_are_showing;
+
+  /* Private field, indicates which text, if any, is being edited */
+  Focus *active_focus;
 };
 
 extern GdkCursor *default_cursor;
@@ -144,6 +159,10 @@ void ddisplay_set_title(DDisplay *ddisp, char *title);
 void ddisplay_set_cursor(DDisplay *ddisp, GdkCursor *cursor);
 void ddisplay_set_all_cursor(GdkCursor *cursor);
 
+gboolean display_get_rulers_showing(DDisplay *ddisp);
+void display_rulers_show (DDisplay *ddisp);
+void display_rulers_hide (DDisplay *ddisp);
+
 gboolean ddisplay_scroll(DDisplay *ddisp, Point *delta);
 gboolean ddisplay_autoscroll(DDisplay *ddisp, int x, int y);
 void ddisplay_scroll_up(DDisplay *ddisp);
@@ -152,6 +171,9 @@ void ddisplay_scroll_left(DDisplay *ddisp);
 void ddisplay_scroll_right(DDisplay *ddisp);
 gboolean ddisplay_scroll_center_point(DDisplay *ddisp, Point *p);
 gboolean ddisplay_scroll_to_object(DDisplay *ddisp, DiaObject *obj);
+gboolean ddisplay_present_object(DDisplay *ddisp, DiaObject *obj);
+
+void ddisplay_show_all (DDisplay *ddisp);
 
 void display_update_menu_state(DDisplay *ddisp);
 void ddisplay_update_statusbar(DDisplay *ddisp);
@@ -161,4 +183,7 @@ void display_set_active(DDisplay *ddisp);
 
 void ddisplay_im_context_preedit_reset(DDisplay *ddisp, Focus *focus);
 
+Focus *ddisplay_active_focus(DDisplay *ddisp);
+void ddisplay_set_active_focus(DDisplay *ddisp, Focus *focus);
+
 #endif /* DDISPLAY_H */
