@@ -18,7 +18,15 @@
 
 #include <config.h>
 
+#ifdef GNOME
+#undef GTK_DISABLE_DEPRECATED /* gnome */
+#include <gnome.h>
+#else
+#undef GTK_DISABLE_DEPRECATED /* GtkTooltips */
 #include <gtk/gtk.h>
+#endif
+#include "gtkwrapbox.h"
+#include "gtkhwrapbox.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -28,7 +36,6 @@
 #include "layer_dialog.h"
 #include "interface.h"
 #include "display.h"
-#include "pixmaps.h"
 #include "preferences.h"
 #include "commands.h"
 #include "dia_dirs.h"
@@ -45,6 +52,8 @@
 #include "dia-app-icons.h"
 #include "diacanvas.h"
 
+#include "pixmaps/swap.xpm"
+#include "pixmaps/default.xpm"
 #include "pixmaps/missing.xpm"
 
 /* HB: file dnd stuff lent by The Gimp, not fully understood but working ...
@@ -64,6 +73,9 @@ static GtkTargetEntry toolbox_target_table[] =
 static guint toolbox_n_targets = (sizeof (toolbox_target_table) /
                                  sizeof (toolbox_target_table[0]));
 
+static void
+use_integrated_ui_for_display_shell(DDisplay *ddisp, char *title);
+
 static void
 dia_dnd_file_drag_data_received (GtkWidget        *widget,
                                  GdkDragContext   *context,
@@ -143,63 +155,69 @@ ToolButton tool_data[] =
     "ToolsModify",
     { MODIFY_TOOL, NULL, NULL}
   },
+  { (char **) dia_textedit_tool_icon,
+    N_("Text edit(s)\nUse <Esc> to leave this tool"),
+    "F2",
+    "ToolsTextedit",
+    { TEXTEDIT_TOOL, NULL, NULL}
+  },
   { (char **) dia_zoom_tool_icon,
     N_("Magnify"),
-    "<alt>M",
+    "M",
     "ToolsMagnify",
     { MAGNIFY_TOOL, NULL, NULL}
   },
   { (char **) dia_scroll_tool_icon,
     N_("Scroll around the diagram"),
-    "<alt>S",
+    "S",
     "ToolsScroll",
     { SCROLL_TOOL, NULL, NULL}
   },
   { NULL,
     N_("Text"),
-    "<alt>T",
+    "T",
     "ToolsText",
     { CREATE_OBJECT_TOOL, "Standard - Text", NULL }
   },
   { NULL,
     N_("Box"),
-    "<alt>R",
+    "R",
     "ToolsBox",
     { CREATE_OBJECT_TOOL, "Standard - Box", NULL }
   },
   { NULL,
     N_("Ellipse"),
-    "<alt>E",
+    "E",
     "ToolsEllipse",
     { CREATE_OBJECT_TOOL, "Standard - Ellipse", NULL }
   },
   { NULL,
     N_("Polygon"),
-    "<alt>P",
+    "P",
     "ToolsPolygon",
     { CREATE_OBJECT_TOOL, "Standard - Polygon", NULL }
   },
   { NULL,
     N_("Beziergon"),
-    "<alt>B",
+    "B",
     "ToolsBeziergon",
     { CREATE_OBJECT_TOOL, "Standard - Beziergon", NULL }
   },
   { NULL,
     N_("Line"),
-    "<alt>L",
+    "L",
     "ToolsLine",
     { CREATE_OBJECT_TOOL, "Standard - Line", NULL }
   },
   { NULL,
     N_("Arc"),
-    "<alt>A",
+    "A",
     "ToolsArc",
     { CREATE_OBJECT_TOOL, "Standard - Arc", NULL }
   },
   { NULL,
     N_("Zigzagline"),
-    "<alt>Z",
+    "Z",
     "ToolsZigzagline",
     { CREATE_OBJECT_TOOL, "Standard - ZigZagLine", NULL }
   },
@@ -211,15 +229,23 @@ ToolButton tool_data[] =
   },
   { NULL,
     N_("Bezierline"),
-    "<alt>C",
+    "C",
     "ToolsBezierline",
     { CREATE_OBJECT_TOOL, "Standard - BezierLine", NULL }
   },
   { NULL,
     N_("Image"),
-    "<alt>I",
+    "I",
     "ToolsImage",
     { CREATE_OBJECT_TOOL, "Standard - Image", NULL }
+#ifdef HAVE_CAIRO
+  },
+  { NULL,
+    N_("Outline"),
+    NULL,
+    "ToolsOutline",
+    { CREATE_OBJECT_TOOL, "Standard - Outline", NULL }
+#endif
   }
 };
 
@@ -231,11 +257,32 @@ const int num_tools = NUM_TOOLS;
 
 static GtkWidget *toolbox_shell = NULL;
 static GtkWidget *tool_widgets[NUM_TOOLS];
+
+static struct 
+{
+    GtkWindow    * main_window;
+    GtkToolbar   * toolbar;
+    GtkNotebook  * diagram_notebook;
+    GtkStatusbar * statusbar;
+    GtkWidget    * layer_view;
+} ui;
+
 /*static*/ GtkTooltips *tool_tips;
 static GSList *tool_group = NULL;
 
 GtkWidget *modify_tool_button;
 
+/** 
+ * Used to determine if the current user interface is the integrated interface or 
+ * the distributed interface.  This cannot presently be determined by the preferences
+ * setting because changing that setting at run time does not change the interface.
+ * @return Non-zero if the integrated interface is present, else zero.
+ */
+int is_integrated_ui (void)
+{
+  return ui.main_window == NULL? 0 : 1;
+}
+
 static void
 grid_toggle_snap(GtkWidget *widget, gpointer data)
 {
@@ -269,6 +316,26 @@ origin_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
   return FALSE;
 }
 
+void
+view_zoom_set (float factor)
+{
+  DDisplay *ddisp;
+  real scale;
+  Point middle;
+  Rectangle *visible;
+
+  ddisp = ddisplay_active();
+  if (!ddisp) return;
+
+  visible = &ddisp->visible;
+  middle.x = visible->left*0.5 + visible->right*0.5;
+  middle.y = visible->top*0.5 + visible->bottom*0.5;
+
+  scale = ((real) factor)/1000.0 * DDISPLAY_NORMAL_ZOOM;
+
+  ddisplay_zoom(ddisp, &middle, scale / ddisp->zoom_factor);  
+}
+
 static void
 zoom_activate_callback(GtkWidget *item, gpointer user_data) {
   DDisplay *ddisp = (DDisplay *)user_data;
@@ -284,6 +351,15 @@ zoom_activate_callback(GtkWidget *item, gpointer user_data) {
     Point middle;
     Rectangle *visible;
 
+    /* Set limits to avoid crashes, see bug #483384 */
+    if (zoom_amount < .1) {
+      zoom_amount = .1;
+    } else if (zoom_amount > 1e4) {
+      zoom_amount = 1e4;
+    }
+    zoomamount = g_strdup_printf("%f%%\n", zoom_amount);
+    gtk_entry_set_text(GTK_ENTRY(gtk_object_get_user_data(GTK_OBJECT(ddisp->zoom_status))), zoomamount);
+    g_free(zoomamount);
     magnify = (zoom_amount*DDISPLAY_NORMAL_ZOOM/100.0)/ddisp->zoom_factor;
     if (fabs(magnify - 1.0) > 0.000001) {
       visible = &ddisp->visible;
@@ -332,7 +408,7 @@ create_zoom_widget(DDisplay *ddisp) {
 		    G_CALLBACK(zoom_activate_callback),
 		      ddisp);
   gtk_box_pack_start_defaults(GTK_BOX(combo), entry);
-  gtk_object_set_user_data(GTK_OBJECT(combo), entry);
+  g_object_set_data (G_OBJECT(combo), "user_data", entry);
   gtk_entry_set_width_chars(GTK_ENTRY(entry), 8);
   gtk_widget_show(entry);
 
@@ -341,7 +417,7 @@ create_zoom_widget(DDisplay *ddisp) {
   arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
   gtk_container_add(GTK_CONTAINER(button), arrow);
   gtk_box_pack_start_defaults(GTK_BOX(combo), button);
-  gtk_object_set_user_data(GTK_OBJECT(combo), entry);
+  g_object_set_data (G_OBJECT(combo), "user_data", entry);
   gtk_widget_show_all(button);
 
   menu = gtk_menu_new();
@@ -399,6 +475,233 @@ display_data_received_callback (GtkWidget *widget,
     dia_dnd_file_drag_data_received (widget, context, x, y, data, info, time, ddisp);
 }
 
+/**
+ * @param button The notebook close button.
+ * @param user_data Container widget (e.g. VBox).
+ */
+void
+close_notebook_page_callback (GtkButton *button,
+                              gpointer   user_data)
+{
+  GtkBox      *page     = user_data;
+  DDisplay    *ddisp    = g_object_get_data (G_OBJECT (page), "DDisplay");
+
+  /* When the page widget is destroyed it removes itself from the notebook */
+  ddisplay_close (ddisp);
+}
+
+/**
+ * @param ddisp The diagram display object that a window is created for
+ * @param title
+ */
+static void
+use_integrated_ui_for_display_shell(DDisplay *ddisp, char *title)
+{
+  GtkWidget *table;
+  GtkWidget *navigation_button;
+  /* GtkWidget *status_hbox; */
+  /* GtkWidget *zoom_hbox, *zoom_label; */
+  GtkWidget *label;                /* Text label for the notebook page */
+  GtkWidget *tab_label_container;  /* Container to hold text label & close button */
+  int width, height;               /* Width/Heigth of the diagram */
+  GtkWidget *image;
+  GtkWidget *close_button;         /* Close button for the notebook page */
+  /* GtkWidget *widget; */
+  GtkRcStyle *rcstyle;
+  gint       notebook_page_index;
+	
+  ddisp->is_standalone_window = FALSE;
+
+  ddisp->shell = GTK_WIDGET (ui.main_window);
+
+  /* Statusbar */
+  ddisp->modified_status = GTK_WIDGET (ui.statusbar);
+ 
+  tab_label_container = gtk_hbox_new(FALSE,3);
+  
+  label = gtk_label_new( title );
+  gtk_box_pack_start( GTK_BOX(tab_label_container), label, FALSE, FALSE, 0 );
+  gtk_widget_show (label);
+
+  /* Create a new tab page */
+  ddisp->container = gtk_vbox_new(FALSE, 0);
+
+  /* Create a new tab page */
+  ddisp->container = gtk_vbox_new(FALSE, 0);
+
+  /* <from GEdit> */
+  /* don't allow focus on the close button */
+  close_button = gtk_button_new();
+  gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+  gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
+
+  /* make it as small as possible */
+  rcstyle = gtk_rc_style_new ();
+  rcstyle->xthickness = rcstyle->ythickness = 0;
+  gtk_widget_modify_style (close_button, rcstyle);
+  gtk_rc_style_unref (rcstyle),
+
+  image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
+                                    GTK_ICON_SIZE_MENU);
+
+  gtk_container_add (GTK_CONTAINER(close_button), image);
+  gtk_signal_connect (GTK_OBJECT (close_button), "clicked", 
+                      GTK_SIGNAL_FUNC (close_notebook_page_callback), ddisp->container);
+  /* </from GEdit> */
+
+  gtk_box_pack_start( GTK_BOX(tab_label_container), close_button, FALSE, FALSE, 0 );
+  gtk_widget_show (close_button);
+  gtk_widget_show (image);
+
+  /* Set events for new tab page */
+  gtk_widget_set_events (ddisp->container,
+                         GDK_POINTER_MOTION_MASK |
+                         GDK_POINTER_MOTION_HINT_MASK |
+                         GDK_FOCUS_CHANGE_MASK);
+
+  notebook_page_index = gtk_notebook_append_page (GTK_NOTEBOOK(ui.diagram_notebook),
+                                                  ddisp->container,
+                                                  tab_label_container);
+
+  g_object_set_data (G_OBJECT (ddisp->container), "DDisplay",  ddisp);
+  g_object_set_data (G_OBJECT (ddisp->container), "tab-label", label);
+  g_object_set_data (G_OBJECT (ddisp->container), "window",    ui.main_window);
+
+  /*  the table containing all widgets  */
+  table = gtk_table_new (3, 3, FALSE);
+  gtk_table_set_col_spacing (GTK_TABLE (table), 0, 1);
+  gtk_table_set_col_spacing (GTK_TABLE (table), 1, 2);
+  gtk_table_set_row_spacing (GTK_TABLE (table), 0, 1);
+  gtk_table_set_row_spacing (GTK_TABLE (table), 1, 2);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 2);
+
+  gtk_box_pack_start( GTK_BOX(ddisp->container), table, TRUE, TRUE, 0 );
+
+  /*  scrollbars, rulers, canvas, menu popup button  */
+  ddisp->origin = gtk_frame_new (NULL);
+  gtk_frame_set_shadow_type (GTK_FRAME (ddisp->origin), GTK_SHADOW_OUT);
+
+  ddisp->hrule = gtk_hruler_new ();
+  g_signal_connect_swapped (GTK_OBJECT (ddisp->container), "motion_notify_event",
+                            G_CALLBACK(GTK_WIDGET_GET_CLASS (ddisp->hrule)->motion_notify_event),
+                            GTK_OBJECT (ddisp->hrule));
+
+  ddisp->vrule = gtk_vruler_new ();
+  g_signal_connect_swapped (GTK_OBJECT (ddisp->container), "motion_notify_event",
+			    G_CALLBACK(GTK_WIDGET_GET_CLASS (ddisp->vrule)->motion_notify_event),
+                            GTK_OBJECT (ddisp->vrule));
+
+  /* Get the width/height of the Notebook child area */
+  /* TODO: Fix width/height hardcoded values */
+  width = 100;
+  height = 100;
+
+  /*  The adjustment datums  */
+  ddisp->hsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, width, 1, (width-1)/4, width-1));
+  ddisp->vsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, height, 1, (height-1)/4, height-1));
+
+  ddisp->hsb = gtk_hscrollbar_new (ddisp->hsbdata);
+  GTK_WIDGET_UNSET_FLAGS (ddisp->hsb, GTK_CAN_FOCUS);
+  ddisp->vsb = gtk_vscrollbar_new (ddisp->vsbdata);
+  GTK_WIDGET_UNSET_FLAGS (ddisp->vsb, GTK_CAN_FOCUS);
+
+  /*  set up the scrollbar observers  */
+  g_signal_connect (GTK_OBJECT (ddisp->hsbdata), "value_changed",
+		    G_CALLBACK(ddisplay_hsb_update),
+		      ddisp);
+  g_signal_connect (GTK_OBJECT (ddisp->vsbdata), "value_changed",
+		    G_CALLBACK(ddisplay_vsb_update),
+		      ddisp);
+
+  /*  Popup button between scrollbars for navigation window  */
+  navigation_button = navigation_popup_new(ddisp);
+  gtk_tooltips_set_tip(tool_tips, navigation_button,
+                       _("Pops up the Navigation window."), NULL);
+  gtk_widget_show(navigation_button);
+
+  /*  Canvas  */
+  ddisp->canvas = dia_canvas_new();
+  
+  /* Dia's canvas does it's double buffering alone so switch off GTK's */
+  gtk_widget_set_double_buffered (ddisp->canvas, FALSE);
+
+  gtk_widget_set_events (ddisp->canvas, CANVAS_EVENT_MASK);
+  GTK_WIDGET_SET_FLAGS (ddisp->canvas, GTK_CAN_FOCUS);
+  g_signal_connect (GTK_OBJECT (ddisp->canvas), "event",
+                    G_CALLBACK(ddisplay_canvas_events),
+                    ddisp);
+  
+  gtk_drag_dest_set(ddisp->canvas, GTK_DEST_DEFAULT_ALL,
+		    display_target_table, display_n_targets, GDK_ACTION_COPY);
+  g_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_drop",
+		    G_CALLBACK(display_drop_callback), NULL);
+  g_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_data_received",
+		    G_CALLBACK(display_data_received_callback), ddisp);
+
+  /*  place all the widgets  */
+  gtk_table_attach (GTK_TABLE (table), ddisp->origin, 0, 1, 0, 1,
+                    GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), ddisp->hrule, 1, 2, 0, 1,
+                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), ddisp->vrule, 0, 1, 1, 2,
+                    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), ddisp->canvas, 1, 2, 1, 2,
+                    GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), ddisp->hsb, 0, 2, 2, 3,
+                    GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), ddisp->vsb, 2, 3, 0, 2,
+                    GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+  gtk_table_attach (GTK_TABLE (table), navigation_button, 2, 3, 2, 3,
+                    GTK_FILL, GTK_FILL, 0, 0);
+
+  ddisp->common_toolbar = ui.toolbar;
+
+  /* Stand-alone window menubar */
+  ddisp->menu_bar = NULL;
+
+  /* Stand-alone window Zoom status/menu */
+  ddisp->zoom_status = NULL;
+
+  /* Stand-alone window Grid on/off button */
+  ddisp->grid_status = NULL;
+
+  /* Stand-alone window Object Snapping button */
+  ddisp->mainpoint_status = NULL;
+
+  gtk_widget_show (ddisp->container);
+  gtk_widget_show (table);
+  gtk_widget_show (ddisp->hsb);
+  gtk_widget_show (ddisp->vsb);
+  display_rulers_show (ddisp);
+  gtk_widget_show (ddisp->canvas);
+
+  /* Show new page */
+  gtk_notebook_set_current_page (ui.diagram_notebook, notebook_page_index);
+
+  integrated_ui_toolbar_grid_snap_synchronize_to_display (ddisp);
+
+  integrated_ui_toolbar_object_snap_synchronize_to_display (ddisp);
+
+  /* TODO: Figure out how to detect if anti-aliased renderer was set */
+  /** For the distributed display this is called when the ddisp->canvas is shown.
+   * The show causes a GDK_CONFIGURE event but this is not happening here.  If this
+   * is not set a seg-fault occurs when dia_renderer_get_width_pixels() is called
+   */
+  ddisplay_set_renderer(ddisp, ddisp->aa_renderer);
+
+  /*  set the focus to the canvas area  */
+  gtk_widget_grab_focus (ddisp->canvas);
+}
+
+/**
+ * @param ddisp The diagram display object that a window is created for
+ * @param width Diagram widgth
+ * @param height Diagram Height
+ * @param title Window title
+ * @param use_mbar Flag to indicate whether to add a menubar to the window
+ * @param top_level_window
+ */
 void
 create_display_shell(DDisplay *ddisp,
 		     int width, int height,
@@ -411,6 +714,16 @@ create_display_shell(DDisplay *ddisp,
   GtkWidget *zoom_hbox, *zoom_label;
   int s_width, s_height;
 
+  if (app_is_interactive() && is_integrated_ui())
+  {
+    use_integrated_ui_for_display_shell(ddisp, title);
+    return;
+  }
+ 
+  ddisp->is_standalone_window = TRUE;
+  ddisp->container            = NULL;
+
+
   if (!tool_tips) /* needed here if we dont create_toolbox() */
     tool_tips = gtk_tooltips_new ();
     
@@ -430,7 +743,10 @@ create_display_shell(DDisplay *ddisp,
     ddisp->shell = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     gtk_window_set_title (GTK_WINDOW (ddisp->shell), title);
     gtk_window_set_role (GTK_WINDOW (ddisp->shell), "diagram_window");
-    {
+    gtk_window_set_icon_name (GTK_WINDOW (ddisp->shell), "dia");
+    gtk_window_set_default_size(GTK_WINDOW (ddisp->shell), width, height);
+    /* set_icon_name needs registered theme icons, not always available: provide fallback */
+    if (!gtk_window_get_icon (GTK_WINDOW (ddisp->shell))) {
       static GdkPixbuf *pixbuf = NULL;
 
       if (!pixbuf)
@@ -438,11 +754,10 @@ create_display_shell(DDisplay *ddisp,
       if (pixbuf)
         gtk_window_set_icon (GTK_WINDOW (ddisp->shell), pixbuf);
     }
-    gtk_window_set_default_size(GTK_WINDOW (ddisp->shell), width, height);
   } else {
     ddisp->shell = gtk_event_box_new ();
   }
-  gtk_object_set_user_data (GTK_OBJECT (ddisp->shell), (gpointer) ddisp);
+  g_object_set_data (G_OBJECT (ddisp->shell), "user_data", (gpointer) ddisp);
 
   gtk_widget_set_events (ddisp->shell,
 			 GDK_POINTER_MOTION_MASK |
@@ -561,7 +876,7 @@ create_display_shell(DDisplay *ddisp,
   g_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_data_received",
 		    G_CALLBACK(display_data_received_callback), ddisp);
 
-  gtk_object_set_user_data (GTK_OBJECT (ddisp->canvas), (gpointer) ddisp);
+  g_object_set_data (G_OBJECT (ddisp->canvas), "user_data", (gpointer) ddisp);
 
   /*  pack all the widgets  */
   gtk_table_attach (GTK_TABLE (table), ddisp->origin, 0, 1, 0, 1,
@@ -589,7 +904,6 @@ create_display_shell(DDisplay *ddisp,
     ddisp->menu_bar = menus_create_display_menubar (&ddisp->ui_manager, &ddisp->actions);
     g_assert (ddisp->menu_bar);
     gtk_box_pack_start (GTK_BOX (root_vbox), ddisp->menu_bar, FALSE, TRUE, 0);
-    menus_initialize_updatable_items (&ddisp->updatable_menu_items, ddisp->actions);
   }
 
   /* the statusbars */
@@ -645,9 +959,7 @@ create_display_shell(DDisplay *ddisp,
 
   gtk_widget_show (ddisp->hsb);
   gtk_widget_show (ddisp->vsb);
-  gtk_widget_show (ddisp->origin);
-  gtk_widget_show (ddisp->hrule);
-  gtk_widget_show (ddisp->vrule);
+  display_rulers_show (ddisp);
   gtk_widget_show (ddisp->zoom_status);
   gtk_widget_show (zoom_hbox);
   gtk_widget_show (zoom_label);
@@ -694,7 +1006,7 @@ tool_select_update (GtkWidget *w,
   ToolButtonData *tooldata = (ToolButtonData *) data;
 
   if (tooldata == NULL) {
-    g_warning(_("NULL tooldata in tool_select_update"));
+    g_warning("NULL tooldata in tool_select_update");
     return;
   }
 
@@ -806,7 +1118,7 @@ create_tools(GtkWidget *parent)
   GtkWidget *pixmapwidget;
   GdkPixmap *pixmap = NULL;
   GdkBitmap *mask = NULL;
-  GtkStyle *style;
+  /* GtkStyle *style; */
   char **pixmap_data;
   int i;
 
@@ -941,6 +1253,17 @@ fill_sheet_wbox(Sheet *sheet)
       
       pixbuf = gdk_pixbuf_new_from_file(sheet_obj->pixmap_file, &gerror);
       if (pixbuf != NULL) {
+          int width = gdk_pixbuf_get_width (pixbuf);
+          int height = gdk_pixbuf_get_height (pixbuf);
+          if (width > 22) {
+	    GdkPixbuf *cropped;
+	    g_warning ("Shape icon '%s' size wrong, cropped.", sheet_obj->pixmap_file);
+	    cropped = gdk_pixbuf_new_subpixbuf (pixbuf, 
+	                                       (width - 22) / 2, height > 22 ? (height - 22) / 2 : 0, 
+					       22, height > 22 ? 22 : height);
+	    g_object_unref (pixbuf);
+	    pixbuf = cropped;
+	  }
           gdk_pixbuf_render_pixmap_and_mask_for_colormap(pixbuf, gtk_widget_get_colormap(sheet_wbox), &pixmap, &mask, 1.0);
           gdk_pixbuf_unref(pixbuf);
       } else {
@@ -1005,8 +1328,9 @@ fill_sheet_wbox(Sheet *sheet)
 }
 
 static void
-sheet_menu_callback(DiaDynamicMenu *menu, const gchar *string, void *user_data)
+sheet_option_menu_changed(DiaDynamicMenu *menu, gpointer user_data)
 {
+  char *string = dia_dynamic_menu_get_entry(menu);
   Sheet *sheet = get_sheet_by_name(string);
   if (sheet == NULL) {
     message_warning(_("No sheet named %s"), string);
@@ -1014,6 +1338,7 @@ sheet_menu_callback(DiaDynamicMenu *menu, const gchar *string, void *user_data)
     persistence_set_string("last-sheet-selected", string);
     fill_sheet_wbox(sheet);
   }
+  g_free(string);
 }
 
 static int
@@ -1047,19 +1372,21 @@ create_sheet_dropdown_menu(GtkWidget *parent)
 
   sheet_option_menu =
     dia_dynamic_menu_new_stringlistbased(_("Other sheets"), sheet_names, 
-					 sheet_menu_callback,
 					 NULL, "sheets");
+  g_signal_connect(DIA_DYNAMIC_MENU(sheet_option_menu), "value-changed",
+		   G_CALLBACK(sheet_option_menu_changed), sheet_option_menu);
   dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(sheet_option_menu),
 				     "Assorted");
+  dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(sheet_option_menu),
+				     "Flowchart");
   dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(sheet_option_menu),
 				     "UML");
   /*    gtk_widget_set_size_request(sheet_option_menu, 20, -1);*/
   gtk_wrap_box_pack_wrapped(GTK_WRAP_BOX(parent), sheet_option_menu,
 			    TRUE, TRUE, FALSE, FALSE, TRUE);    
-  /* 15 is a magic number that goes beyond the standard objects and
-   * the divider. */
+  /* 15 was a magic number that goes beyond the standard objects and the divider. */
   gtk_wrap_box_reorder_child(GTK_WRAP_BOX(parent),
-			     sheet_option_menu, 15);
+			     sheet_option_menu, NUM_TOOLS+1);
   gtk_widget_show(sheet_option_menu);
 }
 
@@ -1104,8 +1431,7 @@ create_sheets(GtkWidget *parent)
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), sheet_wbox);
   gtk_widget_show(sheet_wbox);
 
-  persistence_register_string("last-sheet-selected", _("Misc"));
-  sheetname = persistence_get_string("last-sheet-selected");
+  sheetname = persistence_register_string("last-sheet-selected", _("Flowchart"));
   sheet = get_sheet_by_name(sheetname);
   if (sheet == NULL) {
     /* Couldn't find it */
@@ -1212,12 +1538,15 @@ create_lineprops_area(GtkWidget *parent)
   Arrow arrow;
   real dash_length;
   LineStyle style;
+  gchar *arrow_name;
 
   chooser = dia_arrow_chooser_new(TRUE, change_start_arrow_style, NULL, tool_tips);
   gtk_wrap_box_pack_wrapped(GTK_WRAP_BOX(parent), chooser, FALSE, TRUE, FALSE, TRUE, TRUE);
   arrow.width = persistence_register_real("start-arrow-width", DEFAULT_ARROW_WIDTH);
   arrow.length = persistence_register_real("start-arrow-length", DEFAULT_ARROW_LENGTH);
-  arrow.type = arrow_type_from_name(persistence_register_string("start-arrow-type", "None"));
+  arrow_name = persistence_register_string("start-arrow-type", "None");
+  arrow.type = arrow_type_from_name(arrow_name);
+  g_free(arrow_name);
   dia_arrow_chooser_set_arrow(DIA_ARROW_CHOOSER(chooser), &arrow);
   attributes_set_default_start_arrow(arrow);
   gtk_tooltips_set_tip(tool_tips, chooser, _("Arrow style at the beginning of new lines.  Click to pick an arrow, or set arrow parameters with Details..."), NULL);
@@ -1234,7 +1563,9 @@ create_lineprops_area(GtkWidget *parent)
   chooser = dia_arrow_chooser_new(FALSE, change_end_arrow_style, NULL, tool_tips);
   arrow.width = persistence_register_real("end-arrow-width", DEFAULT_ARROW_WIDTH);
   arrow.length = persistence_register_real("end-arrow-length", DEFAULT_ARROW_LENGTH);
-  arrow.type = arrow_type_from_name(persistence_register_string("end-arrow-type", "Filled Concave"));
+  arrow_name = persistence_register_string("end-arrow-type", "Filled Concave");
+  arrow.type = arrow_type_from_name(arrow_name);
+  g_free(arrow_name);
   dia_arrow_chooser_set_arrow(DIA_ARROW_CHOOSER(chooser), &arrow);
   attributes_set_default_end_arrow(arrow);
 
@@ -1266,7 +1597,156 @@ toolbox_delete (GtkWidget *widget, GdkEvent *event, gpointer data)
   return FALSE;
 }
 
+static void
+app_set_icon (GtkWindow *window)
+{
+  gtk_window_set_icon_name (window, "dia");
+  if (!gtk_window_get_icon (window)) {
+    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline (-1, dia_app_icon, FALSE, NULL);
+    if (pixbuf) {
+      gtk_window_set_icon (window, pixbuf);
+      g_object_unref (pixbuf);
+    }
+  }
+}
 
+/**
+ * Create integrated user interface
+ */
+void 
+create_integrated_ui (void)
+{
+  GtkWidget *window;
+  GtkWidget *main_vbox;
+  GtkWidget *hbox;
+  GtkWidget *wrapbox;
+  GtkWidget *menubar;
+  GtkWidget *toolbar;
+  GtkWidget *notebook;
+  GtkWidget *statusbar;
+  GtkAccelGroup *accel_group;
+  GdkPixbuf *pixbuf;
+
+  GtkWidget *layer_view;
+	
+#ifdef GNOME
+  window = gnome_app_new ("Dia", _("Diagram Editor"));
+#else
+  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_widget_ref (window);
+  gtk_window_set_title (GTK_WINDOW (window), "Dia v" VERSION);
+#endif
+
+  /* hint to window manager on X so the wm can put the window in the same  
+     as it was when the application shut down                             */      
+  gtk_window_set_role (GTK_WINDOW (window), DIA_MAIN_WINDOW);
+  
+  gtk_window_set_default_size (GTK_WINDOW (window), 146, 349);
+ 
+  app_set_icon (GTK_WINDOW (window));
+
+  g_signal_connect (GTK_OBJECT (window), "delete_event",
+		    G_CALLBACK (toolbox_delete),
+		      window);
+
+  g_signal_connect (GTK_OBJECT (window), "destroy",
+		    G_CALLBACK (toolbox_destroy),
+		      window);
+
+  main_vbox = gtk_vbox_new (FALSE, 1);
+  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 1);
+#ifdef GNOME
+  gnome_app_set_contents (GNOME_APP(window), main_vbox);
+#else
+  gtk_container_add (GTK_CONTAINER (window), main_vbox);
+#endif
+  gtk_widget_show (main_vbox);
+
+  /* Applicatioon Statusbar */
+  statusbar = gtk_statusbar_new ();
+  gtk_box_pack_end (GTK_BOX (main_vbox), statusbar, FALSE, TRUE, 0);
+  /* HBox for everything below the menubar and toolbars */
+  hbox = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (main_vbox), hbox, TRUE, TRUE, 0);
+  gtk_widget_show (hbox);
+  
+  /* Layer View  */
+  layer_view = create_layer_view_widget ();
+  gtk_box_pack_end (GTK_BOX (hbox), layer_view, FALSE, FALSE, 0);
+	 
+  /* Diagram Notebook */
+  notebook = gtk_notebook_new ();
+  gtk_box_pack_end (GTK_BOX (hbox), notebook, TRUE, TRUE, 0);
+  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE); 
+  gtk_widget_show (notebook);
+
+  /*  tooltips  */
+  tool_tips = gtk_tooltips_new ();
+
+  wrapbox = gtk_hwrap_box_new(FALSE);
+  gtk_wrap_box_set_aspect_ratio(GTK_WRAP_BOX(wrapbox), 144.0 / 318.0);
+  gtk_wrap_box_set_justify(GTK_WRAP_BOX(wrapbox), GTK_JUSTIFY_TOP);
+  gtk_wrap_box_set_line_justify(GTK_WRAP_BOX(wrapbox), GTK_JUSTIFY_LEFT);
+
+  /* pack the rest of the stuff */
+  gtk_box_pack_start (GTK_BOX (hbox), wrapbox, FALSE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (wrapbox), 0);
+  gtk_widget_show (wrapbox);
+
+  create_tools (wrapbox);
+  create_sheets (wrapbox);
+  create_color_area (wrapbox);
+  create_lineprops_area (wrapbox);
+
+  /* Setup toolbox area as file drop destination */
+  gtk_drag_dest_set (wrapbox,
+		     GTK_DEST_DEFAULT_ALL,
+		     toolbox_target_table, toolbox_n_targets,
+		     GDK_ACTION_COPY);
+  g_signal_connect (GTK_OBJECT (wrapbox), "drag_data_received",
+		    G_CALLBACK (dia_dnd_file_drag_data_received),
+                    NULL); /* userdata == NULL here intentionally */
+
+  /* menus -- initialised afterwards, because initing the display menus
+   * uses the tool buttons*/
+  menus_get_integrated_ui_menubar(&menubar, &toolbar, &accel_group);
+  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+#ifdef GNOME
+  gnome_app_set_menus (GNOME_APP (window), GTK_MENU_BAR (menubar));
+#else
+  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
+  gtk_widget_show (menubar);
+#endif
+
+  /* Toolbar */
+  /* TODO: maybe delete set_style(toolbar,ICONS) */
+  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_ICONS);
+  gtk_box_pack_start (GTK_BOX (main_vbox), toolbar, FALSE, TRUE, 0);
+  gtk_widget_show (toolbar);
+
+  persistence_register_window(GTK_WINDOW(window));
+
+  ui.main_window      = GTK_WINDOW    (window);
+  ui.toolbar          = GTK_TOOLBAR   (toolbar);
+  ui.diagram_notebook = GTK_NOTEBOOK  (notebook);
+  ui.statusbar        = GTK_STATUSBAR (statusbar);
+  ui.layer_view       =                layer_view;
+
+  /* NOTE: These functions use ui.xxx assignments above and so must come after
+   *       the user interface components are set.                              */
+  integrated_ui_main_toolbar_show ();
+  integrated_ui_main_statusbar_show ();
+
+  /* For access outside here: */
+  g_object_set_data (G_OBJECT (ui.main_window), DIA_MAIN_NOTEBOOK, notebook);
+
+  /* TODO: Figure out what to do about toolbox_shell for integrated UI */
+  toolbox_shell = window;
+}
+
+/**
+ * Create toolbox component for distributed user interface 
+ */
 void
 create_toolbox ()
 {
@@ -1275,7 +1755,6 @@ create_toolbox ()
   GtkWidget *wrapbox;
   GtkWidget *menubar;
   GtkAccelGroup *accel_group;
-  GdkPixbuf *pixbuf;
 
 #ifdef GNOME
   window = gnome_app_new ("Dia", _("Diagram Editor"));
@@ -1287,10 +1766,13 @@ create_toolbox ()
   gtk_window_set_role (GTK_WINDOW (window), "toolbox_window");
   gtk_window_set_default_size(GTK_WINDOW(window), 146, 349);
 
-  pixbuf = gdk_pixbuf_new_from_inline (-1, dia_app_icon, FALSE, NULL);
-  if (pixbuf) {
-    gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
-    g_object_unref (pixbuf);
+  app_set_icon (GTK_WINDOW (window));
+  if (!gtk_window_get_icon (GTK_WINDOW (window))) {
+    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_inline (-1, dia_app_icon, FALSE, NULL);
+    if (pixbuf) {
+      gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
+      g_object_unref (pixbuf);
+    }
   }
 
   g_signal_connect (GTK_OBJECT (window), "delete_event",
@@ -1379,5 +1861,127 @@ create_tree_window(void)
 GtkWidget *
 interface_get_toolbox_shell(void)
 {
+  if (is_integrated_ui ())
+    return GTK_WIDGET(ui.main_window);
+
   return toolbox_shell;
 }
+
+/* Indicate if the integrated UI toolbar is showing.
+ * @return TRUE if showing, FALSE if not showing or doesn't exist 
+ */ 
+gboolean integrated_ui_main_toolbar_is_showing (void)
+{
+  if (ui.toolbar)
+  {
+    return GTK_WIDGET_VISIBLE (ui.toolbar)? TRUE : FALSE;
+  }
+  return FALSE;
+}
+
+/* show() integrated UI main toolbar and set pulldown menu action. */
+void integrated_ui_main_toolbar_show (void)
+{
+  if (ui.toolbar)
+  {
+    GtkAction *action = NULL;
+    gtk_widget_show (GTK_WIDGET (ui.toolbar));
+    action = menus_get_action (VIEW_MAIN_TOOLBAR_ACTION);
+    if (action)
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  }
+}
+
+/* hide() integrated UI main toolbar and reset pulldown menu action. */
+void integrated_ui_main_toolbar_hide (void)
+{
+  if (ui.toolbar)
+  {
+    GtkAction *action = NULL;
+    gtk_widget_hide (GTK_WIDGET (ui.toolbar));
+    action = menus_get_action (VIEW_MAIN_TOOLBAR_ACTION);
+    if (action)
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+  }
+}
+
+/* Indicate if the integrated UI Layer View is showing.
+ * @return TRUE if showing, FALSE if not showing or doesn't exist 
+ */ 
+gboolean integrated_ui_layer_view_is_showing (void)
+{
+  if (ui.layer_view)
+  {
+    return GTK_WIDGET_VISIBLE (ui.layer_view)? TRUE : FALSE;
+  }
+  return FALSE;
+}
+
+void integrated_ui_layer_view_show (void)
+{
+  if (ui.layer_view)
+  {
+    GtkAction *action = NULL;
+    gtk_widget_show (ui.layer_view); 
+    action = menus_get_action (VIEW_LAYERS_ACTION);
+    if (action) 
+    {
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+    }
+  }
+}
+
+void integrated_ui_layer_view_hide (void)
+{
+  if (ui.layer_view)
+  {
+    GtkAction *action = NULL;
+    gtk_widget_hide (ui.layer_view); 
+    action = menus_get_action (VIEW_LAYERS_ACTION);
+    if (action)
+    {
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+    }
+  }
+}
+
+/* Indicate if the integrated UI statusbar is showing.
+ * @return TRUE if showing, FALSE if not showing or doesn't exist 
+ */ 
+gboolean integrated_ui_main_statusbar_is_showing (void)
+{
+  if (ui.statusbar)
+  {
+    return GTK_WIDGET_VISIBLE (ui.statusbar)? TRUE : FALSE;
+  }
+  return FALSE;
+}
+
+/* show() integrated UI main statusbar and set pulldown menu action. */
+void integrated_ui_main_statusbar_show (void)
+{
+  if (ui.statusbar)
+  {
+    GtkAction *action = NULL;
+    gtk_widget_show (GTK_WIDGET (ui.statusbar));
+    action = menus_get_action (VIEW_MAIN_STATUSBAR_ACTION);
+    if (action)
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+  }
+}
+
+/* hide() integrated UI main statusbar and reset pulldown menu action. */
+void integrated_ui_main_statusbar_hide (void)
+{
+  if (ui.statusbar)
+  {
+    GtkAction *action = NULL;
+    gtk_widget_hide (GTK_WIDGET (ui.statusbar));
+    action = menus_get_action (VIEW_MAIN_STATUSBAR_ACTION);
+    if (action)
+      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+  }
+}
+
+
+
