@@ -33,23 +33,35 @@
 #endif
 #include <gdk/gdk.h>
 #include <gtk/gtk.h> /* just for gtk_get_default_language() */
-
+#ifdef GDK_WINDOWING_WIN32
+/* avoid namespace clashes caused by inclusion of windows.h */
+#define Rectangle Win32Rectangle
+#define WIN32_LEAN_AND_MEAN
+#include <pango/pangowin32.h>
+#undef Rectangle
+#endif
 #include "font.h"
 #include "message.h"
 #include "intl.h"
-
-/** Define this if you want to use the layout cache to speed up output.
- *  Appears to be buggy, see bug #307320. */
-#undef LAYOUT_CACHE
+#include "textline.h"
 
 static PangoContext* pango_context = NULL;
 
 struct _DiaFont 
 {
-    GObject parent_instance;
+  GObject parent_instance;
 
-    PangoFontDescription* pfd;    
-    /* mutable */ char* legacy_name;    
+  PangoFontDescription* pfd;    
+  /* mutable */ char* legacy_name;
+
+  /* there is a difference between Pango's font size and Dia's font height */
+  /* Calculated  font_size is to make 'font_height = ascent + descent */
+  /* The font_height is used as default line height, there used to be a hard-coded size = 0.7 * height  */
+  /* before using pango_set_absolute_size() to overcome font size differences between renderers */
+  real height;
+  /* Need to load a font to query it's metrics */
+  PangoFont *loaded;
+  PangoFontMetrics *metrics;
 };
 
 
@@ -59,45 +71,20 @@ struct _DiaFont
  */
 static real global_zoom_factor = 20.0;
 
-#if 0
-/* For debugging: Sort families */
-static int
-cmp_families (const void *a, const void *b)
-{
-  const char *a_name = pango_font_family_get_name (*(PangoFontFamily **)a);
-  const char *b_name = pango_font_family_get_name (*(PangoFontFamily **)b);
-  
-  return g_utf8_collate (a_name, b_name);
-}
-
-/* For debugging: List all font families */
 static void
-list_families()
+dia_font_check_for_font(int font) 
 {
-  PangoFontFamily **families;
-  int nfamilies;
-  int i;
+  DiaFont *check;
+  PangoFont *loaded;
+  static real height = 1.0;
 
-  pango_context_list_families(dia_font_get_context(), &families, &nfamilies);
-  qsort (families, nfamilies, sizeof (PangoFontFamily *), cmp_families);
-  for (i = 0; i < nfamilies; i++) {
-    puts(pango_font_family_get_name(families[i]));
+  check = dia_font_new_from_style(font, height);
+  loaded = pango_context_load_font(dia_font_get_context(), check->pfd);
+  if (!loaded) {
+    message_error(_("Can't load font %s.\n"), dia_font_get_family(check));
+  } else {
+    g_object_unref (loaded);
   }
-  g_free(families);
-}
-#endif
-
-static void
-dia_font_check_for_font(int font) {
-    DiaFont *check;
-    PangoFont *loaded;
-
-    check = dia_font_new_from_style(font, 1.0);
-    loaded = pango_context_load_font(dia_font_get_context(),
-				     check->pfd);
-    if (loaded == NULL) {
-      message_error(_("Can't load font %s.\n"), dia_font_get_family(check));
-    }
 }
 
 void
@@ -111,16 +98,20 @@ dia_font_init(PangoContext* pcontext)
 }
 
 /* We might not need these anymore, when using FT2/Win32 fonts only */
-static GList *pango_contexts;
+static GList *pango_contexts = NULL;
 
+/*! DEPRECATED: there should be only one "measure context" */
 void
-dia_font_push_context(PangoContext *pcontext) {
+dia_font_push_context(PangoContext *pcontext) 
+{
+  dia_font_init (pcontext); /* not needed, just testing */
   pango_contexts = g_list_prepend(pango_contexts, pango_context);
   pango_context = pcontext;
   pango_context_set_language (pango_context, gtk_get_default_language ());
   g_object_ref(pcontext);
 }
 
+/*! DEPRECATED: there should be only one "measure context" */
 void
 dia_font_pop_context() {
   g_object_unref(pango_context);
@@ -130,19 +121,35 @@ dia_font_pop_context() {
 }
 
 PangoContext *
-dia_font_get_context() {
+dia_font_get_context() 
+{
   if (pango_context == NULL) {
+/* Maybe this one with pangocairo
+     dia_font_push_context (pango_cairo_font_map_create_context (pango_cairo_font_map_get_default()));
+ * but it gives: 
+     (lt-dia:30476): Pango-CRITICAL **: pango_renderer_draw_layout: assertion `PANGO_IS_RENDERER (renderer)' failed
+ */
 #ifdef HAVE_FREETYPE
     /* This is suggested by new Pango (1.2.4+), but doesn't get us the
      * right resolution:(
      dia_font_push_context(pango_ft2_font_map_create_context(pango_ft2_font_map_new()));
+     */
+    /* with 96x96 it gives consistent - too big - sizes with the cairo renderer, it was 75x75 with 0.96.x
+    dia_font_push_context(pango_ft2_get_context(96,96));
     */
     dia_font_push_context(pango_ft2_get_context(75,75));
 #else
-    dia_font_push_context(gdk_pango_context_get());
+    if (gdk_display_get_default ())
+      dia_font_push_context(gdk_pango_context_get());
+    else {
+#  ifdef GDK_WINDOWING_WIN32
+      dia_font_push_context(pango_win32_get_context ());
+#  else
+      g_warning ("dia_font_get_context() : not font context w/o display. Crashing soon.");
+#  endif
+    }
 #endif
   }
-
   return pango_context;
 }
 
@@ -197,20 +204,59 @@ dia_font_init_instance(DiaFont* font)
         /*GObject *gobject = G_OBJECT(font);  */
 }
 
+static void 
+dia_pfd_set_height(PangoFontDescription* pfd, real height)
+{ 
+  /* ONLY place for the magic factor! */
+  pango_font_description_set_absolute_size(pfd, dcm_to_pdu(height) * 0.8);
+}
+
+/*!
+ * In Dia a font is usually referred to by it's (line-) height, not it's size.
+ *
+ * This methods "calculates" the latter from the former. This used to be some magic factor of 0.7 which did not 
+ * solve the resolution dependencance of the former calculation. In fact there is new magic factor now because
+ * really calculating the font size from the height would involve two font loads which seem to be two expensive.
+ */
+static void
+_dia_font_adjust_size (DiaFont *font, real height, gboolean recalc_alwways)
+{
+  
+  if (font->height != height || !font->metrics || recalc_alwways) {
+    PangoFont *loaded;
+
+    dia_pfd_set_height (font->pfd, height);
+    /* need to load a font to get it's metrics */
+    loaded = font->loaded;
+    font->loaded = pango_context_load_font(dia_font_get_context(), font->pfd);
+    if (loaded) /* drop old reference */
+      g_object_unref (loaded);
+    if (font->metrics)
+      pango_font_metrics_unref (font->metrics);
+
+    /* caching metrics */
+    font->metrics = pango_font_get_metrics (font->loaded, NULL);
+    font->height = height;
+  }
+}
 DiaFont*
 dia_font_new(const char *family, DiaFontStyle style, real height)
 {
-  DiaFont* retval = dia_font_new_from_style(style, height);
+  DiaFont* font = dia_font_new_from_style(style, height);
+  gboolean changed;
 
-  pango_font_description_set_family(retval->pfd,family);
+  changed = family != NULL && strcmp (pango_font_description_get_family(font->pfd), family) != 0;
+  pango_font_description_set_family(font->pfd, family);
 
-  pango_context_load_font(dia_font_get_context(), retval->pfd);
+  if (changed)
+    _dia_font_adjust_size (font, font->height, TRUE);
 
-  return retval;
+  return font;
 }
 
 static void
-dia_pfd_set_family(PangoFontDescription* pfd, DiaFontFamily fam) {
+dia_pfd_set_family(PangoFontDescription* pfd, DiaFontFamily fam) 
+{
   switch (fam) {
   case DIA_FONT_SANS :
     pango_font_description_set_family(pfd, "sans");
@@ -222,13 +268,15 @@ dia_pfd_set_family(PangoFontDescription* pfd, DiaFontFamily fam) {
     pango_font_description_set_family(pfd, "monospace");
     break;
   default :
-          /* Pango does allow fonts without a name */
-      ;
+    /* Pango does _not_ allow fonts without a name (or at least they are not useful) */
+    pango_font_description_set_family(pfd, "sans");
+    break;
   }
 }
 
 static void
-dia_pfd_set_weight(PangoFontDescription* pfd, DiaFontWeight fw) {
+dia_pfd_set_weight(PangoFontDescription* pfd, DiaFontWeight fw) 
+{
   switch (fw) {
   case DIA_FONT_ULTRALIGHT :
     pango_font_description_set_weight(pfd, PANGO_WEIGHT_ULTRALIGHT);
@@ -261,7 +309,8 @@ dia_pfd_set_weight(PangoFontDescription* pfd, DiaFontWeight fw) {
 }
 
 static void
-dia_pfd_set_slant(PangoFontDescription* pfd, DiaFontSlant fo) {
+dia_pfd_set_slant(PangoFontDescription* pfd, DiaFontSlant fo) 
+{
   switch (fo) {
   case DIA_FONT_NORMAL :
     pango_font_description_set_style(pfd,PANGO_STYLE_NORMAL);
@@ -277,12 +326,6 @@ dia_pfd_set_slant(PangoFontDescription* pfd, DiaFontSlant fo) {
   }
 }
 
-static void dia_pfd_set_size(PangoFontDescription* pfd, real height)
-{ /* inline candidate... */
-  pango_font_description_set_size(pfd, dcm_to_pdu(height) );
-}
-
-
 DiaFont*
 dia_font_new_from_style(DiaFontStyle style, real height)
 {
@@ -295,41 +338,52 @@ dia_font_new_from_style(DiaFontStyle style, real height)
   dia_pfd_set_family(pfd,DIA_FONT_STYLE_GET_FAMILY(style));
   dia_pfd_set_weight(pfd,DIA_FONT_STYLE_GET_WEIGHT(style));
   dia_pfd_set_slant(pfd,DIA_FONT_STYLE_GET_SLANT(style));
-  dia_pfd_set_size(pfd,height);
+  dia_pfd_set_height(pfd,height);
   
   retval = DIA_FONT(g_object_new(DIA_TYPE_FONT, NULL));
-  
   retval->pfd = pfd;
+  _dia_font_adjust_size (retval, height, FALSE);
   retval->legacy_name = NULL;
   return retval;
 }
 
 DiaFont* dia_font_copy(const DiaFont* font)
 {
-    if (!font) return NULL;
-    return dia_font_new(dia_font_get_family(font),
-                            dia_font_get_style(font),
-                            dia_font_get_height(font));
+  if (!font) 
+    return NULL;
+  return dia_font_new(dia_font_get_family(font),
+                      dia_font_get_style(font),
+                      dia_font_get_height(font));
 }
 
 void
 dia_font_finalize(GObject* object)
 {
-    DiaFont* font;
-    font = DIA_FONT(object);
-    if (font->pfd) pango_font_description_free(font->pfd);
-    G_OBJECT_CLASS(parent_class)->finalize(object);
+  DiaFont* font = DIA_FONT(object);
+  
+  if (font->pfd)
+    pango_font_description_free(font->pfd);
+  font->pfd = NULL;
+  if (font->metrics)
+    pango_font_metrics_unref(font->metrics);
+  font->metrics = NULL;
+  if (font->loaded) 
+    g_object_unref(font->loaded);
+  font->loaded = NULL;
+  G_OBJECT_CLASS(parent_class)->finalize(object);
 }
 
-DiaFont* dia_font_ref(DiaFont* font)
+DiaFont *
+dia_font_ref(DiaFont* font)
 {
-    g_object_ref(G_OBJECT(font));
-    return font;
+  g_object_ref(G_OBJECT(font));
+  return font;
 }
 
-void dia_font_unref(DiaFont* font)
+void 
+dia_font_unref(DiaFont* font)
 { 
-    g_object_unref(G_OBJECT(font));
+  g_object_unref(G_OBJECT(font));
 }
 
 DiaFontStyle 
@@ -373,72 +427,118 @@ dia_font_get_description(const DiaFont* font)
 real
 dia_font_get_height(const DiaFont* font)
 {
+  return font->height;
+}
+
+real
+dia_font_get_size(const DiaFont* font)
+{
+  if (!pango_font_description_get_size_is_absolute (font->pfd))
+    g_warning ("dia_font_get_size() : no absolute size");
   return pdu_to_dcm(pango_font_description_get_size(font->pfd));
 }
 
 void
 dia_font_set_height(DiaFont* font, real height)
 {
-  pango_font_description_set_size(font->pfd, dcm_to_pdu(height));    
+  _dia_font_adjust_size (font, height, FALSE);
 }
 
 
 G_CONST_RETURN char*
 dia_font_get_psfontname(const DiaFont *font)
 {
-    /* FIXME: this will very likely not work ! */
-  return dia_font_get_legacy_name(font);
+  /* This hack corrects a couple fonts that were misnamed in
+   * earlier versions of Dia.   See bug #477079.
+   */
+  const char *fontname = dia_font_get_legacy_name(font);
+
+  if (!fontname)
+    return NULL;
+
+  if (strcmp(fontname, "NewCenturySchoolbook-Roman") == 0)
+    return "NewCenturySchlbk-Roman";
+  else if (strcmp(fontname, "NewCenturySchoolbook-Italic") == 0)
+    return "NewCenturySchlbk-Italic";
+  else if (strcmp(fontname, "NewCenturySchoolbook-Bold") == 0)
+    return "NewCenturySchlbk-Bold";
+  else if (strcmp(fontname, "NewCenturySchoolbook-BoldItalic") == 0)
+    return "NewCenturySchlbk-BoldItalic";
+  
+  return fontname;
 }
 
-void dia_font_set_any_family(DiaFont* font, const char* family)
+void 
+dia_font_set_any_family(DiaFont* font, const char* family)
 {
-  g_assert(font != NULL);
+  gboolean changed;
+  
+  g_return_if_fail(font != NULL);
+  
+  changed = strcmp (pango_font_description_get_family(font->pfd), family) != 0;
   pango_font_description_set_family(font->pfd, family);
+  if (changed) /* force recalculation on name change */
+    _dia_font_adjust_size (font, font->height, TRUE);
   if (font->legacy_name) {
-      g_free(font->legacy_name);
-      font->legacy_name = NULL;
+    g_free(font->legacy_name);
+    font->legacy_name = NULL;
   }
 }
 
-void dia_font_set_family(DiaFont* font, DiaFontFamily family)
+void 
+dia_font_set_family(DiaFont* font, DiaFontFamily family)
 {
-  g_assert(font != NULL);
+  g_return_if_fail(font != NULL);
+
   dia_pfd_set_family(font->pfd,family);  
   if (font->legacy_name) {
-      g_free(font->legacy_name);
-      font->legacy_name = NULL;
+    g_free(font->legacy_name);
+    font->legacy_name = NULL;
   }
 }
 
-void dia_font_set_weight(DiaFont* font, DiaFontWeight weight)
+void 
+dia_font_set_weight(DiaFont* font, DiaFontWeight weight)
 {
-  g_assert(font != NULL);    
+  DiaFontWeight old_weight = DIA_FONT_STYLE_GET_WEIGHT(dia_font_get_style(font));
+  g_return_if_fail(font != NULL);    
   dia_pfd_set_weight(font->pfd,weight);
+  if (old_weight != weight)
+    _dia_font_adjust_size (font, font->height, TRUE);
 }
 
-void dia_font_set_slant(DiaFont* font, DiaFontSlant slant)
+void 
+dia_font_set_slant(DiaFont* font, DiaFontSlant slant)
 {
-  g_assert(font != NULL);    
+  DiaFontSlant old_slant = DIA_FONT_STYLE_GET_SLANT(dia_font_get_style(font));
+  g_return_if_fail(font != NULL);    
   dia_pfd_set_slant(font->pfd,slant);
+  if (slant != old_slant)
+    _dia_font_adjust_size (font, font->height, TRUE);
 }
 
 
-struct weight_name { DiaFontWeight fw; const char *name; };    
-static const struct weight_name weight_names[] = {
-    {DIA_FONT_ULTRALIGHT, "200"},
-    {DIA_FONT_LIGHT,"300"},
-    {DIA_FONT_WEIGHT_NORMAL,"normal"},
-    {DIA_FONT_WEIGHT_NORMAL,"400"},
-    {DIA_FONT_MEDIUM, "500"},
-    {DIA_FONT_DEMIBOLD, "600"},
-    {DIA_FONT_BOLD, "700"},
-    {DIA_FONT_ULTRABOLD, "800"},
-    {DIA_FONT_HEAVY, "900"},
-    {0,NULL}};
+typedef struct _WeightName { 
+  DiaFontWeight fw; 
+  const char *name; 
+} WeightName;
+static const WeightName weight_names[] = {
+  {DIA_FONT_ULTRALIGHT, "200"},
+  {DIA_FONT_LIGHT,"300"},
+  {DIA_FONT_WEIGHT_NORMAL,"normal"},
+  {DIA_FONT_WEIGHT_NORMAL,"400"},
+  {DIA_FONT_MEDIUM, "500"},
+  {DIA_FONT_DEMIBOLD, "600"},
+  {DIA_FONT_BOLD, "700"},
+  {DIA_FONT_ULTRABOLD, "800"},
+  {DIA_FONT_HEAVY, "900"},
+  {0,NULL}
+};
 
-G_CONST_RETURN char *dia_font_get_weight_string(const DiaFont* font)
+G_CONST_RETURN char *
+dia_font_get_weight_string(const DiaFont* font)
 {
-    const struct weight_name* p;
+    const WeightName* p;
     DiaFontWeight fw = DIA_FONT_STYLE_GET_WEIGHT(dia_font_get_style(font));
     
     for (p = weight_names; p->name != NULL; ++p) {
@@ -447,9 +547,11 @@ G_CONST_RETURN char *dia_font_get_weight_string(const DiaFont* font)
     return "normal";
 }
 
-void dia_font_set_weight_from_string(DiaFont* font, const char* weight) {
+void 
+dia_font_set_weight_from_string(DiaFont* font, const char* weight) 
+{
     DiaFontWeight fw = DIA_FONT_WEIGHT_NORMAL;
-    const struct weight_name* p;
+    const WeightName* p;
 
     for (p = weight_names; p->name != NULL; ++p) {
         if (0 == strncmp(weight,p->name,8)) {
@@ -462,42 +564,49 @@ void dia_font_set_weight_from_string(DiaFont* font, const char* weight) {
 }
 
 
-struct slant_name { DiaFontSlant fo; const char *name; };    
-static const struct slant_name slant_names[] = {
-    { DIA_FONT_NORMAL, "normal"},
-    { DIA_FONT_OBLIQUE, "oblique"},
-    { DIA_FONT_ITALIC, "italic"},
-    { 0, NULL} };
+typedef struct _SlantName { 
+  DiaFontSlant fo; 
+  const char *name; 
+} SlantName;
+static const SlantName slant_names[] = {
+  { DIA_FONT_NORMAL, "normal" },
+  { DIA_FONT_OBLIQUE, "oblique" },
+  { DIA_FONT_ITALIC, "italic" },
+  { 0, NULL} 
+};
 
 G_CONST_RETURN char *
 dia_font_get_slant_string(const DiaFont* font)
 {
-    const struct slant_name* p;
-    DiaFontSlant fo =
+  const SlantName* p;
+  DiaFontSlant fo =
     DIA_FONT_STYLE_GET_SLANT(dia_font_get_style(font));
     
-    for (p = slant_names; p->name != NULL; ++p) {
-        if (p->fo == fo) return p->name;
-    }
-    return "normal";
+  for (p = slant_names; p->name != NULL; ++p) {
+    if (p->fo == fo) 
+      return p->name;
+  }
+  return "normal";
 }
 
-void dia_font_set_slant_from_string(DiaFont* font, const char* obli) {
-    DiaFontSlant fo = DIA_FONT_NORMAL;
-    const struct slant_name* p;
+void 
+dia_font_set_slant_from_string(DiaFont* font, const char* obli) 
+{
+  DiaFontSlant fo = DIA_FONT_NORMAL;
+  const SlantName* p;
 
-    DiaFontStyle old_style;
-    DiaFontSlant old_fo;
-    old_style = dia_font_get_style(font);
-    old_fo = DIA_FONT_STYLE_GET_SLANT(old_style);
+  DiaFontStyle old_style;
+  DiaFontSlant old_fo;
+  old_style = dia_font_get_style(font);
+  old_fo = DIA_FONT_STYLE_GET_SLANT(old_style);
     
-    for (p = slant_names; p->name != NULL; ++p) {
-        if (0 == strncmp(obli,p->name,8)) {
-            fo = p->fo;
-            break;
-        }
+  for (p = slant_names; p->name != NULL; ++p) {
+    if (0 == strncmp(obli,p->name,8)) {
+      fo = p->fo;
+      break;
     }
-    dia_font_set_slant(font,fo);
+  }
+  dia_font_set_slant(font,fo);
 }
 
 
@@ -508,358 +617,224 @@ void dia_font_set_slant_from_string(DiaFont* font, const char* obli) {
 real
 dia_font_string_width(const char* string, DiaFont *font, real height)
 {
-    return dia_font_scaled_string_width(string,font,height,global_zoom_factor);
+  real result = 0;
+  if (string && *string) {
+    TextLine *text_line = text_line_new(string, font, height);
+    result = text_line_get_width(text_line);
+    text_line_destroy(text_line);
+  }
+  return result;
 }
 
 real
 dia_font_ascent(const char* string, DiaFont* font, real height)
 {
-    return dia_font_scaled_ascent(string,font,height,global_zoom_factor);
-}
-
-real dia_font_descent(const char* string, DiaFont* font, real height)
-{
-    return dia_font_scaled_descent(string,font,height,global_zoom_factor);
-}
-
-typedef struct {
-  gchar *string;
-  DiaFont *font;
-  PangoLayout *layout;
-  int usecount;
-} LayoutCacheItem;
-
-static GHashTable *layoutcache;
-
-static gboolean
-layout_cache_equals(gconstpointer e1, gconstpointer e2) 
-{
-  LayoutCacheItem *i1 = (LayoutCacheItem*)e1,
-    *i2 = (LayoutCacheItem*)e2;
-
-  /* don't try strcmp() with null pointers ... */
-  if (!i1->string || i2->string)
-    return FALSE;
-  
-  return strcmp(i1->string, i2->string) == 0 &&
-    pango_font_description_equal(i1->font->pfd, i2->font->pfd);
-}
-
-static guint
-layout_cache_hash(gconstpointer el) 
-{
-  LayoutCacheItem *item = (LayoutCacheItem*)el;
-
-  return g_str_hash(item->string) ^
-    pango_font_description_hash(item->font->pfd);
-}
-
-static long layout_cache_last_use;
-
-static gboolean
-layout_cache_cleanup_entry(gpointer key, gpointer value, gpointer data)
-{
-  LayoutCacheItem *item = (LayoutCacheItem*)value;
-  /** Remove unused items */
-  if (item->usecount == 0) return TRUE;
-  item->usecount = 0;
-  return FALSE;
-}
-
-/** The actual hash cleanup, called when idle. */
-static gboolean
-layout_cache_cleanup_idle(gpointer data)
-{
-  GHashTable *table = (GHashTable*)(data);
-
-  g_hash_table_foreach_remove(table, layout_cache_cleanup_entry, NULL);
-  return FALSE;
-}
-
-/** Every ten minutes, clean up those strings that haven't seen use since
- * last cleanup.
- */
-static gboolean
-layout_cache_cleanup(gpointer data)
-{
-  /* Only cleanup if there has been font activity since last cleanup */
-  if (time(0) - layout_cache_last_use < 10) {
-    /* Don't go directly to cleanup, wait till there's a pause. */
-    g_idle_add(layout_cache_cleanup_idle, data);
+  if (font->metrics) {
+    real ascent = pdu_to_dcm(pango_font_metrics_get_ascent (font->metrics));
+    return ascent * (height / font->height);
+  } else {
+    /* previous, _expensive_ but string specific way */
+    TextLine *text_line = text_line_new(string, font, height);
+    real result = text_line_get_ascent(text_line);
+    text_line_destroy(text_line);
+    return result;
   }
-  /* Keep doing this */
-  return TRUE;
 }
 
-static void
-layout_cache_free_key(gpointer data)
+real 
+dia_font_descent(const char* string, DiaFont* font, real height)
 {
-    LayoutCacheItem *item = (LayoutCacheItem*)data;
-
-    if (item->string != NULL) {
-      g_free(item->string);
-      item->string = NULL;
-    }
-
-    if (item->font != NULL) {
-      dia_font_unref(item->font);
-      item->font = NULL;
-    }
-
-    if (item->layout != NULL) {
-      g_object_unref(item->layout);
-      item->layout = NULL;
-    }
-    g_free(item);
+  if (font->metrics) {
+    real descent = pdu_to_dcm(pango_font_metrics_get_descent (font->metrics));
+    return descent * (height / font->height);
+  } else {
+    /* previous, _expensive_ but string specific way */
+    TextLine *text_line = text_line_new(string, font, height);
+    real result = text_line_get_descent(text_line);
+    text_line_destroy(text_line);
+    return result;
+  }
 }
 
 
 PangoLayout*
 dia_font_build_layout(const char* string, DiaFont* font, real height)
 {
-    PangoLayout* layout;
-    PangoAttrList* list;
-    PangoAttribute* attr;
-    guint length;
-    gchar *desc = NULL;
+  PangoLayout* layout;
+  PangoAttrList* list;
+  PangoAttribute* attr;
+  guint length;
+  PangoFontDescription *pfd;
+  real factor;
 
-#ifdef LAYOUT_CACHE
-    LayoutCacheItem *cached, *item;
+  layout = pango_layout_new(dia_font_get_context());
 
-    layout_cache_last_use = time(0);
-    if (layoutcache == NULL) {
-      /** Note that key and value are the same -- it's a HashSet */
-      layoutcache = g_hash_table_new_full(layout_cache_hash,
-					  layout_cache_equals,
-					  layout_cache_free_key,
-					  NULL);
-      /** Check for cache cleanup every 10 seconds. */
-      /** This frequent a check is really a hack while we figure out the
-       *  exact problems with reffing the fonts.
-       *  Note to self:  The equals function should compare pfd's, but
-       *  then DiaFonts are freed too early. 
-       */
-      g_timeout_add(10*1000, layout_cache_cleanup, (gpointer)layoutcache);
-    }
-#endif
-
-    height *= 0.7;
-    dia_font_set_height(font, height);
-
-#ifdef LAYOUT_CACHE
-    item = g_new0(LayoutCacheItem,1);
-    item->string = g_strdup(string);
-    item->font = font;
-    
-    /* If it's in the cache, use that instead. */
-    cached = g_hash_table_lookup(layoutcache, item);
-    if (cached != NULL) {
-      g_object_ref(cached->layout);
-      g_free(item->string);
-      g_free(item);
-      cached->usecount ++;
-      return cached->layout;
-    }
-
-    item->font = dia_font_copy(font);
-    dia_font_ref(item->font);
-#endif
-
-    /* This could should account for DPI, but it doesn't do right.  Grrr...
-    {
-      GdkScreen *screen = gdk_screen_get_default();
-      real dpi = gdk_screen_get_width(screen)/
-		 (gdk_screen_get_width_mm(screen)/25.4);
-      printf("height = %f, dpi = %f, new height = %f\n",
-	     height, dpi, height * (dpi/100));
-      height *= dpi/100;
-    }
-    */
-
-    layout = pango_layout_new(dia_font_get_context());
-
-    length = string ? strlen(string) : 0;
-    pango_layout_set_text(layout, string, length);
+  length = string ? strlen(string) : 0;
+  pango_layout_set_text(layout, string, length);
         
-    list = pango_attr_list_new();
-    desc = g_utf8_strdown(pango_font_description_get_family(font->pfd), -1);
-    pango_font_description_set_family(font->pfd, desc);
-    g_free(desc);
-    attr = pango_attr_font_desc_new(font->pfd);
-    attr->start_index = 0;
-    attr->end_index = length;
-    pango_attr_list_insert(list,attr); /* eats attr */
-    
-    pango_layout_set_attributes(layout,list);
-    pango_attr_list_unref(list);
+  list = pango_attr_list_new();
 
-    pango_layout_set_indent(layout,0);
-    pango_layout_set_justify(layout,FALSE);
-    pango_layout_set_alignment(layout,PANGO_ALIGN_LEFT);
+  pfd = pango_font_description_copy (font->pfd);
+  /* account for difference between size and height as well as between font height and given one */
+  factor = dia_font_get_size(font) / dia_font_get_height (font);
+  pango_font_description_set_absolute_size (pfd, dcm_to_pdu (height) * factor);
+  attr = pango_attr_font_desc_new(pfd);
+  pango_font_description_free (pfd);
+
+  attr->start_index = 0;
+  attr->end_index = length;
+  pango_attr_list_insert(list,attr); /* eats attr */
+    
+  pango_layout_set_attributes(layout,list);
+  pango_attr_list_unref(list);
+
+  pango_layout_set_indent(layout,0);
+  pango_layout_set_justify(layout,FALSE);
+  pango_layout_set_alignment(layout,PANGO_ALIGN_LEFT);
   
-#ifdef LAYOUT_CACHE
-    item->layout = layout;
-    g_object_ref(layout);
-    item->usecount = 1;
-    g_hash_table_replace(layoutcache, item, item);
-#endif
-
-    return layout;
+  return layout;
 }
 
-/* ************************************************************************ */
-/* scaled versions of the utility routines                                  */
-/* ************************************************************************ */
-
-void
-dia_font_set_nominal_zoom_factor(real size_one)
-{ global_zoom_factor = size_one; }
-
-
-
-real
-dia_font_scaled_string_width(const char* string, DiaFont *font, real height,
-                            real zoom_factor)
+/** Find the offsets of the individual letters in the iter and place them
+ * in an array.
+ * This currently assumes only one run per iter, which is all we can input.
+ * @param iter The PangoLayoutIter to count characters in.
+ * @param offsets The place to return the offsets
+ * @param n_offsets The place to return the number of offsets
+ */
+static void
+get_string_offsets(PangoLayoutIter *iter, real** offsets, int* n_offsets)
 {
-    int lw,lh;
-    real result;
-    PangoLayout* layout;
+  int i;
+  PangoLayoutLine*   line = pango_layout_iter_get_line(iter);
+  PangoGlyphItem* item;
+  PangoGlyphString* string;
 
-    if (string == NULL || string[0] == '\0') {
-      return 0.0;
-    }
-
-    layout = dia_font_scaled_build_layout(string, font, height, zoom_factor);
-    pango_layout_get_size(layout,&lw,&lh);
-    g_object_unref(G_OBJECT(layout));
-    
-    result = pdu_to_dcm(lw);
-    /* Scale the result back for the zoom factor */
-    result /= (zoom_factor/global_zoom_factor);
-    return result;
-}
-
-static gboolean
-dia_font_vertical_extents(const char* string, DiaFont* font,
-                          real height, real zoom_factor,
-                          guint line_no,
-                          real* top, real* baseline, real* bottom)
-{
-    PangoRectangle ink_rect,logical_rect;
-    PangoLayout* layout;
-    PangoLayoutIter* iter;
-    guint i;
-
-    if (string == NULL || string[0] == '\0') {
-      return FALSE;
-    }
-
-    layout = dia_font_scaled_build_layout(string, font,
-                                          height, zoom_factor);
-    iter = pango_layout_get_iter(layout);
-    for (i = 0; i < line_no; ++i) {
-       if (!pango_layout_iter_next_line(iter)) {
-           pango_layout_iter_free(iter);
-           g_object_unref(G_OBJECT(layout));
-           return FALSE;
-       }
-    }
-    
-    pango_layout_iter_get_line_extents(iter,&ink_rect,&logical_rect);
-
-    *top = pdu_to_dcm(logical_rect.y);
-    *bottom = pdu_to_dcm(logical_rect.y + logical_rect.height);
-    *baseline = pdu_to_dcm(pango_layout_iter_get_baseline(iter));
-
-    pango_layout_iter_free(iter);
-    g_object_unref(G_OBJECT(layout));
-
-    return TRUE;
-}
-    
-
-real
-dia_font_scaled_ascent(const char* string, DiaFont* font, real height,
-                      real zoom_factor)
-{
-  real top,bline,bottom;
-  if (!string || string[0] == '\0') {
-    /* This hack won't work for fonts that don't cover ASCII */
-    dia_font_vertical_extents("XjgM149",font,height,zoom_factor,
-			      0,&top,&bline,&bottom);
-  } else {
-    dia_font_vertical_extents(string,font,height,zoom_factor,
-			      0,&top,&bline,&bottom);
+  if(0 == line->length)
+  {
+    *n_offsets = 0;
+    return;
   }
-  return (bline-top)/(zoom_factor/global_zoom_factor);
-}
+  item = (PangoGlyphItem*)line->runs->data;
+  string = item->glyphs;
 
-real dia_font_scaled_descent(const char* string, DiaFont* font,
-                            real height, real zoom_factor)
-{
-  real top,bline,bottom;
+  *n_offsets = string->num_glyphs;
+  *offsets = g_new(real, *n_offsets);
 
-  if (!string || string[0] == '\0') {
-    /* This hack won't work for fonts that don't cover ASCII */
-    dia_font_vertical_extents("XjgM149",font,height,zoom_factor,
-			      0,&top,&bline,&bottom);
-  } else {
-    dia_font_vertical_extents(string,font,height,zoom_factor,
-                              0,&top,&bline,&bottom);
+  for (i = 0; i < string->num_glyphs; i++) {
+    PangoGlyphGeometry geom = string->glyphs[i].geometry;
+    
+    (*offsets)[i] = pdu_to_dcm(geom.width) / global_zoom_factor;
   }
-  return (bottom-bline)/(zoom_factor/global_zoom_factor);
 }
 
-PangoLayout*
-dia_font_scaled_build_layout(const char* string, DiaFont* font,
-                            real height, real zoom_factor)
+/** Copy the offsets into a storage object, adjusting for exaggerated size.
+ */
+static void
+get_layout_offsets(PangoLayoutLine *line, PangoLayoutLine **layout_line)
 {
-    DiaFont* altered_font;
-    real scaling;
-    real nozoom_width;
-    real target_zoomed_width;
-    PangoLayout* layout;
-    real altered_scaling;
-    real real_width;
+  /* Some info not copied. */
+  GSList *layout_runs = NULL;
+  GSList *runs = line->runs;
 
-    scaling = zoom_factor / global_zoom_factor;
-    if (fabs(1.0 - scaling) < 1E-7) {
-        return dia_font_build_layout(string,font,height);
+  *layout_line = g_new0(PangoLayoutLine, 1);
+
+  /* A LayoutLine contains a GSList runs of PangoGlyphItems.
+   * Each PangoGlyphItem contains an array glyphs of PangoGlyphString.
+   * This array is run->item->num_chars long?
+   * Each PangoGlyphString contains an array glyphs of PangoGlyphInfo.
+   * This array is run->glyphs[i].num_glyphs long.
+   */
+  for (; runs != NULL; runs = g_slist_next(runs)) {
+    PangoGlyphItem *run = (PangoGlyphItem *) runs->data;
+    PangoGlyphItem *layout_run = g_new0(PangoGlyphItem, 1);
+    int j;
+    /* Make single pointer */
+    PangoGlyphString *glyph_string = run->glyphs;
+    PangoGlyphString *layout_glyph_string;
+      
+    layout_run->glyphs = g_new0(PangoGlyphString, 1);
+    layout_glyph_string = layout_run->glyphs;
+
+    layout_glyph_string->num_glyphs = glyph_string->num_glyphs;
+    layout_glyph_string->glyphs = 
+      g_new0(PangoGlyphInfo, glyph_string->num_glyphs);
+    for (j = 0; j < layout_glyph_string->num_glyphs; j++) {
+      PangoGlyphInfo *info = &glyph_string->glyphs[j];
+      PangoGlyphInfo *layout_info = &layout_glyph_string->glyphs[j];
+      layout_info->geometry.width = info->geometry.width;
+      layout_info->geometry.x_offset = info->geometry.x_offset;
+      layout_info->geometry.y_offset = info->geometry.y_offset;
     }
-
-    nozoom_width = dia_font_string_width(string,font,height);
-    target_zoomed_width = nozoom_width * scaling;
-
-        /* First try: no tweaks. */
-    real_width = dia_font_string_width(string,font, height * scaling);
-    if (real_width <= target_zoomed_width) {
-        return dia_font_build_layout(string,font,height*scaling);
-    }
-
-    altered_font = dia_font_copy(font);
-
-        /* Last try. Using the "reduce overall size" strategy. */
-    for (altered_scaling = scaling;
-         altered_scaling > (scaling / 2);
-         altered_scaling *= (target_zoomed_width/real_width>0.98?0.98:
-			     target_zoomed_width/real_width) ) {
-      real_width = dia_font_string_width(string,font,
-					 height * altered_scaling);
-
-        if (real_width <= target_zoomed_width) {
-            layout = dia_font_build_layout(string,altered_font,
-                                          height*altered_scaling);
-            dia_font_unref(altered_font);
-            return layout;
-        }
-    }
-
-    /* Everything has failed. Returning non-tweaked variant. */
-    g_warning("Failed to appropriately tweak zoomed font for zoom factor %f.", zoom_factor);
-    dia_font_unref(altered_font);
-    return dia_font_build_layout(string,font,height*scaling);    
+    layout_runs = g_slist_append(layout_runs, layout_run);
+  }
+  (*layout_line)->runs = layout_runs;
 }
 
+/** Get size information for the given string, font and height.
+ *
+ * @returns an array of offsets of the individual glyphs in the layout.
+ */
+real*
+dia_font_get_sizes(const char* string, DiaFont *font, real height,
+		   real *width, real *ascent, real *descent, int *n_offsets,
+		   PangoLayoutLine **layout_offsets)
+{
+  PangoLayout* layout;
+  PangoLayoutIter* iter;
+  real top, bline, bottom;
+  const gchar* non_empty_string;
+  PangoRectangle ink_rect,logical_rect;
+  real* offsets = NULL; /* avoid: 'offsets' may be used uninitialized in this function */
+
+  /* We need some reasonable ascent/descent values even for empty strings. */
+  if (string == NULL || string[0] == '\0') {
+    non_empty_string = "XjgM149";
+  } else {
+    non_empty_string = string;
+  }
+  layout = dia_font_build_layout(non_empty_string, font, height * global_zoom_factor);
+  
+  /* Only one line here ? */
+  iter = pango_layout_get_iter(layout);
+
+  pango_layout_iter_get_line_extents(iter, &ink_rect, &logical_rect);
+
+  top = pdu_to_dcm(logical_rect.y) / global_zoom_factor;
+  bottom = pdu_to_dcm(logical_rect.y + logical_rect.height) / global_zoom_factor;
+  bline = pdu_to_dcm(pango_layout_iter_get_baseline(iter)) / global_zoom_factor;
+
+  get_string_offsets(iter, &offsets, n_offsets);
+  get_layout_offsets(pango_layout_get_line(layout, 0), layout_offsets);
+
+  /* FIXME: the above assumption of 'one line' is wrong. At least calculate the overall width correctly
+   * to avoid text overflowing its box, like in bug #482585 */
+  while (pango_layout_iter_next_line (iter)) {
+    PangoRectangle more_ink_rect, more_logical_rect;
+    
+    pango_layout_iter_get_line_extents(iter, &more_ink_rect, &more_logical_rect);
+    if (more_logical_rect.width > logical_rect.width)
+      logical_rect.width = more_logical_rect.width;
+    /* also calculate for the ink rect (true space needed for drawing the glyphs) */
+    if (more_ink_rect.width > ink_rect.width)
+      ink_rect.width = more_ink_rect.width;
+  }
+
+  pango_layout_iter_free(iter);
+  g_object_unref(G_OBJECT(layout));
+
+  *ascent = bline-top;
+  *descent = bottom-bline;
+  if (non_empty_string != string) {
+    *width = 0.0;
+  } else {
+    /* take the bigger rectangle to avoid cutting of any part of the string */
+    *width = pdu_to_dcm(logical_rect.width > ink_rect.width ? logical_rect.width : ink_rect.width) / global_zoom_factor;
+  }
+  return offsets;
+}
+
+
 /**
  * Compatibility with older files out of pre Pango Time. 
  * Make old files look as similar as possible
@@ -867,6 +842,15 @@ dia_font_scaled_build_layout(const char* string, DiaFont* font,
  * duplicates the one with the preferred newname comes first.
  *
  * FIXME: DIA_FONT_FAMILY_ANY in the list below does mean noone knows better
+ *
+ * The PostScript names can be found on page 139 of 
+ * http://partners.adobe.com/public/developer/en/ps/PS3010and3011.Supplement.pdf
+ * 
+ * Note that these are not strictly the Adobe names, as a few were used
+ * incorrectly and are kept for backwards compatibility.  The latin-1
+ * postscript renderer mangles these.
+ *
+ * Some additional fairly standard fonts for asian scripts are also added.
  */
 static struct _legacy_font {
   gchar*       oldname;
@@ -981,7 +965,7 @@ dia_font_get_legacy_name(const DiaFont *font)
   family = dia_font_get_family (font);
   style = dia_font_get_style (font);
   for (i = 0; i < G_N_ELEMENTS(legacy_fonts); i++) {
-    if (0 == g_strcasecmp (legacy_fonts[i].newname, family)) {
+    if (0 == g_ascii_strcasecmp (legacy_fonts[i].newname, family)) {
       /* match weight and slant */
       DiaFontStyle st = legacy_fonts[i].style;
       if ((DIA_FONT_STYLE_GET_SLANT(style) | DIA_FONT_STYLE_GET_WEIGHT(style))
