@@ -24,7 +24,6 @@
 #include <pango/pango.h>
 #include "dia-enums.h"
 #include "geometry.h"
-#include "diavar.h"
 
 /* Do NOT put these strings through the .po mechanism. If you need to add
    non-Roman alternatives, please insert them in the list */
@@ -108,7 +107,6 @@ void dia_font_push_context(PangoContext *pcontext);
 void dia_font_pop_context(void);
 /* Retrieve the current context (used for the font widget) */
 PangoContext *dia_font_get_context(void);
-                         
                              
     /* Get a font matching family,style,height. MUST be freed with
        dia_font_unref(). */
@@ -144,6 +142,8 @@ G_CONST_RETURN PangoFontDescription *dia_font_get_description (const DiaFont* fo
 real dia_font_get_height(const DiaFont* font);
     /* Change the height inside a font record. */
 void dia_font_set_height(DiaFont* font, real height);
+    /* Delivers the size of the font */                         
+real dia_font_get_size(const DiaFont* font);
 
     /* Changes the slant of an existing font */
 void dia_font_set_slant(DiaFont* font, DiaFontSlant slant);
@@ -199,9 +199,5 @@ real* dia_font_get_sizes(const char* string, DiaFont *font, real height,
 /* -------- Font and string functions - scaled versions.
    Use these version in Renderers, exclusively. */
 
-    /* Call once at the beginning of a rendering pass, to let dia know
-     what is 1:1 scale. zoom_factor will then be divided by size_one. */
-void dia_font_set_nominal_zoom_factor(real size_one);
-
 
 #endif /* FONT_H */
