@@ -21,6 +21,7 @@
 #include "intl.h"
 #undef GTK_DISABLE_DEPRECATED /* GtkOptionMenu, ... */
 #include "widgets.h"
+#include "units.h"
 #include "message.h"
 #include "dia_dirs.h"
 #include "arrows.h"
@@ -38,7 +39,34 @@
 #include <time.h>
 #include <gdk/gdkkeysyms.h>
 
-#include "diagtkfontsel.h"
+/* hidden internals for two reasosn: 
+ * - noone is supposed to mess with the internals 
+ * - it uses deprecated stuff
+ */
+struct _DiaDynamicMenu {
+  GtkOptionMenu parent;
+
+  GList *default_entries;
+
+  DDMCreateItemFunc create_func;
+  DDMCallbackFunc activate_func;
+  gpointer userdata;
+
+  GtkMenuItem *other_item;
+
+  gchar *persistent_name;
+  gint cols;
+
+  gchar *active;
+  /** For the list-based versions, these are the options */
+  GList *options;
+
+};
+
+struct _DiaDynamicMenuClass {
+  GtkOptionMenuClass parent_class;
+};
+
 
 /************* DiaSizeSelector: ***************/
 /* A widget that selects two sizes, width and height, optionally keeping
@@ -60,6 +88,13 @@ struct _DiaSizeSelectorClass
   GtkHBoxClass parent_class;
 };
 
+enum {
+    DSS_VALUE_CHANGED,
+    DSS_LAST_SIGNAL
+};
+
+static guint dss_signals[DSS_LAST_SIGNAL] = { 0 };
+
 static void
 dia_size_selector_unrealize(GtkWidget *widget)
 {
@@ -75,6 +110,14 @@ dia_size_selector_class_init (DiaSizeSelectorClass *class)
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   widget_class->unrealize = dia_size_selector_unrealize;
+
+  dss_signals[DSS_VALUE_CHANGED]
+      = g_signal_new("value-changed",
+		     G_TYPE_FROM_CLASS(class),
+		     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
 }
 
 static void
@@ -82,7 +125,8 @@ dia_size_selector_adjust_width(DiaSizeSelector *ss)
 {
   real height = 
     gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(ss->height));
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ss->width), height*ss->ratio);
+  if (fabs(ss->ratio) > 1e-6)
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(ss->width), height*ss->ratio);
 }
 
 static void
@@ -90,31 +134,36 @@ dia_size_selector_adjust_height(DiaSizeSelector *ss)
 {
   real width = 
     gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(ss->width));
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ss->height), width/ss->ratio);
+  if (fabs(ss->ratio) > 1e-6)
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(ss->height), width/ss->ratio);
 }
 
 static void
 dia_size_selector_ratio_callback(GtkAdjustment *limits, gpointer userdata) 
 {
-  static gboolean in_progress;
+  static gboolean in_progress = FALSE;
   DiaSizeSelector *ss = DIA_SIZE_SELECTOR(userdata);
 
   ss->last_adjusted = limits;
 
-  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ss->aspect_locked)) 
-      || ss->ratio == 0.0)
-    return;
+  if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ss->aspect_locked)) 
+      && ss->ratio != 0.0) {
 
-  if (in_progress) return;
-  in_progress = TRUE;
+    if (in_progress) 
+      return;
+    in_progress = TRUE;
 
-  if (limits == gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ss->width))) {
-    dia_size_selector_adjust_height(ss);
-  } else {
-    dia_size_selector_adjust_width(ss);
+    if (limits == gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ss->width))) {
+      dia_size_selector_adjust_height(ss);
+    } else {
+      dia_size_selector_adjust_width(ss);
+    }
+
+    in_progress = FALSE;
   }
 
-  in_progress = FALSE;
+  g_signal_emit(ss, dss_signals[DSS_VALUE_CHANGED], 0);
+
 }
 
 /** Update the ratio of this DSS to be the ratio of width to height.
@@ -149,7 +198,7 @@ dia_size_selector_init (DiaSizeSelector *ss)
   ss->ratio = 0.0;
   /* Here's where we set up the real thing */
   adj = GTK_ADJUSTMENT(gtk_adjustment_new(1.0, 0.01, 10,
-					  0.1, 1.0, 1.0));
+					  0.1, 1.0, 0));
   ss->width = GTK_SPIN_BUTTON(gtk_spin_button_new(adj, 1.0, 2));
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(ss->width), TRUE);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(ss->width), TRUE);
@@ -157,7 +206,7 @@ dia_size_selector_init (DiaSizeSelector *ss)
   gtk_widget_show(GTK_WIDGET(ss->width));
 
   adj = GTK_ADJUSTMENT(gtk_adjustment_new(1.0, 0.01, 10,
-					  0.1, 1.0, 1.0));
+					  0.1, 1.0, 0));
   ss->height = GTK_SPIN_BUTTON(gtk_spin_button_new(adj, 1.0, 2));
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(ss->height), TRUE);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(ss->height), TRUE);
@@ -259,6 +308,10 @@ dia_size_selector_get_size(DiaSizeSelector *ss, real *width, real *height)
 
 /************* DiaFontSelector: ***************/
 
+/* Should these structs be in widgets.h instead? : 
+ * _no_ they should not, noone is supposed to mess with the internals, ever heard of information hiding? 
+ */
+
 struct _DiaFontSelector
 {
   GtkHBox hbox;
@@ -273,6 +326,13 @@ struct _DiaFontSelectorClass
   GtkHBoxClass parent_class;
 };
 
+enum {
+    DFONTSEL_VALUE_CHANGED,
+    DFONTSEL_LAST_SIGNAL
+};
+
+static guint dfontsel_signals[DFONTSEL_LAST_SIGNAL] = { 0 };
+
 
 /* New and improved font selector:  Contains the three standard fonts
  * and an 'Other fonts...' entry that opens the font dialog.  The fonts
@@ -291,8 +351,9 @@ struct _DiaFontSelectorClass
  * +----------------+
  */
 
-static void dia_font_selector_fontmenu_callback(DiaDynamicMenu *button,
-						const gchar *fontname,
+static void dia_font_selector_fontmenu_callback(DiaDynamicMenu *ddm,
+						gpointer data);
+static void dia_font_selector_stylemenu_callback(GtkMenu *menu,
 						gpointer data);
 static void dia_font_selector_set_styles(DiaFontSelector *fs,
 					 const gchar *name,
@@ -307,6 +368,14 @@ dia_font_selector_class_init (DiaFontSelectorClass *class)
   GtkObjectClass *object_class;
   
   object_class = (GtkObjectClass*) class;
+
+  dfontsel_signals[DFONTSEL_VALUE_CHANGED]
+      = g_signal_new("value_changed",
+		     G_TYPE_FROM_CLASS(class),
+		     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
 }
 
 static int
@@ -314,7 +383,7 @@ dia_font_selector_sort_fonts(const void *p1, const void *p2)
 {
   const gchar *n1 = pango_font_family_get_name(PANGO_FONT_FAMILY(*(void**)p1));
   const gchar *n2 = pango_font_family_get_name(PANGO_FONT_FAMILY(*(void**)p2));
-  return g_strcasecmp(n1, n2);
+  return g_ascii_strcasecmp(n1, n2);
 }
 
 static gchar*
@@ -366,15 +435,13 @@ dia_font_selector_init (DiaFontSelector *fs)
 			      g_strdup(pango_font_family_get_name(families[i])));
   }
   g_free (families);
-
-  fs->font_omenu = 
-    GTK_OPTION_MENU
-    (dia_dynamic_menu_new_listbased(dia_font_selector_create_string_item,
-				    dia_font_selector_fontmenu_callback,
-				    fs,
-				    _("Other fonts"),
-				    fontnames,
-				    "font-menu"));
+  
+  fs->font_omenu =
+      GTK_OPTION_MENU(dia_dynamic_menu_new_listbased(dia_font_selector_create_string_item,
+				     fs, _("Other fonts"),
+				     fontnames, "font-menu"));
+  g_signal_connect(DIA_DYNAMIC_MENU(fs->font_omenu), "value-changed",
+		   G_CALLBACK(dia_font_selector_fontmenu_callback), fs);
   dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(fs->font_omenu),
 				     "sans");
   dia_dynamic_menu_add_default_entry(DIA_DYNAMIC_MENU(fs->font_omenu),
@@ -387,6 +454,7 @@ dia_font_selector_init (DiaFontSelector *fs)
   omenu = gtk_option_menu_new();
   fs->style_omenu = GTK_OPTION_MENU(omenu);
   menu = gtk_menu_new ();
+  /* No callback needed since fs->style_menu keeps getting replaced. */
   fs->style_menu = GTK_MENU(menu);
   gtk_option_menu_set_menu (GTK_OPTION_MENU (fs->style_omenu), menu);
 
@@ -436,7 +504,7 @@ dia_font_selector_get_family_from_name(GtkWidget *widget, const gchar *fontname)
 			       &families, &n_families);
   /* Doing it the slow way until I find a better way */
   for (i = 0; i < n_families; i++) {
-    if (!(g_strcasecmp(pango_font_family_get_name(families[i]), fontname))) {
+    if (!(g_ascii_strcasecmp(pango_font_family_get_name(families[i]), fontname))) {
       PangoFontFamily *fam = families[i];
       g_free(families);
       return fam;
@@ -448,10 +516,22 @@ dia_font_selector_get_family_from_name(GtkWidget *widget, const gchar *fontname)
 }
 
 static void
-dia_font_selector_fontmenu_callback(DiaDynamicMenu *ddm, const gchar *fontname, gpointer data) 
+dia_font_selector_fontmenu_callback(DiaDynamicMenu *ddm, gpointer data) 
 {
   DiaFontSelector *fs = DIAFONTSELECTOR(data);
+  char *fontname = dia_dynamic_menu_get_entry(ddm);
   dia_font_selector_set_styles(fs, fontname, -1);
+  g_signal_emit(GTK_OBJECT(fs),
+		dfontsel_signals[DFONTSEL_VALUE_CHANGED], 0);
+  g_free(fontname);
+}
+
+static void
+dia_font_selector_stylemenu_callback(GtkMenu *menu, gpointer data)
+{
+  DiaFontSelector *fs = DIAFONTSELECTOR(data);
+  g_signal_emit(GTK_OBJECT(fs),
+		dfontsel_signals[DFONTSEL_VALUE_CHANGED], 0);
 }
 
 static char *style_labels[] = {
@@ -486,15 +566,19 @@ dia_font_selector_set_style_menu(DiaFontSelector *fs,
 				 PangoFontFamily *pff,
 				 DiaFontStyle dia_style)
 {
-  int i=0, select = 0;
+  int select = 0;
   PangoFontFace **faces = NULL;
-  int nfaces = 0;
+  guint nfaces = 0;
+  guint i=0;
   GtkWidget *menu = NULL;
   long stylebits = 0;
   int menu_item_nr = 0;
   GSList *group = NULL;
 
   menu = gtk_menu_new ();
+  g_signal_connect(menu, "selection-done",
+		   G_CALLBACK(dia_font_selector_stylemenu_callback), fs);
+  
   pango_font_family_list_faces(pff, &faces, &nfaces);
 
   for (i = 0; i < nfaces; i++) {
@@ -579,6 +663,8 @@ dia_font_selector_set_font(DiaFontSelector *fs, DiaFont *font)
   const gchar *fontname = dia_font_get_family(font);
   /* side effect: adds fontname to presistence list */
   dia_dynamic_menu_select_entry(DIA_DYNAMIC_MENU(fs->font_omenu), fontname);
+  g_signal_emit(GTK_OBJECT(fs),
+		dfontsel_signals[DFONTSEL_VALUE_CHANGED], 0);
   dia_font_selector_set_styles(fs, fontname, dia_font_get_style (font));
 }
 
@@ -724,12 +810,27 @@ struct _DiaLineStyleSelectorClass
   GtkVBoxClass parent_class;
 };
 
+enum {
+    DLS_VALUE_CHANGED,
+    DLS_LAST_SIGNAL
+};
+
+static guint dls_signals[DLS_LAST_SIGNAL] = { 0 };
+
 static void
 dia_line_style_selector_class_init (DiaLineStyleSelectorClass *class)
 {
   GtkObjectClass *object_class;
   
   object_class = (GtkObjectClass*) class;
+
+  dls_signals[DLS_VALUE_CHANGED]
+      = g_signal_new("value-changed",
+		     G_TYPE_FROM_CLASS(class),
+		     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
 }
 
 static void
@@ -747,9 +848,18 @@ set_linestyle_sensitivity(DiaLineStyleSelector *fs)
 }
 
 static void
-linestyle_type_change_callback(GtkObject *as, gboolean arg1, gpointer data)
+linestyle_type_change_callback(GtkMenu *menu, gpointer data)
 {
-  set_linestyle_sensitivity(DIALINESTYLESELECTOR(as));
+  set_linestyle_sensitivity(DIALINESTYLESELECTOR(data));
+  g_signal_emit(DIALINESTYLESELECTOR(data),
+		dls_signals[DLS_VALUE_CHANGED], 0);
+}
+
+static void
+linestyle_dashlength_change_callback(GtkSpinButton *sb, gpointer data)
+{
+  g_signal_emit(DIALINESTYLESELECTOR(data),
+		dls_signals[DLS_VALUE_CHANGED], 0);
 }
 
 static void
@@ -816,9 +926,8 @@ dia_line_style_selector_init (DiaLineStyleSelector *fs)
   
   gtk_menu_set_active(GTK_MENU (menu), DEFAULT_LINESTYLE);
   gtk_option_menu_set_menu (GTK_OPTION_MENU (fs->omenu), menu);
-  gtk_signal_connect_object(GTK_OBJECT(menu), "selection-done", 
-			    GTK_SIGNAL_FUNC(linestyle_type_change_callback), 
-			    (gpointer)fs);
+  g_signal_connect(GTK_OBJECT(menu), "selection-done", 
+		   G_CALLBACK(linestyle_type_change_callback), fs);
  
   gtk_box_pack_start(GTK_BOX(fs), GTK_WIDGET(fs->omenu), FALSE, TRUE, 0);
   gtk_widget_show(GTK_WIDGET(fs->omenu));
@@ -831,7 +940,7 @@ dia_line_style_selector_init (DiaLineStyleSelector *fs)
   gtk_box_pack_start_defaults(GTK_BOX(box), label);
   gtk_widget_show(label);
 
-  adj = (GtkAdjustment *)gtk_adjustment_new(0.1, 0.00, 10.0, 0.1, 1.0, 1.0);
+  adj = (GtkAdjustment *)gtk_adjustment_new(0.1, 0.00, 10.0, 0.1, 1.0, 0);
   length = gtk_spin_button_new(adj, DEFAULT_LINESTYLE_DASHLEN, 2);
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(length), TRUE);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(length), TRUE);
@@ -839,6 +948,9 @@ dia_line_style_selector_init (DiaLineStyleSelector *fs)
   gtk_box_pack_start_defaults(GTK_BOX (box), length);
   gtk_widget_show (length);
 
+  g_signal_connect(GTK_OBJECT(length), "changed", 
+		   G_CALLBACK(linestyle_dashlength_change_callback), fs);
+
   set_linestyle_sensitivity(fs);
   gtk_box_pack_start_defaults(GTK_BOX(fs), box);
   gtk_widget_show(box);
@@ -948,11 +1060,6 @@ dia_color_selector_more_ok(GtkWidget *ok, gpointer userdata)
   gtk_widget_destroy(colorsel);
 }
 
-static void
-dia_color_selector_activate(DiaDynamicMenu *ddm, const gchar *entry, gpointer data)
-{
-}
-
 static void
 dia_color_selector_more_callback(GtkWidget *widget, gpointer userdata)
 {
@@ -1024,7 +1131,6 @@ dia_color_selector_new ()
 {
   GtkWidget *otheritem = gtk_menu_item_new_with_label(_("More colors..."));
   GtkWidget *ddm = dia_dynamic_menu_new(dia_color_selector_create_string_item,
-					dia_color_selector_activate,
 					NULL,
 					GTK_MENU_ITEM(otheritem),
 					"color-menu");
@@ -1081,6 +1187,8 @@ dia_color_selector_set_color (GtkWidget *widget,
 
 
 /************* DiaArrowSelector: ***************/
+
+/* FIXME: Should these structs be in widgets.h instead? */
 struct _DiaArrowSelector
 {
   GtkVBox vbox;
@@ -1097,9 +1205,23 @@ struct _DiaArrowSelectorClass
   GtkVBoxClass parent_class;
 };
 
+enum {
+    DAS_VALUE_CHANGED,
+    DAS_LAST_SIGNAL
+};
+
+static guint das_signals[DAS_LAST_SIGNAL] = {0};
+
 static void
 dia_arrow_selector_class_init (DiaArrowSelectorClass *class)
 {
+  das_signals[DAS_VALUE_CHANGED]
+      = g_signal_new("value_changed",
+		     G_TYPE_FROM_CLASS(class),
+		     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
 }
   
 static void
@@ -1108,7 +1230,7 @@ set_size_sensitivity(DiaArrowSelector *as)
   int state;
   gchar *entryname = dia_dynamic_menu_get_entry(DIA_DYNAMIC_MENU(as->omenu));
 
-  state = (entryname != NULL) && (0 != g_strcasecmp(entryname, "None"));
+  state = (entryname != NULL) && (0 != g_ascii_strcasecmp(entryname, "None"));
   g_free(entryname);
 
   gtk_widget_set_sensitive(GTK_WIDGET(as->sizelabel), state);
@@ -1116,9 +1238,18 @@ set_size_sensitivity(DiaArrowSelector *as)
 }
 
 static void
-arrow_type_change_callback(DiaDynamicMenu *ddm, const gchar *name, gpointer userdata)
+arrow_type_change_callback(DiaDynamicMenu *ddm, gpointer userdata)
 {
   set_size_sensitivity(DIA_ARROW_SELECTOR(userdata));
+  g_signal_emit(DIA_ARROW_SELECTOR(userdata),
+		das_signals[DAS_VALUE_CHANGED], 0);
+}
+
+static void
+arrow_size_change_callback(DiaSizeSelector *size, gpointer userdata)
+{
+  g_signal_emit(DIA_ARROW_SELECTOR(userdata),
+		das_signals[DAS_VALUE_CHANGED], 0);
 }
 
 static GtkWidget *
@@ -1145,7 +1276,7 @@ dia_arrow_selector_init (DiaArrowSelector *as,
   
   GList *arrow_names = get_arrow_names();
   omenu = dia_dynamic_menu_new_listbased(create_arrow_menu_item,
-					 arrow_type_change_callback, as,
+					 as,
 					 _("More arrows"),
 					 arrow_names,
 					 "arrow-menu");
@@ -1156,6 +1287,10 @@ dia_arrow_selector_init (DiaArrowSelector *as,
   gtk_box_pack_start(GTK_BOX(as), omenu, FALSE, TRUE, 0);
   gtk_widget_show(omenu);
 
+  g_signal_connect(DIA_DYNAMIC_MENU(omenu),
+		   "value-changed", G_CALLBACK(arrow_type_change_callback),
+		   as);
+
   box = gtk_hbox_new(FALSE,0);
   as->sizebox = GTK_HBOX(box);
 
@@ -1167,13 +1302,14 @@ dia_arrow_selector_init (DiaArrowSelector *as,
   size = dia_size_selector_new(0.0, 0.0);
   as->size = DIA_SIZE_SELECTOR(size);
   gtk_box_pack_start_defaults(GTK_BOX(box), size);
-  gtk_widget_show(size);  
+  gtk_widget_show(size);
+  g_signal_connect(size, "value-changed",
+		   G_CALLBACK(arrow_size_change_callback), as);
 
   set_size_sensitivity(as);
   gtk_box_pack_start_defaults(GTK_BOX(as), box);
 
   gtk_widget_show(box);
-
 }
 
 GType
@@ -1233,7 +1369,7 @@ dia_arrow_selector_set_arrow (DiaArrowSelector *as,
 			      Arrow arrow)
 {
   dia_dynamic_menu_select_entry(DIA_DYNAMIC_MENU(as->omenu),
-				arrow_types[arrow_index_from_type(arrow.type)].name);
+				arrow_get_name_from_type(arrow.type));
   set_size_sensitivity(as);
   dia_size_selector_set_size(DIA_SIZE_SELECTOR(as->size), arrow.width, arrow.length);
 }
@@ -1244,7 +1380,7 @@ struct _DiaFileSelector
   GtkHBox hbox;
   GtkEntry *entry;
   GtkButton *browse;
-  GtkFileSelection *dialog;
+  GtkWidget *dialog;
   gchar *sys_filename;
 };
 
@@ -1253,6 +1389,13 @@ struct _DiaFileSelectorClass
   GtkHBoxClass parent_class;
 };
 
+enum {
+    DFILE_VALUE_CHANGED,
+    DFILE_LAST_SIGNAL
+};
+
+static guint dfile_signals[DFILE_LAST_SIGNAL] = { 0 };
+
 static void
 dia_file_selector_unrealize(GtkWidget *widget)
 {
@@ -1279,6 +1422,22 @@ dia_file_selector_class_init (DiaFileSelectorClass *class)
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   widget_class->unrealize = dia_file_selector_unrealize;
+
+  dfile_signals[DFILE_VALUE_CHANGED]
+      = g_signal_new("value-changed",
+		     G_TYPE_FROM_CLASS(class),
+		     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
+}
+
+static void
+dia_file_selector_entry_changed(GtkEditable *editable
+				, gpointer data)
+{
+  DiaFileSelector *fs = DIAFILESELECTOR(data);
+  g_signal_emit(fs, dfile_signals[DFILE_VALUE_CHANGED], 0);
 }
 
 static void
@@ -1310,7 +1469,9 @@ dia_file_selector_browse_pressed(GtkWidget *widget, gpointer data)
     toplevel = NULL;
 
   if (fs->dialog == NULL) {
-    dialog = fs->dialog =
+    GtkFileFilter *filter;
+    
+    dialog = fs->dialog = 
       gtk_file_chooser_dialog_new (_("Select image file"), toplevel ? GTK_WINDOW(toplevel) : NULL,
                                    GTK_FILE_CHOOSER_ACTION_OPEN,
                                    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -1323,6 +1484,16 @@ dia_file_selector_browse_pressed(GtkWidget *widget, gpointer data)
 			GTK_SIGNAL_FUNC (gtk_widget_destroyed),
 			&fs->dialog);
     
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("Supported Formats"));
+    gtk_file_filter_add_pixbuf_formats (filter);
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("All Files"));
+    gtk_file_filter_add_pattern (filter, "*");
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);
+
     gtk_object_set_user_data(GTK_OBJECT(dialog), fs);
   }
 
@@ -1342,9 +1513,13 @@ dia_file_selector_init (DiaFileSelector *fs)
   /* Here's where we set up the real thing */
   fs->dialog = NULL;
   fs->sys_filename = NULL;  
+
   fs->entry = GTK_ENTRY(gtk_entry_new());
   gtk_box_pack_start(GTK_BOX(fs), GTK_WIDGET(fs->entry), FALSE, TRUE, 0);
+  g_signal_connect(GTK_OBJECT(fs->entry), "changed",
+		   G_CALLBACK(dia_file_selector_entry_changed), fs);
   gtk_widget_show(GTK_WIDGET(fs->entry));
+
   fs->browse = GTK_BUTTON(gtk_button_new_with_label(_("Browse")));
   gtk_box_pack_start(GTK_BOX(fs), GTK_WIDGET(fs->browse), FALSE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (fs->browse), "clicked",
@@ -1411,171 +1586,75 @@ dia_file_selector_get_file(DiaFileSelector *fs)
  *  limits, and b) avoid rounding problems while editing.
  */
 
-typedef struct _DiaUnitDef DiaUnitDef;
-struct _DiaUnitDef {
-  char* name;
-  char* unit;
-  float factor;
-};
-
-/* from gnome-libs/libgnome/gnome-paper.c */
-static const DiaUnitDef units[] =
-{
-  /* XXX does anyone *really* measure paper size in feet?  meters? */
-
-  /* human name, abreviation, points per unit */
-  { "Feet",       "ft", 864 },
-  { "Meter",      "m",  2834.6457 },
-  { "Decimeter",  "dm", 283.46457 },
-  { "Millimeter", "mm", 2.8346457 },
-  { "Point",      "pt", 1. },
-  { "Centimeter", "cm", 28.346457 },
-  { "Inch",       "in", 72 },
-  { "Pica",       "pi", 12 },
-  { 0 }
-};
-
-static GtkObjectClass *parent_class;
-static GtkObjectClass *entry_class;
-
-static void dia_unit_spinner_class_init(DiaUnitSpinnerClass *class);
 static void dia_unit_spinner_init(DiaUnitSpinner *self);
 
-GtkType
+GType
 dia_unit_spinner_get_type(void)
 {
-  static GtkType us_type = 0;
+  static GType us_type = 0;
 
   if (!us_type) {
-    static const GtkTypeInfo us_info = {
-      "DiaUnitSpinner",
-      sizeof(DiaUnitSpinner),
+    static const GTypeInfo us_info = {
       sizeof(DiaUnitSpinnerClass),
-      (GtkClassInitFunc) dia_unit_spinner_class_init,
-      (GtkObjectInitFunc) dia_unit_spinner_init,
-      NULL,
-      NULL,
-      (GtkClassInitFunc) NULL,
+      NULL, /* base_init */
+      NULL, /* base_finalize */
+      NULL, /* class_init*/
+      NULL, /* class_finalize */
+      NULL, /* class_data */
+      sizeof(DiaUnitSpinner),
+      0,    /* n_preallocs */
+      (GInstanceInitFunc) dia_unit_spinner_init,
     };
-    us_type = gtk_type_unique(gtk_spin_button_get_type(), &us_info);
+    us_type = g_type_register_static(GTK_TYPE_SPIN_BUTTON,
+                                     "DiaUnitSpinner",
+                                     &us_info, 0);
   }
   return us_type;
 }
 
-/** Updates the spinner display to show digits and units */
-static void
-dia_unit_spinner_value_changed(GtkAdjustment *adjustment,
-			       DiaUnitSpinner *spinner)
-{
-  char buf[256];
-  GtkSpinButton *sbutton = GTK_SPIN_BUTTON(spinner);
-
-  g_snprintf(buf, sizeof(buf), "%0.*f %s", sbutton->digits, adjustment->value,
-	     units[spinner->unit_num].unit);
-  gtk_entry_set_text(GTK_ENTRY(spinner), buf);
-}
-
-static void dia_unit_spinner_set_value_direct(DiaUnitSpinner *self, gfloat val);
-static gint dia_unit_spinner_focus_out(GtkWidget *widget, GdkEventFocus *ev);
-static gint dia_unit_spinner_button_press(GtkWidget *widget,GdkEventButton*ev);
-static gint dia_unit_spinner_key_press(GtkWidget *widget, GdkEventKey *event);
-static void dia_unit_spinner_activate(GtkEntry *editable);
-
-static void
-dia_unit_spinner_class_init(DiaUnitSpinnerClass *class)
-{
-  GtkObjectClass *object_class;
-  GtkWidgetClass *widget_class;
-  GtkEntryClass  *editable_class;
-
-  object_class = (GtkObjectClass *)class;
-  widget_class = (GtkWidgetClass *)class;
-  editable_class = (GtkEntryClass *)class;
-
-  widget_class->focus_out_event    = dia_unit_spinner_focus_out;
-  widget_class->button_press_event = dia_unit_spinner_button_press;
-  widget_class->key_press_event    = dia_unit_spinner_key_press;
-  editable_class->activate         = dia_unit_spinner_activate;
-
-  parent_class = gtk_type_class(GTK_TYPE_SPIN_BUTTON);
-  entry_class  = gtk_type_class(GTK_TYPE_ENTRY);
-}
 
 static void
 dia_unit_spinner_init(DiaUnitSpinner *self)
 {
-  /* change over to our own print function that appends the unit name on the
-   * end */
-  if (self->parent.adjustment) {
-    gtk_signal_disconnect_by_data(GTK_OBJECT(self->parent.adjustment),
-				  (gpointer) self);
-    g_signal_connect(GTK_OBJECT(self->parent.adjustment), "value_changed",
-		      G_CALLBACK(dia_unit_spinner_value_changed),
-		       (gpointer) self);
-  }
-
   self->unit_num = DIA_UNIT_CENTIMETER;
 }
 
+/*
+  Callback functions for the "input" and "output" signals emitted by
+  GtkSpinButton. All the normal work is done by the spin button, we
+  simply modify how the text in the GtkEntry is treated.
+*/
+static gboolean
+dia_unit_spinner_input(DiaUnitSpinner *self, gdouble *value);
+static gboolean dia_unit_spinner_output(DiaUnitSpinner *self);
+
 GtkWidget *
-dia_unit_spinner_new(GtkAdjustment *adjustment, guint digits, DiaUnit adj_unit)
+dia_unit_spinner_new(GtkAdjustment *adjustment, DiaUnit adj_unit)
 {
-  DiaUnitSpinner *self = gtk_type_new(dia_unit_spinner_get_type());
-
-  self->unit_num = adj_unit;
-
-  gtk_spin_button_configure(GTK_SPIN_BUTTON(self), adjustment, 0.0, digits);
-
+  DiaUnitSpinner *self;
+  
   if (adjustment) {
-    gtk_signal_disconnect_by_data(GTK_OBJECT(adjustment),
-				  (gpointer) self);
-    g_signal_connect(GTK_OBJECT(adjustment), "value_changed",
-		     G_CALLBACK(dia_unit_spinner_value_changed),
-		       (gpointer) self);
-    dia_unit_spinner_set_value(self, adjustment->value);
-  } else {
-    /* Don't know any better, hopefully it'll be set later. */
-    dia_unit_spinner_set_value(self, 1.0);
+    g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), NULL);
   }
+  
+  self = gtk_type_new(dia_unit_spinner_get_type());
+  self->unit_num = adj_unit;
+  
+  gtk_spin_button_configure(GTK_SPIN_BUTTON(self),
+                            adjustment, 0.0, units[adj_unit].digits);
+
+  g_signal_connect(GTK_SPIN_BUTTON(self), "output",
+                   G_CALLBACK(dia_unit_spinner_output),
+                   NULL);
+  g_signal_connect(GTK_SPIN_BUTTON(self), "input",
+                   G_CALLBACK(dia_unit_spinner_input),
+                   NULL);
 
   return GTK_WIDGET(self);
 }
 
-/** Set the value (in cm).
- * */
-void
-dia_unit_spinner_set_value(DiaUnitSpinner *self, gfloat val)
-{
-  dia_unit_spinner_set_value_direct(self, val /
-				    (units[self->unit_num].factor / units[DIA_UNIT_CENTIMETER].factor));
-}
-
-/** Set the value (in preferred units) */
-static void
-dia_unit_spinner_set_value_direct(DiaUnitSpinner *self, gfloat val)
-{
-  GtkSpinButton *sbutton = GTK_SPIN_BUTTON(self);
-
-  if (val < sbutton->adjustment->lower)
-    val = sbutton->adjustment->lower;
-  else if (val > sbutton->adjustment->upper)
-    val = sbutton->adjustment->upper;
-  sbutton->adjustment->value = val;
-  dia_unit_spinner_value_changed(sbutton->adjustment, self);
-}
-
-/** Get the value (in cm) */
-gfloat
-dia_unit_spinner_get_value(DiaUnitSpinner *self)
-{
-  GtkSpinButton *sbutton = GTK_SPIN_BUTTON(self);
-
-  return sbutton->adjustment->value *
-    (units[self->unit_num].factor / units[DIA_UNIT_CENTIMETER].factor);
-}
-
-static void
-dia_unit_spinner_update(DiaUnitSpinner *self)
+static gboolean
+dia_unit_spinner_input(DiaUnitSpinner *self, gdouble *value)
 {
   gfloat val, factor = 1.0;
   gchar *extra = NULL;
@@ -1588,50 +1667,75 @@ dia_unit_spinner_update(DiaUnitSpinner *self)
     int i;
 
     for (i = 0; units[i].name != NULL; i++)
-      if (!g_strcasecmp(units[i].unit, extra)) {
+      if (!g_ascii_strcasecmp(units[i].unit, extra)) {
 	factor = units[i].factor / units[self->unit_num].factor;
 	break;
       }
   }
   /* convert to prefered units */
   val *= factor;
-  dia_unit_spinner_set_value_direct(self, val);
+
+  /* Store value in the location provided by the signal emitter. */
+  *value = val;
+
+  /* Return true, so that the default input function is not invoked. */
+  return TRUE;
 }
 
-static gint
-dia_unit_spinner_focus_out(GtkWidget *widget, GdkEventFocus *event)
+static gboolean dia_unit_spinner_output(DiaUnitSpinner *self)
 {
-  if (GTK_ENTRY (widget)->editable)
-    dia_unit_spinner_update(DIA_UNIT_SPINNER(widget));
-  return GTK_WIDGET_CLASS(entry_class)->focus_out_event(widget, event);
+  char buf[256];
+
+  GtkSpinButton *sbutton = GTK_SPIN_BUTTON(self);
+  GtkAdjustment *adjustment = gtk_spin_button_get_adjustment(sbutton);
+
+  g_snprintf(buf, sizeof(buf), "%0.*f %s",
+             gtk_spin_button_get_digits(sbutton),
+             gtk_adjustment_get_value(adjustment),
+	      units[self->unit_num].unit);
+  gtk_entry_set_text(GTK_ENTRY(self), buf);
+
+  /* Return true, so that the default output function is not invoked. */
+  return TRUE;
 }
 
-static gint
-dia_unit_spinner_button_press(GtkWidget *widget, GdkEventButton *event)
+/** Set the value (in cm).
+ * */
+void
+dia_unit_spinner_set_value(DiaUnitSpinner *self, gdouble val)
 {
-  dia_unit_spinner_update(DIA_UNIT_SPINNER(widget));
-  return GTK_WIDGET_CLASS(parent_class)->button_press_event(widget, event);
+  GtkSpinButton *sbutton = GTK_SPIN_BUTTON(self);
+
+  gtk_spin_button_set_value(sbutton,
+                            val /
+                            (units[self->unit_num].factor /
+                             units[DIA_UNIT_CENTIMETER].factor));
 }
 
-static gint
-dia_unit_spinner_key_press(GtkWidget *widget, GdkEventKey *event)
+/** Get the value (in cm) */
+gdouble
+dia_unit_spinner_get_value(DiaUnitSpinner *self)
 {
-  gint key = event->keyval;
+  GtkSpinButton *sbutton = GTK_SPIN_BUTTON(self);
 
-  if (GTK_ENTRY (widget)->editable &&
-      (key == GDK_Up || key == GDK_Down || 
-       key == GDK_Page_Up || key == GDK_Page_Down))
-    dia_unit_spinner_update (DIA_UNIT_SPINNER(widget));
-  return GTK_WIDGET_CLASS(parent_class)->key_press_event(widget, event);
+  return gtk_spin_button_get_value(sbutton) *
+      (units[self->unit_num].factor / units[DIA_UNIT_CENTIMETER].factor);
 }
 
-static void
-dia_unit_spinner_activate(GtkEntry *editable)
+GList *
+get_units_name_list(void)
 {
-  if (editable->editable)
-    dia_unit_spinner_update(DIA_UNIT_SPINNER(editable));
-}
+  int i;
+  static GList *name_list = NULL;
 
+  if (name_list == NULL) {
+    for (i = 0; units[i].name != NULL; i++) {
+      name_list = g_list_append(name_list, units[i].name);
+    }
+  }
+
+  return name_list;
+}
 
 /* ************************ Dynamic menus ************************ */
 
@@ -1643,6 +1747,13 @@ static void dia_dynamic_menu_create_sublist(DiaDynamicMenu *ddm,
 static void dia_dynamic_menu_create_menu(DiaDynamicMenu *ddm);
 static void dia_dynamic_menu_destroy(GtkObject *object);
 
+enum {
+    DDM_VALUE_CHANGED,
+    DDM_LAST_SIGNAL
+};
+
+static guint ddm_signals[DDM_LAST_SIGNAL] = { 0 };
+
 GtkType
 dia_dynamic_menu_get_type(void)
 {
@@ -1670,6 +1781,14 @@ dia_dynamic_menu_class_init(DiaDynamicMenuClass *class)
   GtkObjectClass *object_class = (GtkObjectClass*)class;
 
   object_class->destroy = dia_dynamic_menu_destroy;
+  
+  ddm_signals[DDM_VALUE_CHANGED]
+      = g_signal_new("value-changed",
+		     G_TYPE_FROM_CLASS(class),
+		     G_SIGNAL_RUN_FIRST,
+		     0, NULL, NULL,
+		     g_cclosure_marshal_VOID__VOID,
+		     G_TYPE_NONE, 0);
 }
 
 static void
@@ -1702,7 +1821,7 @@ dia_dynamic_menu_destroy(GtkObject *object)
  */
 GtkWidget *
 dia_dynamic_menu_new(DDMCreateItemFunc create, 
-		     DDMCallbackFunc activate, gpointer userdata,
+		     gpointer userdata,
 		     GtkMenuItem *otheritem, gchar *persist)
 {
   DiaDynamicMenu *ddm;
@@ -1712,7 +1831,6 @@ dia_dynamic_menu_new(DDMCreateItemFunc create,
   ddm = DIA_DYNAMIC_MENU ( gtk_type_new (dia_dynamic_menu_get_type ()));
   
   ddm->create_func = create;
-  ddm->activate_func = activate;
   ddm->userdata = userdata;
   ddm->other_item = otheritem;
   ddm->persistent_name = persist;
@@ -1735,7 +1853,7 @@ dia_dynamic_menu_select_entry(DiaDynamicMenu *ddm, const gchar *name)
       int i = 0;
       for (tmp = ddm->default_entries; tmp != NULL;
 	   tmp = g_list_next(tmp), i++) {
-	if (!g_strcasecmp(tmp->data, name))
+	if (!g_ascii_strcasecmp(tmp->data, name))
 	  gtk_option_menu_set_history(GTK_OPTION_MENU(ddm), i);
       }
       /* Not there after all? */
@@ -1746,9 +1864,10 @@ dia_dynamic_menu_select_entry(DiaDynamicMenu *ddm, const gchar *name)
     else
       gtk_option_menu_set_history(GTK_OPTION_MENU(ddm), 0);
   }
-  if (ddm->activate_func != NULL) {
-    (ddm->activate_func)(ddm, name, ddm->userdata);
-  }
+  
+  g_free(ddm->active);
+  ddm->active = g_strdup(name);
+  g_signal_emit(GTK_OBJECT(ddm), ddm_signals[DDM_VALUE_CHANGED], 0);
 }
 
 static void
@@ -1771,12 +1890,11 @@ dia_dynamic_menu_create_string_item(DiaDynamicMenu *ddm, gchar *string)
  */
 GtkWidget *
 dia_dynamic_menu_new_stringbased(GtkMenuItem *otheritem, 
-				 DDMCallbackFunc activate,
 				 gpointer userdata,
 				 gchar *persist)
 {
   GtkWidget *ddm = dia_dynamic_menu_new(dia_dynamic_menu_create_string_item,
-					activate, userdata,
+					userdata,
 					otheritem, persist);
   return ddm;
 }
@@ -1787,13 +1905,12 @@ dia_dynamic_menu_new_stringbased(GtkMenuItem *otheritem,
  */
 GtkWidget *
 dia_dynamic_menu_new_listbased(DDMCreateItemFunc create,
-			       DDMCallbackFunc activate,
 			       gpointer userdata,
 			       gchar *other_label, GList *items, 
 			       gchar *persist)
 {
   GtkWidget *item = gtk_menu_item_new_with_label(other_label);
-  GtkWidget *ddm = dia_dynamic_menu_new(create, activate, userdata,
+  GtkWidget *ddm = dia_dynamic_menu_new(create, userdata,
 					GTK_MENU_ITEM(item), persist);
   dia_dynamic_menu_create_sublist(DIA_DYNAMIC_MENU(ddm), items,  create);
 
@@ -1807,12 +1924,11 @@ dia_dynamic_menu_new_listbased(DDMCreateItemFunc create,
 GtkWidget *
 dia_dynamic_menu_new_stringlistbased(gchar *other_label, 
 				     GList *items,
-				     DDMCallbackFunc activate,
 				     gpointer userdata,
 				     gchar *persist)
 {
   return dia_dynamic_menu_new_listbased(dia_dynamic_menu_create_string_item,
-					activate, userdata,
+					userdata,
 					other_label, items, persist);
 }
 
@@ -1850,7 +1966,8 @@ dia_dynamic_menu_create_sublist(DiaDynamicMenu *ddm,
 void
 dia_dynamic_menu_add_default_entry(DiaDynamicMenu *ddm, const gchar *entry)
 {
-  ddm->default_entries = g_list_append(ddm->default_entries, g_strdup(entry));
+  ddm->default_entries = g_list_append(ddm->default_entries,
+				       g_strdup(entry));
 
   dia_dynamic_menu_create_menu(ddm);
 }
@@ -1882,11 +1999,8 @@ dia_dynamic_menu_add_entry(DiaDynamicMenu *ddm, const gchar *entry)
   GList *tmp;
   gboolean existed;
 
-  g_free(ddm->active);
-  ddm->active = g_strdup(entry);
-
   for (tmp = ddm->default_entries; tmp != NULL; tmp = g_list_next(tmp)) {
-    if (!g_strcasecmp(tmp->data, entry))
+    if (!g_ascii_strcasecmp(tmp->data, entry))
       return 0;
   }
   existed = persistent_list_add(ddm->persistent_name, entry);
@@ -1985,14 +2099,14 @@ dia_dynamic_menu_reset(GtkWidget *item, gpointer userdata)
 {
   DiaDynamicMenu *ddm = DIA_DYNAMIC_MENU(userdata);
   PersistentList *plist = persistent_list_get(ddm->persistent_name);
-  gchar *active = g_strdup (ddm->active); 
+  gchar *active = dia_dynamic_menu_get_entry(ddm);
   g_list_foreach(plist->glist, (GFunc)g_free, NULL);
   g_list_free(plist->glist);
   plist->glist = NULL;
   dia_dynamic_menu_create_menu(ddm);
   if (active)
     dia_dynamic_menu_select_entry(ddm, active);
-  g_free (active);
+  g_free(active);
 }
 
 /** Set the maximum number of non-default entries.
@@ -2081,6 +2195,7 @@ dia_toggle_button_new(GtkWidget *on_widget, GtkWidget *off_widget)
   gtk_widget_style_get_property(GTK_WIDGET(button), "focus-padding", prop);
   i = g_value_get_int(prop);
   g_value_set_int(prop, 0);
+  g_free(prop);
 
   gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
   /*  gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);*/
@@ -2111,3 +2226,5 @@ dia_toggle_button_new_with_icons(const guint8 *on_icon,
   return dia_toggle_button_new(gtk_image_new_from_pixbuf(p1),
 			       gtk_image_new_from_pixbuf(p2));
 }
+
+
