@@ -118,11 +118,13 @@ static ObjectOps bezierline_ops = {
   (MoveFunc)            bezierline_move,
   (MoveHandleFunc)      bezierline_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
-  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ApplyPropertiesDialogFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      bezierline_get_object_menu,
   (DescribePropsFunc)   bezierline_describe_props,
   (GetPropsFunc)        bezierline_get_props,
   (SetPropsFunc)        bezierline_set_props,
+  (TextEditFunc) 0,
+  (ApplyPropertiesListFunc) object_apply_props,
 };
 
 static PropNumData gap_range = { -G_MAXFLOAT, G_MAXFLOAT, 0.1};
@@ -153,14 +155,16 @@ bezierline_describe_props(Bezierline *bezierline)
 
 static PropOffset bezierline_offsets[] = {
   BEZCONN_COMMON_PROPERTIES_OFFSETS,
-  { "line_width", PROP_TYPE_REAL, offsetof(Bezierline, line_width) },
+  { PROP_STDNAME_LINE_WIDTH, PROP_STDTYPE_LINE_WIDTH, offsetof(Bezierline, line_width) },
   { "line_colour", PROP_TYPE_COLOUR, offsetof(Bezierline, line_color) },
   { "line_style", PROP_TYPE_LINESTYLE,
     offsetof(Bezierline, line_style), offsetof(Bezierline, dashlength) },
   { "start_arrow", PROP_TYPE_ARROW, offsetof(Bezierline, start_arrow) },
   { "end_arrow", PROP_TYPE_ARROW, offsetof(Bezierline, end_arrow) },
+  PROP_OFFSET_FRAME_BEGIN("gaps"),
   { "absolute_start_gap", PROP_TYPE_REAL, offsetof(Bezierline, absolute_start_gap) },
   { "absolute_end_gap", PROP_TYPE_REAL, offsetof(Bezierline, absolute_end_gap) },
+  PROP_OFFSET_FRAME_END("gaps"),
   { NULL, 0, 0 }
 };
 
@@ -377,11 +381,11 @@ bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer)
 					 &bezierline->end_arrow);
     exchange_bez_gap_points(bez,gap_points);
   } else {
-  renderer_ops->draw_bezier_with_arrows(renderer, bez->points, bez->numpoints,
-					 bezierline->line_width,
-					 &bezierline->line_color,
-					 &bezierline->start_arrow,
-					 &bezierline->end_arrow);
+    renderer_ops->draw_bezier_with_arrows(renderer, bez->points, bez->numpoints,
+					  bezierline->line_width,
+					  &bezierline->line_color,
+					  &bezierline->start_arrow,
+					  &bezierline->end_arrow);
   }
 
 #if 0
@@ -411,7 +415,7 @@ bezierline_draw(Bezierline *bezierline, DiaRenderer *renderer)
    * only checking while selected.  But we'll do that if needed.
    */
   if (renderer->is_interactive &&
-      dia_object_is_selected((DiaObject*)bezierline)) {
+      dia_object_is_selected(&bezierline->bez.object)) {
     bezierconn_draw_control_lines(&bezierline->bez, renderer);
   }
 }
@@ -451,8 +455,6 @@ bezierline_create(Point *startpoint,
     bezierconn_set_points(bez, bcd->num_points, bcd->points);
   }
 
-  bezierline_update_data(bezierline);
-
   bezierline->line_width =  attributes_get_default_linewidth();
   bezierline->line_color = attributes_get_foreground();
   attributes_get_default_line_style(&bezierline->line_style,
@@ -462,6 +464,9 @@ bezierline_create(Point *startpoint,
 
   *handle1 = bez->object.handles[0];
   *handle2 = bez->object.handles[3];
+
+  bezierline_update_data(bezierline);
+
   return &bezierline->bez.object;
 }
 
@@ -508,33 +513,63 @@ bezierline_update_data(Bezierline *bezierline)
 
   bezierconn_update_data(bez);
     
-  extra->start_trans =  (bezierline->line_width / 2.0);
-  extra->end_trans =     (bezierline->line_width / 2.0);
-  extra->middle_trans = (bezierline->line_width / 2.0);
-    
-  if (bezierline->start_arrow.type != ARROW_NONE) 
-    extra->start_trans = MAX(extra->start_trans,bezierline->start_arrow.width);
-  if (bezierline->end_arrow.type != ARROW_NONE) 
-    extra->end_trans = MAX(extra->end_trans,bezierline->end_arrow.width);
-
-  extra->start_long = (bezierline->line_width / 2.0);
-  extra->end_long   = (bezierline->line_width / 2.0);
+  extra->start_trans = extra->start_long = 
+  extra->middle_trans =
+  extra->end_trans = extra->end_long = (bezierline->line_width / 2.0);
 
   obj->position = bez->points[0].p1;
 
   if (connpoint_is_autogap(bez->object.handles[0]->connected_to) ||
       connpoint_is_autogap(bez->object.handles[3*(bez->numpoints-1)]->connected_to) ||
-      bezierline->absolute_start_gap || bezierline->absolute_end_gap) {
+      bezierline->absolute_start_gap || bezierline->absolute_end_gap ||
+      bezierline->start_arrow.type != ARROW_NONE || bezierline->end_arrow.type != ARROW_NONE) {
     Point gap_points[4];
+    Rectangle bbox_union = {bez->points[0].p1.x, bez->points[0].p1.y, bez->points[0].p1.x, bez->points[0].p1.y};
     compute_gap_points(bezierline, gap_points);
     exchange_bez_gap_points(bez,gap_points);
+    /* further modifying the points data, accounts for corrcet arrow and bezier bounding box */
+    if (bezierline->start_arrow.type != ARROW_NONE) {
+      Rectangle bbox;
+      Point move_arrow, move_line;
+      Point to = bez->points[0].p1, from = bez->points[1].p1;
+      
+      calculate_arrow_point(&bezierline->start_arrow, &to, &from, &move_arrow, &move_line, bezierline->line_width);
+      point_sub(&to, &move_arrow);
+      point_sub(&bez->points[0].p1, &move_line);
+      arrow_bbox (&bezierline->start_arrow, bezierline->line_width, &to, &from, &bbox);
+      rectangle_union (&bbox_union, &bbox);
+    }
+    if (bezierline->end_arrow.type != ARROW_NONE) {
+      Rectangle bbox;
+      Point move_arrow, move_line;
+      int num_points = bez->numpoints;
+      Point to = bez->points[num_points-1].p3, from = bez->points[num_points-1].p2;
+      
+      calculate_arrow_point(&bezierline->end_arrow, &to, &from, &move_arrow, &move_line, bezierline->line_width);
+      point_sub(&to, &move_arrow);
+      point_sub(&bez->points[num_points-1].p3, &move_line);
+      arrow_bbox (&bezierline->end_arrow, bezierline->line_width, &to, &from, &bbox);
+      rectangle_union (&bbox_union, &bbox);
+    }
     bezierconn_update_boundingbox(bez);
-    exchange_bez_gap_points(bez,gap_points);
-          
+    rectangle_union (&obj->bounding_box, &bbox_union);
+    exchange_bez_gap_points(bez,gap_points);          
   } else {
-          bezierconn_update_boundingbox(bez);
+    bezierconn_update_boundingbox(bez);
+  }
+    /* add control points to the bounding box, needed to make them visible when showing all 
+      * and to remove traces from them */
+  {
+    int i, num_points = bez->numpoints;
+    obj->enclosing_box = obj->bounding_box;
+    /* starting with the second point, the first one is MOVE_TO */
+    for (i = 1; i < num_points; ++i) {
+      if (bez->points[i].type != BEZ_CURVE_TO)
+        continue;
+      rectangle_add_point(&obj->enclosing_box, &bez->points[i].p1);      
+      rectangle_add_point(&obj->enclosing_box, &bez->points[i].p2);      
+    }
   }
-
 }
 
 static void
@@ -557,7 +592,7 @@ bezierline_save(Bezierline *bezierline, ObjectNode obj_node,
 		   &bezierline->line_color);
   
   if (bezierline->line_width != 0.1)
-    data_add_real(new_attribute(obj_node, "line_width"),
+    data_add_real(new_attribute(obj_node, PROP_STDNAME_LINE_WIDTH),
 		  bezierline->line_width);
   
   if (bezierline->line_style != LINESTYLE_SOLID)
@@ -594,10 +629,6 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
   BezierConn *bez;
   DiaObject *obj;
   AttributeNode attr;
-  /* When the bezier is loaded it is not yet connected to the other objects
-   * hence we cannot compute it's real bounding box if auto gaps are used 
-   * bb is a backup bounding box */
-  Rectangle bb; 
 
   bezierline = g_new0(Bezierline, 1);
 
@@ -615,7 +646,7 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
     data_color(attribute_first_data(attr), &bezierline->line_color);
 
   bezierline->line_width = 0.1;
-  attr = object_find_attribute(obj_node, "line_width");
+  attr = object_find_attribute(obj_node, PROP_STDNAME_LINE_WIDTH);
   if (attr != NULL)
     bezierline->line_width = data_real(attribute_first_data(attr));
 
@@ -644,9 +675,11 @@ bezierline_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     bezierline->absolute_end_gap =  data_real( attribute_first_data(attr) );
   
-  bb = obj->bounding_box;
-  bezierline_update_data(bezierline); /* screws up the bounding box if auto_gap */
-  obj->bounding_box =  bb;
+  /* if "screws up the bounding box if auto_gap" it must be fixed there
+   * not by copying some meaningless bounding_box before this function call!
+   * But the real fix is in connectionpoint.c(connpoint_is_autogap)
+   */
+  bezierline_update_data(bezierline); 
 
   return &bezierline->bez.object;
 }
