@@ -148,7 +148,7 @@ static void draw_string(DiaRenderer *self,
 static void draw_image(DiaRenderer *self,
 		       Point *point,
 		       real width, real height,
-		       DiaImage image);
+		       DiaImage *image);
   
 static void draw_line_with_arrows(DiaRenderer *renderer, Point *start, Point *end, 
                                   real line_width, Color *line_color,
@@ -371,7 +371,7 @@ static void
 set_linejoin(DiaRenderer *self, LineJoin mode)
 {
     PgfRenderer *renderer = PGF_RENDERER(self);
-    int ps_mode;
+    /* int ps_mode; */
   
     switch(mode) {
     case LINEJOIN_MITER:
@@ -470,7 +470,7 @@ static void
 set_font(DiaRenderer *self, DiaFont *font, real height)
 {
     PgfRenderer *renderer = PGF_RENDERER(self);
-    gchar d_buf[DTOSTR_BUF_SIZE];
+    /* gchar d_buf[DTOSTR_BUF_SIZE]; */
 
     fprintf(renderer->file, "%% setfont left to latex\n");
 }
@@ -699,7 +699,7 @@ pgf_arc(PgfRenderer *renderer,
    
     fprintf(renderer->file,"\\pgfpathmoveto{\\pgfpoint{%s\\du}{%s\\du}}\n",
 	    stx_buf, sty_buf);
-    fprintf(renderer->file,"\\pgfpatharc{%s}{%s}{%s\\du/%s\\du}\n",
+    fprintf(renderer->file,"\\pgfpatharc{%s}{%s}{%s\\du and %s\\du}\n",
 	    angle1_buf, angle2_buf,
 	    r1_buf, r2_buf);
 	    
@@ -1175,7 +1175,7 @@ static void
 draw_image(DiaRenderer *self,
 	   Point *point,
 	   real width, real height,
-	   DiaImage image)
+	   DiaImage *image)
 {
     PgfRenderer *renderer = PGF_RENDERER(self);
 
@@ -1193,10 +1193,10 @@ export_pgf(DiagramData *data, const gchar *filename,
     double scale;
     Rectangle *extent;
     const char *name;
-    gchar el_buf[DTOSTR_BUF_SIZE];
+    /* gchar el_buf[DTOSTR_BUF_SIZE];
     gchar er_buf[DTOSTR_BUF_SIZE];
     gchar eb_buf[DTOSTR_BUF_SIZE];
-    gchar et_buf[DTOSTR_BUF_SIZE];
+    gchar et_buf[DTOSTR_BUF_SIZE]; */
     gchar scale1_buf[DTOSTR_BUF_SIZE];
     gchar scale2_buf[DTOSTR_BUF_SIZE];
 
