@@ -11,11 +11,13 @@
 
 #include <string.h>
 #include <math.h>
-#include <glib.h>
 #include <stdlib.h>
 #include <errno.h>
 #include <locale.h>
 
+#include <glib.h>
+#include <glib/gstdio.h>
+
 #include "intl.h"
 #include "message.h"
 #include "geometry.h"
@@ -36,6 +38,9 @@
 #define XFIG_RENDERER_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST ((klass), XFIG_TYPE_RENDERER, XfigRendererClass))
 #define XFIG_IS_RENDERER(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), XFIG_TYPE_RENDERER))
 #define XFIG_RENDERER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), XFIG_TYPE_RENDERER, XfigRendererClass))
+#define DTOSTR_BUF_SIZE G_ASCII_DTOSTR_BUF_SIZE
+#define xfig_dtostr(buf,d) \
+	g_ascii_formatd(buf, sizeof(buf), "%f", d)
 
 GType xfig_renderer_get_type (void) G_GNUC_CONST;
 
@@ -74,7 +79,7 @@ struct _XfigRenderer
 /* check whether there exists an arrow head */
 static int hasArrow(Arrow *arrow)
 {
-  return (ARROW_NONE==arrow->type) ? 0 : 1;
+  return (!arrow || ARROW_NONE==arrow->type) ? 0 : 1;
 }
 
 static void begin_render(DiaRenderer *self);
@@ -164,7 +169,7 @@ static void draw_string(DiaRenderer *self,
 static void draw_image(DiaRenderer *self,
 		       Point *point,
 		       real width, real height,
-		       DiaImage image);
+		       DiaImage *image);
 static void draw_object(DiaRenderer *self,
 			DiaObject *object);
 
@@ -203,8 +208,6 @@ xfig_renderer_get_type (void)
 static void
 xfig_renderer_finalize (GObject *object)
 {
-  XfigRenderer *xfig_renderer = XFIG_RENDERER (object);
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -407,7 +410,7 @@ static guchar *
 figText(XfigRenderer *renderer, const guchar *text) 
 {
   int i, j;
-  int len = strlen(text);
+  int len = strlen((char *) text);
   int newlen = len;
   guchar *returntext;
   for (i = 0; i < len; i++) {
@@ -436,6 +439,10 @@ static void
 figArrow(XfigRenderer *renderer, Arrow *arrow, real line_width)
 {
   int type, style;
+  gchar lw_buf[DTOSTR_BUF_SIZE];
+  gchar aw_buf[DTOSTR_BUF_SIZE];
+  gchar al_buf[DTOSTR_BUF_SIZE];
+  
   switch (arrow->type) {
   case ARROW_NONE:
     return;
@@ -451,17 +458,19 @@ figArrow(XfigRenderer *renderer, Arrow *arrow, real line_width)
   case ARROW_FILLED_DIAMOND:
     type = 3; style = 1; break;
   default:
-    message_warning(_("FIG format has no equivalent of arrow style %s, using simple arrow.\n"), arrow_types[arrow->type].name);
+    message_warning(_("FIG format has no equivalent of arrow style %s, using simple arrow.\n"), 
+		    arrow_get_name_from_type(arrow->type));
     /* Notice fallthrough */
   case ARROW_FILLED_CONCAVE:
     type = 2; style = 1; break;
   case ARROW_BLANKED_CONCAVE:
     type = 2; style = 0; break;
   }
-  fprintf(renderer->file, "  %d %d %f %f %f\n",
-	  type, style, figAltCoord(renderer, line_width), 
-	  figCoord(renderer, arrow->width),
-	  figCoord(renderer, arrow->length));	  
+  fprintf(renderer->file, "  %d %d %s %s %s\n",
+	  type, style,
+	  xfig_dtostr(lw_buf, figAltCoord(renderer, line_width)), 
+	  xfig_dtostr(aw_buf, figCoord(renderer, arrow->width)),
+	  xfig_dtostr(al_buf, figCoord(renderer, arrow->length)) );	  
 }
 
 static void 
@@ -556,17 +565,18 @@ draw_line(DiaRenderer *self,
           Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 2\n",
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 2\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
-	  figCapsStyle(renderer));
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), figCapsStyle(renderer) );
   fprintf(renderer->file, "\t%d %d %d %d\n",
 	  (int)figCoord(renderer, start->x), (int)figCoord(renderer, start->y), 
 	  (int)figCoord(renderer, end->x), (int)figCoord(renderer, end->y));
@@ -581,16 +591,18 @@ draw_line_with_arrows(DiaRenderer *self,
 		      Arrow *end_arrow) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %f %d %d 0 %d %d 2\n",
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 %d %d 2\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer),
 	  hasArrow(end_arrow),
 	  hasArrow(start_arrow));
@@ -612,16 +624,18 @@ draw_polyline(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 %d\n",
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer), num_points);
   fprintf(renderer->file, "\t");
   for (i = 0; i < num_points; i++) {
@@ -641,16 +655,18 @@ draw_polyline_with_arrows(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %f %d %d 0 %d %d %d\n",
+  fprintf(renderer->file, "2 1 %d %d %d 0 %d 0 -1 %s %d %d 0 %d %d %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer),
 	  hasArrow(end_arrow),
 	  hasArrow(start_arrow), num_points);
@@ -675,16 +691,18 @@ draw_polygon(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 %d\n",
+  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer), num_points+1);
   fprintf(renderer->file, "\t");
   for (i = 0; i < num_points; i++) {
@@ -702,17 +720,19 @@ fill_polygon(DiaRenderer *self,
 {
   int i;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d 0 %d %d %d 0 20 %f %d %d 0 0 0 %d\n",
+  fprintf(renderer->file, "2 3 %d 0 %d %d %d 0 20 %s %d %d 0 0 0 %d\n",
 	  figLineStyle(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer),
 	  figCapsStyle(renderer), num_points+1);
   fprintf(renderer->file, "\t");
   for (i = 0; i < num_points; i++) {
@@ -729,16 +749,18 @@ draw_rect(DiaRenderer *self,
           Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %f %d %d 0 0 0 5\n",
+  fprintf(renderer->file, "2 3 %d %d %d 0 %d 0 -1 %s %d %d 0 0 0 5\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer));
   fprintf(renderer->file, "\t%d %d %d %d %d %d %d %d %d %d\n",
 	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, ul_corner->y), 
@@ -754,17 +776,19 @@ fill_rect(DiaRenderer *self,
           Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "2 3 %d %d %d %d %d -1 20 %f %d %d 0 0 0 5\n",
+  fprintf(renderer->file, "2 3 %d %d %d %d %d -1 20 %s %d %d 0 0 0 5\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer));
   fprintf(renderer->file, "\t%d %d %d %d %d %d %d %d %d %d\n",
 	  (int)figCoord(renderer, ul_corner->x), (int)figCoord(renderer, ul_corner->y), 
@@ -782,39 +806,63 @@ draw_arc(DiaRenderer *self,
          Color *color) 
 {
   Point first, second, last;
+  int direction = 1; /* Dia always gives counterclockwise */
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar dl_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
+  fprintf (renderer->file, "#draw_arc center=(%g,%g) radius=%g angle1=%g° angle2=%g°\n", center->x, center->y, (width+height)/4.0, angle1, angle2);
+  /* adjust to radians */
+  angle1 *= (M_PI/180.0);
+  angle2 *= (M_PI/180.0);
 
   first = *center;
   first.x += (width/2.0)*cos(angle1);
   first.y -= (height/2.0)*sin(angle1);
 
   second = *center;
-  second.x += (width/2.0)*cos(angle1+(angle2-angle1)/2.0);
-  second.y -= (height/2.0)*sin(angle1+(angle2-angle1)/2.0);
+  second.x += (width/2.0)*cos((angle1+angle2)/2.0);
+  second.y -= (height/2.0)*sin((angle1+angle2)/2.0);
 
   last = *center;
   last.x += (width/2.0)*cos(angle2);
   last.y -= (height/2.0)*sin(angle2);
 
-  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %f %d 1 0 0 %f %f %d %d %d %d %d %d\n",
+  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %s %d %d 0 0 %s %s %d %d %d %d %d %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(dl_buf, figDashLength(renderer)),
 	  figCapsStyle(renderer),
-	  figCoord(renderer, center->x), 
-	  figCoord(renderer, center->y),
+	  direction,
+	  xfig_dtostr(cx_buf, figCoord(renderer, center->x)), 
+	  xfig_dtostr(cy_buf, figCoord(renderer, center->y)),
 	  (int)figCoord(renderer, first.x), (int)figCoord(renderer, first.y), 
 	  (int)figCoord(renderer, second.x), (int)figCoord(renderer, second.y), 
 	  (int)figCoord(renderer, last.x), (int)figCoord(renderer, last.y));
 
 }
 
+/* once more copied from lib/diarenderer.c (see also objects/standard/arc.c */
+static gboolean
+is_right_hand (const Point *a, const Point *b, const Point *c)
+{
+  Point dot1, dot2;
+
+  dot1 = *a;
+  point_sub(&dot1, c);
+  point_normalize(&dot1);
+  dot2 = *b;
+  point_sub(&dot2, c);
+  point_normalize(&dot2);
+  return point_cross(&dot1, &dot2) > 0;
+}
+
 static void 
 draw_arc_with_arrows(DiaRenderer *self, 
 		     Point *startpoint,
@@ -826,48 +874,37 @@ draw_arc_with_arrows(DiaRenderer *self,
 		     Arrow *end_arrow) 
 {
   Point center;
-  Point t1, t2;
-  Point l1, l2;
-  Point p1, p2;
+  int direction = 0;
+  real radius = -1.0;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar dl_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  /* Since the lines between points on the circle form tangents, the
-   * intersection between the perpendiculars to those give the center.
-   */
-  
-  t1 = t2 = *midpoint;
-  point_sub(&t1, startpoint);
-  point_sub(&t2, endpoint);
-  point_scale(&t1, .5);
-  point_scale(&t2, .5);
-  p1 = p2 = *midpoint;
-  point_add(&p1, &t1);
-  point_add(&p2, &t2);
-  /* Now p1 and p2 are the midpoints of the tangents */
-  point_get_perp(&l1, &t1);
-  point_get_perp(&l2, &t2);
-  /* Now p1->t1 and p2->t2 give vectors crossing at the center.
-   * The direction of the vector (in or out) doesn't matter. */
-  
-
   center.x = 0.0;
   center.y = 0.0;
+  direction = is_right_hand (startpoint, midpoint, endpoint) ? 0 : 1;
+  if (!three_point_circle (startpoint, midpoint, endpoint, &center, &radius))
+    message_warning("xfig: arc conversion failed");
 
-  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %f %d 1 %d %d %f %f %d %d %d %d %d %d\n",
+  fprintf (renderer->file, "#draw_arc_with_arrows center=(%g,%g) radius=%g\n", center.x, center.y, radius);
+
+  fprintf(renderer->file, "5 1 %d %d %d %d %d 0 -1 %s %d %d %d %d %s %s %d %d %d %d %d %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(dl_buf, figDashLength(renderer)),
 	  figCapsStyle(renderer),
+	  direction,
 	  hasArrow(end_arrow),
 	  hasArrow(start_arrow),
-	  figCoord(renderer, center.x), 
-	  figCoord(renderer, center.y),
+	  xfig_dtostr(cx_buf, figCoord(renderer, center.x)),
+	  xfig_dtostr(cy_buf, figCoord(renderer, center.y)),
 	  (int)figCoord(renderer, startpoint->x), (int)figCoord(renderer, startpoint->y), 
 	  (int)figCoord(renderer, midpoint->x), (int)figCoord(renderer, midpoint->y), 
 	  (int)figCoord(renderer, endpoint->x), (int)figCoord(renderer, endpoint->y));
@@ -889,32 +926,40 @@ fill_arc(DiaRenderer *self,
 {
   Point first, second, last;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar dl_buf[DTOSTR_BUF_SIZE];
+  gchar cx_buf[DTOSTR_BUF_SIZE];
+  gchar cy_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
+  fprintf (renderer->file, "#fill_arc center=(%g,%g) radius=%g angle1=%g° angle2=%g°\n", center->x, center->y, (width+height)/4.0, angle1, angle2);
+  /* adjust to radians */
+  angle1 *= (M_PI/180.0);
+  angle2 *= (M_PI/180.0);
+
   first = *center;
   first.x += (width/2.0)*cos(angle1);
   first.y -= (height/2.0)*sin(angle1);
 
   second = *center;
-  second.x += (width/2.0)*cos(angle1+(angle2-angle1)/2.0);
-  second.y -= (height/2.0)*sin(angle1+(angle2-angle1)/2.0);
+  second.x += (width/2.0)*cos((angle1+angle2)/2.0);
+  second.y -= (height/2.0)*sin((angle1+angle2)/2.0);
 
   last = *center;
   last.x += (width/2.0)*cos(angle2);
   last.y -= (height/2.0)*sin(angle2);
 
-  fprintf(renderer->file, "5 1 %d %d %d %d %d 20 0 %f %d 1 0 0 %f %f %d %d %d %d %d %d\n",
+  fprintf(renderer->file, "5 2 %d %d %d %d %d 20 0 %s %d 1 0 0 %s %s %d %d %d %d %d %d\n",
 	  figLineStyle(renderer), figLineWidth(renderer), 
 	  figColor(renderer, color), figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(dl_buf, figDashLength(renderer)),
 	  figCapsStyle(renderer),
-	  figCoord(renderer, center->x), 
-	  figCoord(renderer, center->y),
+	  xfig_dtostr(cx_buf, figCoord(renderer, center->x)), 
+	  xfig_dtostr(cy_buf, figCoord(renderer, center->y)),
 	  (int)figCoord(renderer, first.x), (int)figCoord(renderer, first.y), 
 	  (int)figCoord(renderer, second.x), (int)figCoord(renderer, second.y), 
 	  (int)figCoord(renderer, last.x), (int)figCoord(renderer, last.y));
@@ -928,18 +973,19 @@ draw_ellipse(DiaRenderer *self,
              Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "1 1 %d %d %d -1 %d 0 -1 %f 1 0.0 %d %d %d %d 0 0 0 0\n",
+  fprintf(renderer->file, "1 1 %d %d %d -1 %d 0 -1 %s 1 0.0 %d %d %d %d 0 0 0 0\n",
 	  figLineStyle(renderer), 
 	  figLineWidth(renderer), 
 	  figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
 	  (int)figCoord(renderer, center->x), (int)figCoord(renderer, center->y),
 	  (int)figCoord(renderer, width/2), (int)figCoord(renderer, height/2));
 
@@ -952,19 +998,20 @@ fill_ellipse(DiaRenderer *self,
              Color *color) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  fprintf(renderer->file, "1 1 %d %d %d %d %d 0 20 %f 1 0.0 %d %d %d %d 0 0 0 0\n",
+  fprintf(renderer->file, "1 1 %d %d %d %d %d 0 20 %s 1 0.0 %d %d %d %d 0 0 0 0\n",
 	  figLineStyle(renderer), 
 	  figLineWidth(renderer), 
 	  figColor(renderer, color),
 	  figColor(renderer, color),
 	  figDepth(renderer),
-	  figDashLength(renderer),
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
 	  (int)figCoord(renderer, center->x), (int)figCoord(renderer, center->y),
 	  (int)figCoord(renderer, width/2), (int)figCoord(renderer, height/2));
 }
@@ -1028,20 +1075,21 @@ draw_string(DiaRenderer *self,
 {
   guchar *figtext = NULL;
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass) {
     figCheckColor(renderer, color);
     return;
   }
 
-  figtext = figText(renderer, text);
+  figtext = figText(renderer, (unsigned char *) text);
   /* xfig texts are specials */
-  fprintf(renderer->file, "4 %d %d %d 0 %d %f 0.0 6 0.0 0.0 %d %d %s\\001\n",
+  fprintf(renderer->file, "4 %d %d %d 0 %d %s 0.0 6 0.0 0.0 %d %d %s\\001\n",
 	  figAlignment(renderer, alignment),
 	  figColor(renderer, color), 
 	  figDepth(renderer),
 	  figFont(renderer),
-	  figFontSize(renderer),
+	  xfig_dtostr(d_buf, figFontSize(renderer)),
 	  (int)figCoord(renderer, pos->x),
 	  (int)figCoord(renderer, pos->y),
 	  figtext);
@@ -1052,16 +1100,18 @@ static void
 draw_image(DiaRenderer *self,
            Point *point,
            real width, real height,
-           DiaImage image) 
+           DiaImage *image) 
 {
   XfigRenderer *renderer = XFIG_RENDERER(self);
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
   if (renderer->color_pass)
     return;
 
-  fprintf(renderer->file, "2 5 %d 0 -1 0 %d 0 -1 %f %d %d 0 0 0 5\n",
+  fprintf(renderer->file, "2 5 %d 0 -1 0 %d 0 -1 %s %d %d 0 0 0 5\n",
 	  figLineStyle(renderer), figDepth(renderer),
-	  figDashLength(renderer), figJoinStyle(renderer), 
+	  xfig_dtostr(d_buf, figDashLength(renderer)),
+	  figJoinStyle(renderer), 
 	  figCapsStyle(renderer));
   fprintf(renderer->file, "\t0 %s\n", dia_image_filename(image));
   fprintf(renderer->file, "\t%d %d %d %d %d %d %d %d %d %d\n",
@@ -1093,11 +1143,13 @@ export_fig(DiagramData *data, const gchar *filename,
   XfigRenderer *renderer;
   int i;
   Layer *layer;
+  gchar d_buf[DTOSTR_BUF_SIZE];
 
-  file = fopen(filename, "w");
+  file = g_fopen(filename, "w");
 
   if (file == NULL) {
-    message_error(_("Can't open output file %s: %s\n"), filename, strerror(errno));
+    message_error(_("Can't open output file %s: %s\n"), 
+		  dia_message_filename(filename), strerror(errno));
     return;
   }
 
@@ -1110,7 +1162,7 @@ export_fig(DiagramData *data, const gchar *filename,
   fprintf(file, "Center\n");
   fprintf(file, "Metric\n");
   fprintf(file, "%s\n", data->paper.name);
-  fprintf(file, "%f\n", data->paper.scaling*100.0);
+  fprintf(file, "%s\n", xfig_dtostr(d_buf, data->paper.scaling*100.0));
   fprintf(file, "Single\n"); /* Could we do layers this way? */
   fprintf(file, "-2\n");
   fprintf(file, "1200 2\n");
@@ -1121,8 +1173,10 @@ export_fig(DiagramData *data, const gchar *filename,
   
   for (i=0; i<data->layers->len; i++) {
     layer = (Layer *) g_ptr_array_index(data->layers, i);
-    layer_render(layer, DIA_RENDERER(renderer), NULL, NULL, data, 0);
-    renderer->depth++;
+    if (layer->visible) {
+      layer_render(layer, DIA_RENDERER(renderer), NULL, NULL, data, 0);
+      renderer->depth++;
+    }
   }
   
   DIA_RENDERER_GET_CLASS(renderer)->end_render(DIA_RENDERER(renderer));
@@ -1133,8 +1187,10 @@ export_fig(DiagramData *data, const gchar *filename,
   
   for (i=0; i<data->layers->len; i++) {
     layer = (Layer *) g_ptr_array_index(data->layers, i);
-    layer_render(layer, DIA_RENDERER(renderer), NULL, NULL, data, 0);
-    renderer->depth++;
+    if (layer->visible) {
+      layer_render(layer, DIA_RENDERER(renderer), NULL, NULL, data, 0);
+      renderer->depth++;
+    }
   }
   
   DIA_RENDERER_GET_CLASS(renderer)->end_render(DIA_RENDERER(renderer));
